/***
*Agis.cpp - Compute GIS/SPA for elderly
*
*$Id: Agis.cpp 6.112 2008/08/22 17:07:44 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains functions to compute GIS and SPA for elderly.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 07/06/05 | GIS income exemption applied GISEMPEXP and GISEMPEXM    |
* | LP  | 07/02/13 | GIS income for reduction now uses imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 05/12/16 | Added QPIP benefits to income, subtract QPIP premiums   |
* | CH  | 03/10/17 | Deleted iditogv, replaced idiint + idioinv by idiinvnd  |
* | LP  | 02/06/26 | Revised imgisinc calculation                            |
* | LP  | 01/03/07 | Removed SPAEFLAG, SPAME and SPAFE (pre 1988 parameters) |
* | CH  | 00/05/24 | Added iditrrsp                                          |
* | BBM | 99/02/11 | Drop idiroom from gisinc (now in idisenf as of 7.0)     |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CCH | 98/05/22 | Bug. GIS income deduc: added PYINC to deductions        |
* | BBM | 98/05/20 | Bug. GIS income: dividend income --> taxable dividends  |
* | BBM | 98/05/20 | Add Fed 98 Budget changes to GIS income (no deductions) |
* | BBM | 96/05/27 | add seniors benefits optimization with OAS/GIS          |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Agis     [global]  Compute GIS/SPA for elderly
*   gissub   [static]  Compute income used to tax back GIS/SPA
*
***************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== MANIFEST CONSTANTS ================================ */

/* the following codes are placed in nfoldtyp. */
/* They are available for use in gist.       */

#define GISSINGLE   1    /* single, with OAS                        */
#define SPAWIDOW    2    /* widow(er) aged 60-64                    */
#define GISCOUPLE   3    /* couple, both have OAS                   */
#define GISSPA      4    /* couple, elder has OAS, younger is 60-64 */
#define GISYNG      5    /* couple, only younger has OAS            */
#define GISELD      6    /* couple, only elder has OAS              */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Tgis[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== FUNCTION PROTOTYPES =============================== */

static double gissub(P_in in);


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */

/*****************************************************************************
*Agis - Compute GIS/SPA for elderly
*
*Synopsis:
*    void Agis(            
*    P_hh hh              
*    )                    
*
*Description:
*   The global function 'gis' computes GIS/SPA for all relevant persons
*   in 'hh'.
*
*Calls:
*   isnzero  Is argument non-zero ?
*   gissub   Compute income used to tax back GIS/SPA
*   taxbak1  One level tax-back function
*   lkup2    Table look-up without interpolation
*   taxbak2  Two level tax-back function
*
*Structure Members:
*   MP.BESPA        (i)  Basic GIS portion of extended SPA
*   MP.BGISM        (i)  Basic GIS - married
*   MP.BGISS        (i)  Basic GIS - single
*   MP.BOAS         (i)  Basic OAS
*   MP.ESPAT        (i)  Extended SPA take-up rate by benefit level [benefit,rate]
*   MP.ESPATrows    (i)  number of rows in ESPAT table
*   MP.GISBE2       (i)  Breakeven for GIS/SPA couple
*   MP.GISCT        (i)  GIS take-up rate: pensioner couple by benefit level [benefit,rate]
*   MP.GISCTrows    (i)  number of rows in GISCT table
*   MP.GISFLAG      (i)  Federal GIS/SPA/ESPA flag
*   MP.GISOASFLAG   (i)  1984 GIS top-up to OAS residence shortfall flag
*   MP.GISOT        (i)  GIS take-up rate: one pensioner couple by benefit level [benefit,rate]
*   MP.GISOTrows    (i)  number of rows in GISOT table
*   MP.GISRLM       (i)  Basic GIS reduction level: married pensioners
*   MP.GISRLS       (i)  Basic GIS reduction level: single pensioners
*   MP.GISRRM       (i)  Basic GIS reduction rate: married pensioners
*   MP.GISRRS       (i)  Basic GIS reduction rate: single pensioners
*   MP.GISST        (i)  GIS take-up rate: single pensioner by benefit level [benefit,rate]
*   MP.GISSTrows    (i)  number of rows in GISST table
*   MP.GISTURFLAG   (i)  GIS take-up flag: apply 5 take-up tables
*   MP.SPAOASRR     (i)  OAS portion of SPA taxback rate
*   MP.SPARL        (i)  SPA reduction point: one married/ widowed
*   MP.SPAT         (i)  SPA take-up rate by benefit level [benefit,rate]
*   MP.SPATrows     (i)  number of rows in SPAT table
*   MP.SPAXO        (i)  Benefit Cross-over GIS/SPA vs GIS one pensioner couple
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idmarst      (i)  Marital status
*   id.idrand       (i)  Random numbers [array]
*   im.imioas       (i)  OAS benefits
*   im.imoaspar     (i)  Partial OAS residency flag
*   im.imoasres     (i)  Partial OAS fraction
*   nf.nfageeld     (i)  Age of eldest in nuclear family
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imgisinc     (io) Individual's income for GIS/SPA reduction
*   im.imgismax     (o)  Maximum amount of GIS
*   im.imgistyp     (o)  Type of GIS entitlement
*   im.imigis       (o)  GIS benefits
*   im.imispa       (o)  Spouse's allowance
*   im.imoldtyp     (o)  Type of GIS/SPA nuclear family
*   im.imspamax     (o)  Maximum amount of SPA
*   im.imspatyp     (o)  Type of SPA entitlement
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void Agis(
	P_hh hh
)
{
	register P_in ineld;  /* pointer to data of elder   */
	register P_in inspo;  /* pointer to data of younger */
	P_nf nf;

	register int nfi;

	register int ageeld;      /* age of elder   */
	register int agespo;      /* age of younger */
	int reseld;               /* elder   meets residency requirements (1/0) */
	int resspo;               /* younger meets residency requirements (1/0) */
	int resboth;              /* both    meet  residency requirements (1/0) */

	int spoflg;               /* is married couple (1/0) */

	NUMBER nfgisinc = ZERO;
	NUMBER spaoas = ZERO;
	NUMBER gis = ZERO;
	NUMBER gispen = ZERO;     /* partial pensioner's GIS    Benefits       */
	NUMBER be1 = ZERO;        /* breakeven income for 1 Pensioner Couple   */
	NUMBER be2 = ZERO;        /* breakeven income for GIS/SPA Couple       */
	NUMBER spaxo = ZERO;      /* GIS/SPA Couple crossover with 1 Pensioner */

	DEBUG_ON("Agis");

	/*
	** Check to ensure GISFLAG is on and that at least one person in
	** the neuclear family is age 55 or over
	*/

	if (!MP.GISFLAG) {
		DEBUG1("%s GIS for nuclear family turned off by GISFLAG\n");
		DEBUG_OFF("Agis");
		return;
	}

	/* process each nuclear family in household */
	for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

		/*
		** First perform some initial screenings to improve the clarity
		** and efficiency of the following code.
		** Check to ensure that at least one person in
		** the neuclear family is age 55 or over
		*/

		if (nf->nfageeld < 55) {
			DEBUG1("%s eldest is under 55, skipping further calculations\n");
			continue;
		}

		/* initialise spouse presence flag */

		spoflg = nf->nfspoflg;


		/* Assign working pointers.                                       */
		/* Also assign useful variables, such as family income, etc.      */

		if (spoflg) {
			DEBUG1("%s spouse present\n");
			ineld = nf->nfineld;
			inspo = nf->nfinspo;

			ageeld = ineld->id.idage;
			agespo = inspo->id.idage;

			/*
			** OAS Residential Eligibility flags (0/1 = None/Full or partial)
			*/
			reseld = (0 != isnzero(ineld->im.imoasres));
			resspo = (0 != isnzero(inspo->im.imoasres));
			resboth = reseld & resspo;

			/* compute income concept for GIS/SPA reduction */
			ineld->im.imgisinc = (NUMBER) gissub(ineld);
			inspo->im.imgisinc = (NUMBER) gissub(inspo);
			nfgisinc = ineld->im.imgisinc + inspo->im.imgisinc;

		}
		else {
			DEBUG1("%s no spouse present\n");
			ineld = nf->nfineld;
			ageeld = ineld->id.idage;
			reseld = (0 != isnzero(ineld->im.imoasres));

			/* compute income concept for GIS/SPA reduction */

			nfgisinc = (NUMBER) gissub(ineld);
			ineld->im.imgisinc = nfgisinc;

		}
		DEBUG5("%s Family Unit=%d Spouse=%d Eldest=%d Youngest=%d\n",
					nfi, spoflg, ageeld, agespo);

		/**
		* Determine type and amount of GIS/SPA benefit (if any).
		* Note that to get this far in the code, one must be at least 60
		* if single, and in a couple the eldest must be at least 65.
		**/

		if (!spoflg && ageeld >= 60) { /* single elderly (65+) or near-elderly (60-64) */

			/*-------------------------------------------------------------
			** GIS for a Single Pensioner
			**-------------------------------------------------------------
			*/
			if (reseld && ageeld >= 65) {

				DEBUG1("%s normal single GIS\n");

				ineld->im.imoldtyp = GISSINGLE;
				ineld->im.imgistyp = 1;

				gis = MP.BGISS;
				if (MP.GISOASFLAG && ineld->im.imoaspar) {
					DEBUG1("%s partial OAS residency\n");
					DEBUG1("%s increasing GIS to cover OAS shortfall\n");
					gis += MP.BOAS - ineld->im.imioas;
				}

				/**
				* save amount of GIS before reduction and then reduce and
				* calculate gis to maximum gis ratio for applying take up rates
				**/
				ineld->im.imgismax = gis;
				taxbak1(nfgisinc, MP.GISRLS, &gis, MP.GISRRS);

				/**
				* Apply takeup rates for single pensioner and assign gis
				**/
				if (MP.GISTURFLAG && ineld->id.idrand[1] > (NUMBER) lkup2(MP.GISST, MP.GISSTrows, (double) gis)) {
					DEBUG1("%s failed random take-up test\n");
					/* note imigis is already zero */
				}
				else {
					ineld->im.imigis = gis;
				}
				DEBUG3("%s nfgisinc=%.2f gis=%.2f\n", nfgisinc, ineld->im.imigis);
			}

			/*-------------------------------------------------------------
			** Extended SPA for Widow(er)s age 60-64
			**-------------------------------------------------------------
			*/

			else if (reseld && ineld->id.idmarst == WIDOWED) {

				DEBUG1("%s normal widow(er) SPA\n");

				ineld->im.imoldtyp = SPAWIDOW;
				ineld->im.imspatyp = 1;

				/**
				* Assign OAS portion of Extended SPA
				**/
				if (!MP.GISOASFLAG && ineld->im.imoaspar) {
					/**
					* Partial SPA (pre-1984) was changed to full
					* OAS equivalent (independent of immigration)
					* along with the other aspects of GIS-OAS shortfall
					* replacement.
					**/
					DEBUG1("%s partial OAS residency\n");
					spaoas = MP.BOAS * ineld->im.imoasres;
				}
				else {
					spaoas = MP.BOAS;
				}

				/**
				* Assign GIS portion of SPA and
				* save amount of SPA before reduction
				**/
				gis = MP.BESPA;
				ineld->im.imspamax = spaoas + gis;

				/**
				* Reduce maximum spaoas and gis based on income and
				* calculate ratio to maximum benefits for take-up
				**/
				taxbak2(nfgisinc, MP.SPARL, &spaoas, MP.SPAOASRR, &gis, MP.GISRRS);


				/**
				* Apply takeup rates for SPA Widow(er) and assign spa
				**/
				if (MP.GISTURFLAG && ineld->id.idrand[1] > (NUMBER) lkup2(MP.ESPAT, MP.ESPATrows, (double) gis+spaoas)) {
					DEBUG1("%s failed random ESPA take-up test\n");
					continue;
				}
				else {
					ineld->im.imispa = spaoas + gis;
				}
			}
		}


		else if (nf->nfageeld >= 65) { /* spouse present, eldest is 65+ */

			/**-------------------------------------------------------------
			* GIS for Married Couple Both Eligible for part or full OAS
			*-------------------------------------------------------------
			**/
			if (resboth && agespo >= 65) {

				DEBUG1("%s normal GIS couple\n");

				ineld->im.imoldtyp = GISCOUPLE;
				inspo->im.imoldtyp = GISCOUPLE;
				ineld->im.imgistyp = 3;
				inspo->im.imgistyp = 3;

				/** .....................................................
				* Calculate GIS benefits for eldest pensioner in couple
				**/
				gis = MP.BGISM;
				if (MP.GISOASFLAG && ineld->im.imoaspar) {
					DEBUG1("%s elder has partial OAS residency\n");
					DEBUG1("%s increasing GIS to cover OAS shortfall\n");
					gis += MP.BOAS - ineld->im.imioas;
				}
				/**
				* Save amount of GIS before reduction
				**/
				ineld->im.imgismax = gis;

				/**
				* Reduce maximum gis based on income.
				**/
				taxbak1(nfgisinc, MP.GISRLM, &gis, MP.GISRRM);

				/**
				* Apply takeup rates for nuclear family and assign gis
				**/
				if (MP.GISTURFLAG && ineld->id.idrand[1] > (NUMBER) lkup2(MP.GISCT, MP.GISCTrows, (double) gis)) {
					DEBUG1("%s failed random take-up test\n");
					continue;
				}
				else {
					ineld->im.imigis = gis;
				}


				/** .............................................
				* Calculate GIS for Younger pensioner in couple
				**/
				gis = MP.BGISM;
				if (MP.GISOASFLAG && inspo->im.imoaspar) {
					DEBUG1("%s spouse has partial OAS residency\n");
					DEBUG1("%s increasing GIS to cover OAS shortfall\n");
					gis += MP.BOAS - inspo->im.imioas;
				}
				/**
				* Save amount of GIS before reduction
				**/
				inspo->im.imgismax = gis;

				/**
				* Reduce maximum spaoas and gis based on income
				**/
				taxbak1(nfgisinc, MP.GISRLM, &gis, MP.GISRRM);

				/**
				* Note: takeup rates for married pensioner couple are
				* associated with elder's benefits (above).
				**/

				inspo->im.imigis = gis;
			}

			else if (resboth && agespo >= 60) {
			/*-------------------------------------------------------------
			* GIS for Married Couple: One Eligible for OAS, One for SPA
			**-------------------------------------------------------------
			*/

				DEBUG1("%s normal GIS/SPA couple\n");

				ineld->im.imoldtyp = GISSPA;
				inspo->im.imoldtyp = GISSPA;
				ineld->im.imgistyp = 4;
				inspo->im.imspatyp = 2;

				DEBUG4("%s Breakevens gis=%.2f gisspa=%.2f crossover=%.2f\n",
							MP.GISBE1, MP.GISBE2, MP.SPAXO);

				/** ......................................................
				* Calculate SPA for spouse age 60-64
				**/
				DEBUG1("%s do SPA recipient\n");

				/**
				* Calculate OAS equivalent portion of SPA and GIS,
				* (OAS is Maximum regardless of Immigration since 1984)
				* and save Maximum SPA before reduction
				**/

				spaoas = MP.BOAS;
				gis = MP.BGISM;
				inspo->im.imspamax = spaoas + gis;

				/**
				* Reduce Benefits of SPA recipient (two tier)
				**/
				taxbak2(nfgisinc, MP.SPARL, &spaoas, MP.SPAOASRR, &gis, MP.GISRRM);

				/**
				* Apply takeup rates for SPA spouse and assign SPA
				**/
				if (MP.GISTURFLAG && inspo->id.idrand[1] > (NUMBER) lkup2(MP.SPAT, MP.SPATrows, (double) gis+spaoas)) {
					DEBUG1("%s failed random take-up test\n");
					continue;
				}
				else {
					inspo->im.imispa = spaoas + gis;
				}


				/** ......................................................
				* Calculate GIS for Pensioner over age 64
				**/
				DEBUG1("%s do GIS recipient\n");

				/**
				* Calculate Maximum GIS including partial OAS Top-up
				* and save this amount before reduction
				**/
				gis = MP.BGISM;
				if (MP.GISOASFLAG && ineld->im.imoaspar) {
					DEBUG1("%s elder has partial OAS residency\n");
					DEBUG1("%s increasing GIS to cover OAS shortfall\n");
					gis += MP.BOAS - ineld->im.imioas;
				}
				ineld->im.imgismax = gis;

				/**
				* Split the GIS into pre/post crossover portions
				**/
				gispen = (MP.GISBE2-MP.SPAXO) * MP.GISRRM;
				gis = gis-gispen;

				/**
				* Reduce and assign GIS 
				**/
				taxbak1(nfgisinc, MP.SPARL+MP.BOAS/MP.SPAOASRR, &gis, MP.GISRRM);
				taxbak1(nfgisinc, MP.GISBE2, &gispen, MP.GISRRM);

				ineld->im.imigis = gis+gispen;
			}

			/*-------------------------------------------------------------
			** GIS for Married Couple: Single Pensioner, other ineligible
			**-------------------------------------------------------------
			*/

			/** ......................................................
			* Calculate GIS for Pensioner if youngest spouse
			**/

			else if (resspo && agespo >= 65) {

				DEBUG1("%s younger eligible for GIS, older not eligible\n");

				/**
				* Calculate Maximum GIS including partial OAS Top-up
				**/
				ineld->im.imoldtyp = GISYNG;
				inspo->im.imoldtyp = GISYNG;
				inspo->im.imgistyp = 2;
				gis = MP.BGISS;
				if (MP.GISOASFLAG && inspo->im.imoaspar) {
					DEBUG1("%s partial OAS residency\n");
					DEBUG1("%s increasing GIS to cover OAS shortfall\n");
					gis += MP.BOAS - inspo->im.imioas;
				}

				/**
				* Save the amount of GIS before reduction
				**/
				inspo->im.imgismax = gis;

				/**
				* Reduce maximum gis based on income and assign
				**/
				taxbak1(nfgisinc, MP.BOAS + MP.GISRLM, &gis, MP.GISRRM);

				/**
				* Apply takeup rates for single pensioner and assign gis
				**/
				if (MP.GISTURFLAG && inspo->id.idrand[1] > (NUMBER) lkup2(MP.GISOT, MP.GISOTrows, (double) gis)) {
					DEBUG1("%s failed random take-up test\n");
					continue;
				}
				else {
					inspo->im.imigis = gis;
				}
			}

			/** ......................................................
			* Calculate GIS for Pensioner if oldest spouse
			**/
			else if (reseld) {

				DEBUG1("%s older eligible for GIS, younger ineligible for GIS/SPA\n");

				ineld->im.imoldtyp = GISELD;
				inspo->im.imoldtyp = GISELD;
				ineld->im.imgistyp = 2;

				/**
				* Calculate Maximum GIS including partial OAS Top-up
				**/
				gis = MP.BGISS;
				if (MP.GISOASFLAG && ineld->im.imoaspar) {
					DEBUG1("%s partial OAS residency\n");
					DEBUG1("%s increasing GIS to cover OAS shortfall\n");
					gis += MP.BOAS - ineld->im.imioas;
				}

				/**
				* Save the amount of GIS before reduction
				**/
				ineld->im.imgismax = gis;

				/**
				* Reduce maximum gis based on income and assign
				**/
				taxbak1(nfgisinc, MP.BOAS + MP.GISRLM, &gis, MP.GISRRM);

				/**
				* Apply takeup rates for single pensioner and assign gis
				**/
				if (MP.GISTURFLAG && ineld->id.idrand[1] > (NUMBER) lkup2(MP.GISOT, MP.GISOTrows, (double) gis)) {
					DEBUG1("%s failed random take-up test\n");
					continue;
				}
				else {
					ineld->im.imigis = gis;
				}
			}
		}
	}


	DEBUG_OFF("Agis");
}


/*****************************************************************************
*gissub - Compute income used to tax back GIS/SPA
*
*Synopsis:
*    static double gissub(  
*    P_in in              
*    )                    
*
*Description:
*   The static function 'gissub' computes income used to reduce GIS/SPA
*   for the individual 'in'.
*
*Return Value:
*   Returns a 'double' value which is the income for GIS tax back purposes
*   of the individual 'in' .
*
*Calls:
*   nneg     Change negative numbers to zero
*   minn     Find the minimum of two numbers
*
*Structure Members:
*   MP.GISEMPEXM    (i)  GIS employment income exemption - maximum
*   MP.GISEMPEXP    (i)  GIS employment income exemption - percentage of earnings
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   id.idalexp      (i)  Other allowable employment expenses (229)
*   id.idclergy     (i)  Clergy residential deduction
*   id.idicqp       (i)  CPP/QPP income (114)
*   id.idiemp       (i)  Wages & salaries
*   id.idiinvnd     (i)  Interest and other investment income
*   id.idise        (i)  Self-employed income
*   id.iditoth      (i)  Other taxable income (130)
*   id.iditrrsp     (i)  Taxable RRSP withdrawals
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedft      (i)  Deductions from total income
*   im.imicapgt     (i)  Capital gains (taxable)
*   im.imidivt      (i)  Dividend income (taxable)
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imiuib       (i)  Unemployment Insurance/Employment Insurance benefits
*   im.impentrd     (i)  Pension income deduction from transfer to spouse
*   im.imqpipcbn    (i)  Quebec parental insurance plan calendar year benefits
*   im.imqpipp      (i)  Quebec parental insurance plan premiums for paid workers
*   im.imuic        (i)  UIC contributions
*
*Called by:
*   Agis     Compute GIS/SPA for elderly
*
*****************************************************************************/
static double gissub(
	register P_in in
)
{
	NUMBER temp = ZERO;

	NUMBER empinc = ZERO;
	NUMBER neteinc = ZERO;
	NUMBER othded = ZERO;

	 
	empinc = nneg(in->id.idiemp - in->id.idalexp - in->id.idclergy);
	neteinc = empinc - minn((empinc * MP.GISEMPEXP), MP.GISEMPEXM);
	othded = nneg(in->im.imdedft - in->id.idalexp - in->id.idclergy - in->im.impentrd);

	temp = nneg(MP.PYINC * (in->im.imicapgt
		               + in->im.imidivt
		               + in->id.idiinvnd
		               + neteinc
		               + in->id.idise
		               + in->id.iditoth
		               + in->id.idicqp
		               + in->im.imipnst
                       + in->id.iditrrsp
		               + in->im.imiuib
					   + in->im.imqpipcbn
					   - in->im.imcqppc
					   - in->im.imuic
					   - in->im.imqpipp
					   - othded
	               ));

 
	return(temp);

}
