/***
*Atxhhexp.cpp - Compute and pro-rate household taxes, rent, etc.
*
*$Id: Atxhhexp.cpp 6.101 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/02/19 | Replace CTFTOT and CTPTOT by individual taxes           |
* | LP  | 05/08/19 | Allow Quebec to use SHS prop. tax under IMSHTOPT = 1    |
* | EGA | 98/06/22 | CTFAMSNA by province                                    |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 92/10/26 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxhhexp [global]  Compute and pro-rate household taxes, rent, etc.
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */
#include "apu.h"    /* Database adjustment parameter (user's) declarations  */
#include "ap.h"     /* Database adjustment parameter declarations           */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxhhexp[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxhhexp - Compute and pro-rate household taxes, rent, etc.
*
*Synopsis:
*    void Atxhhexp(        
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   isnzero  Is argument non-zero ?
*
*Structure Members:
*   AP.CTFAMSNA     (i)  SHS->SNA conceptual conversion factor [com]
*   MP.CTFCID       (i)  Federal custom import duties [com]
*   MP.CTFEXD       (i)  Federal excise duties [com]
*   MP.CTFEXT       (i)  Federal excise taxes [com]
*   MP.CTFGST       (i)  Federal GST [com]
*   MP.CTPGAS       (i)  Provincial gasoline tax [com x prov]
*   MP.CTPRST       (i)  Provincial retail sales tax [com x prov]
*   MP.IMSHTOPT     (i)  Paid rent and property tax imputation option
*   fx.fxio         (i)  I/O expenditure categories [array]
*   fx.fxprtax      (i)  Property tax
*   fx.fxtptax      (i)  Transfer of Property taxes
*   hd.hdprov       (i)  Province
*   hh.fx           (i)  SHS data [struct]
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idcfrh       (i)  Relationship to census family head
*   id.idhhrh       (i)  Relationship to head of household
*   id.idproptx     (i)  Net property taxes paid (556)
*   id.idrentpd     (i)  Total rental payments (555)
*   im.improptx     (io) Imputed property tax paid
*   im.imrentpd     (io) Imputed rent paid
*
*Called by:
*   gist     Compute Provincial GIS top-ups for elderly
*   txbc     Compute provincial taxes for British Columbia
*   txman    Compute provincial taxes for Manitoba
*   txont    Compute provincial taxes for Ontario
*   txqcalc  Calculate federal income tax
*
*****************************************************************************/
void Atxhhexp(
	P_hh hh
)
{
	register P_in in;
	register P_fx fx;
	register int ini;
	int prov;
	int impall;              /* total inds in hhld to recieve imputation */
	int impone;              /* total inds in hhld to recieve imputation */
	NUMBER hhrentpd = ZERO;  /* hhld level gross paid rents */
	NUMBER hhproptx = ZERO;  /* hhld level property taxes */

	DEBUG_ON("Atxhhexp");

	if (MP.IMSHTOPT==1) {

		prov = hh->hd.hdprov;
		fx = &hh->fx;

		/* calculate the household level paid rents 
		*  note: perform conceptual adjustment to convert FAMEX-based 
		*        to SNA-based NET expenditure.
		*/
		hhrentpd = (ONE + MP.CTFCID[IO_GPRENT] + MP.CTFEXD[IO_GPRENT]
                        + MP.CTFEXT[IO_GPRENT] + MP.CTFGST[IO_GPRENT]
                        + MP.CTPGAS[IO_GPRENT][prov] + MP.CTPRST[IO_GPRENT][prov])
					* (AP.CTFAMSNA[IO_GPRENT][prov] * fx->fxio[IO_GPRENT]);

		/* calculate the household level property taxes */
		hhproptx = fx->fxprtax + fx->fxtptax;

		DEBUG3("%s hhexp: hhrentpd=%.2f hhproptx=%.2f\n",
	   		hhrentpd, hhproptx);

		/* flag the individuals in the hhld to which the rent and
		*  property tax will be imputed. these are head's of census families
		*  and census family spouses reporting positive net income
		*/
		impall = 0;
		impone = 0;
		
		DEBUG1("%s txhhexp: get paid rent and prop tax from greenbook\n");
		for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
			impone = 0;
			switch (hh->hd.hdprov) {
				case PRQUE:
					if (in->id.idcfrh==0) impone = 1;
					break;
				default:
			 		in->im.imrentpd = in->id.idrentpd;
			 		in->im.improptx = in->id.idproptx;
					break;
			 }

			 DEBUG6("%s hhexp person %d: idrentpd=%.0f imrentpd=%.0f idproptx=%.0f improptx=%.0f\n",
			      	ini, in->id.idrentpd, in->im.imrentpd, in->id.idproptx, in->im.improptx);

			if (impone) {
				in->im.imrentpd = hhrentpd;
				in->im.improptx = hhproptx;
				impall++;
			}
		}

		/* distribute the paid rents and property taxes amongst  
		*  those flagged, if necessary.
		*/

		if (impall > 1) {
			for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
            if (isnzero(in->im.imrentpd)) {
   				in->im.imrentpd /= (NUMBER)impall;
            }
            if (isnzero(in->im.improptx)) {
	   			in->im.improptx /= (NUMBER)impall;
            }
				DEBUG5("%s hhexp person %d: cfrh=%d idrentpd=%.2f imrentpd=%.2f\n",
		   			ini, in->id.idcfrh, in->id.idrentpd, in->im.imrentpd);
				DEBUG5("%s hhexp person %d: cfrh=%d idproptx=%.2f improptx=%.2f\n",
		   			ini, in->id.idcfrh, in->id.idproptx, in->im.improptx);
			}
		}
	}

	if (MP.IMSHTOPT==2) {
		DEBUG1("%s txhhexp: get paid rent and property tax from famex\n");

		prov = hh->hd.hdprov;
		fx = &hh->fx;

		/* calculate the household level paid rents 
		*  note: perform conceptual adjustment to convert FAMEX-based 
		*        to SNA-based NET expenditure.
		*/
		hhrentpd = (ONE + MP.CTFCID[IO_GPRENT] + MP.CTFEXD[IO_GPRENT]
                        + MP.CTFEXT[IO_GPRENT] + MP.CTFGST[IO_GPRENT]
                        + MP.CTPGAS[IO_GPRENT][prov] + MP.CTPRST[IO_GPRENT][prov])
					* (AP.CTFAMSNA[IO_GPRENT][prov] * fx->fxio[IO_GPRENT]);

		/* calculate the household level property taxes */
		hhproptx = fx->fxprtax + fx->fxtptax;

		DEBUG3("%s hhexp: hhrentpd=%.2f hhproptx=%.2f\n",
	   		hhrentpd, hhproptx);

		/* flag the individuals in the hhld to which the rent and
		*  property tax will be imputed. these are head's of census families
		*  and census family spouses reporting positive net income
		*/
		impall = 0;
		impone = 0;
		for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
			impone = 0;
			switch (hh->hd.hdprov) {
				case PRQUE:
					if (in->id.idcfrh==0) impone = 1;
					break;
				case PRONT:
					if (in->id.idcfrh==0) impone = 1;
					break;
				case PRMAN:
					if (in->id.idhhrh == 0) impone = 1;
					break;
				case PRBC:
					if (in->id.idhhrh == 0) impone = 1;
					break;
				default:
					if (in->id.idhhrh == 0) impone = 1;
					break;
			}

			if (impone) {
				in->im.imrentpd = hhrentpd;
				in->im.improptx = hhproptx;
				impall++;
			}
			DEBUG2("%s hhexp: impall=%d\n", impall);
			DEBUG5("%s hhexp person %d: cfrh=%d idrentpd=%.2f imrentpd=%.2f\n",
						ini, in->id.idcfrh, in->id.idrentpd, in->im.imrentpd);
			DEBUG5("%s hhexp person %d: cfrh=%d idproptx=%.2f improptx=%.2f\n",
		 				ini, in->id.idcfrh, in->id.idproptx, in->im.improptx);
		}				


		/* distribute the paid rents and property taxes amongst  
		*  those flagged, if necessary.
		*/
		if (impall > 1) {
			for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
            if (isnzero(in->im.imrentpd)) {
   				in->im.imrentpd /= (NUMBER)impall;
            }
            if (isnzero(in->im.improptx)) {
	   			in->im.improptx /= (NUMBER)impall;
            }
				DEBUG5("%s hhexp person %d: cfrh=%d idrentpd=%.2f imrentpd=%.2f\n",
		   			ini, in->id.idcfrh, in->id.idrentpd, in->im.imrentpd);
				DEBUG5("%s hhexp person %d: cfrh=%d idproptx=%.2f improptx=%.2f\n",
		   			ini, in->id.idcfrh, in->id.idproptx, in->im.improptx);
			}
		}
	}

	DEBUG_OFF("Atxhhexp");
}
