/***
*Atxitax.cpp - Compute taxable income and individual credits
*
*$Id: Atxitax.cpp 6.126 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 08/06/11 | Added caregivers option test for CGTCOPT 1 and 2        |
* | LP  | 08/04/15 | Removed idcgless, no longer used or downloaded          |
* | LP  | 07/02/11 | Fed. pension credit now based on imipnst, not idipens   |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/18 | Removed imuccb from credit tests - uses actual net inc  |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* |     |          | and added imisa imbcss imigis imispa idiworkc to imdedfn|
* | LP  | 06/06/16 | Add imbcss to net income for income tests               |
* | LP  | 06/06/13 | Add nneg to income tests that are reduced by imuccb     |
* | LP  | 06/05/30 | Subtract imuccb from ininet for age tax credit test     |
* | LP  | 06/05/11 | Added the Canada Employment Credit EMPTXC               |
* | LP  | 06/05/05 | Added the text book tax credit TEXTPMFT TEXTPMPT        |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | CH  | 05/09/12 | Deleted iddeppch                                        |
* | LP  | 05/04/15 | Added BXMEXT to personal amount calculation             |
* | LP  | 04/02/18 | Updated net income concepts for tax credit income tests |
* | LP  | 03/12/19 | Added CHATPROP - gifts of depreciable property flag     |
* | LP  | 03/12/18 | Added ODISTC - infirm dependents (18+) tax credit       |
* | LP  | 03/11/18 | Added CGTCOPT - caregivers tax credit calculation option|
* | LP  | 03/11/17 | Added imintsl - interest on student loans tax credit    |
* | CH  | 03/10/17 | Deleted foravg, replaced ideducm by ideducfm            |
* | LP  | 01/06/14 | Implemented idcgless for allowable charitable donations |
* | LP  | 01/03/12 | Removed pre-1988 programs (PEROPT, etc.)                |
* | CH  | 00/06/14 | Added imothpe                                           |
* | CH  | 00/04/14 | Apply CAPGIR to capgex stkded and iloss                 |
* | CH  | 99/04/15 | Supplement is now calculated in txhstr                  |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/04/23 | Ensure that imcapgex <= imicapgt                        |
* | CH  | 98/04/02 | Added FORAVGFLAG                                        |
* | BBM | 98/03/03 | Implement supplement to basic exemption BXM (budget 98) |
* | RSE | 94/02/28 | Changed age amount according to 1994 budget             |
* | GJC | 93/04/01 | remove MP.CAPGEFALR and MP.CAPGAL                       |
* | GJC | 93/02/17 | implement the CAPGEFALR adjustment to cap gains exp     |
* | GJC | 93/02/15 | apply missing greenbook imputation vars to calculations |
* | BBM | 92/06/15 | base calculation of charitable credits on expenses      |
* | BBM | 92/04/21 | base calculation of medical credits on actual expenses  |
* | SFG | 91/12/09 | move social programs repayment from txitax to txinet    |
* | SFG | 91/11/25 | implement REPNETFLAG (clawbacks reduce net income flag) |
* | SFG | 91/11/25 | Reduce net income by UI repayment before FA&OAS clawback|
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxitax  [global]  Compute taxable income
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */
#include "apu.h"    /* Database adjustment parameter (user's) declarations  */
#include "ap.h"     /* Database adjustment parameter declarations           */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxitax[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxitax - Compute taxable income
*
*Synopsis:
*    void Atxitax(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   minn     Find the minimum of two numbers
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*
*Structure Members:
*   MP.AXM          (i)  Age amount
*   MP.AXPI         (i)  Age amount phase in rate for 1994 and beyond
*   MP.AXRR         (i)  Age amount credit reduction rate
*   MP.AXTD         (i)  Age amount net income turndown
*   MP.BXM          (i)  Basic personal exemption/amount
*   MP.BXMEXT       (i)  Basic personal amount extra top-up
*   MP.CAPGIR       (i)  Capital gains inclusion rate
*   MP.CGTC         (i)  Caregiver Tax Credit
*   MP.CGTCFLG      (i)  Caregiver Tax Credit activation flag
*   MP.CGTCOPT      (i)  Caregiver Tax Credit option (1=max, 2=greenbook, 3=model)
*   MP.CHATL1       (i)  Charitable donations amount level 1
*   MP.CHATNF       (i)  Charitable donations maximum net income fraction
*   MP.CHATR1       (i)  Charitable donations tax credit rate 1
*   MP.CHATR2       (i)  Charitable donations tax credit rate 2
*   MP.EDDISPT      (i)  Part time students with disabilities flag
*   MP.EDXPM        (i)  Education amount per month of full-time studies
*   MP.EDXPMPT      (i)  Education amount per month of part-time studies
*   MP.EMPTXC       (i)  Canada Employment Credit
*   MP.FNTCR        (i)  Federal non-refundable tax credit rate
*   MP.INTSLFLG     (i)  Interest on Student Loans Tax Credit Activation Flag
*   MP.MAXDX        (i)  Maximum disability deduction/amount
*   MP.MEDALL       (i)  Medical allowance maximum lower limit
*   MP.MEDANF       (i)  Medical allowance lower limit net income fraction
*   MP.ODISTC       (i)  Maximum infirm dependent 18 or older tax credit
*   MP.REPNETFLAG   (i)  Social program repayments reduce net income flag
*   MP.TEXTPMFT     (i)  Textbook amount per month of full-time studies
*   MP.TEXTPMPT     (i)  Textbook amount per month of part-time studies
*   MP.YPNDL        (i)  Maximum pension income deduction/amount
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idaddded     (i)  Additional deductions from net income (256)
*   id.idage        (i)  Age
*   id.idcapgex     (i)  Capital gains exemptions (254)
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.idcharit     (i)  Charitable donations
*   id.idcloss      (i)  Allowable other years capital loss (253)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.idemplo      (i)  Employee home relocation loan dedn (248)
*   id.idgifts      (i)  Gifts to Canada/provinces/culture (342)
*   id.idiemp       (i)  Wages & salaries
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idnclos      (i)  Allowable other years non-capital loss (252)
*   id.idnorth      (i)  Northern deductions (255)
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idpartlo     (i)  Limited partnership losses (251)
*   id.idstkded     (i)  Stock option deduction (249)
*   im.imbcss       (i)  British Columbia Seniors Supplement
*   im.imicapgt     (i)  Capital gains (taxable)
*   im.imicnet      (i)  Net Income (line 236)
*   im.imigis       (i)  GIS benefits
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imispa       (i)  Spouse's allowance
*   im.imrepay      (i)  Social Benefits Repayments
*   im.imcapgex     (io) Modelled capital gains deduction (254)
*   im.imchara      (io) Allowable charitable donations and gifts (calculated)
*   im.imdedfn      (io) All deductions from net income
*   im.immeda       (io) Medical expenses allowed (computed)
*   im.imstkded     (io) Modelled Stock option deduction (249)
*   im.imatxc       (o)  Age tax credit
*   im.imbtc        (o)  Basic personal tax credit
*   im.imcgtc       (o)  Caregiver Tax Credit Level
*   im.imchartc     (o)  Charitable donations tax credit
*   im.imdisatc     (o)  Disability tax credit
*   im.imedtxc      (o)  Education allowance tax credit
*   im.imemptxc     (o)  Canada Employment Credit
*   im.imintsl      (o)  Federal interest on student loans tax credit
*   im.imitax       (o)  Taxable income
*   im.immedatc     (o)  Medical expenses allowed tax credit
*   im.imothpe      (o)  Other dependant tax credit (306)
*   im.impentxc     (o)  Pension income tax credit
*   im.imtxttxc     (o)  Federal textbook tax credit
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void Atxitax(
	P_hh hh
)
{
	register P_in in;
	register int ini;
	NUMBER netinc = ZERO;
	NUMBER medt = ZERO;
	NUMBER agered = ZERO;
	NUMBER adjamt = ZERO;

	DEBUG_ON("Atxitax");

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
		         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);
	
		/* calculate the net income concept to be used for income tests on credits */
		/*  The net income should equal tax form net income therefore we take imicnet */

		netinc = in->im.imicnet;

		/* calculate personal tax credit */
		
		in->im.imbtc = MP.FNTCR * (MP.BXM + MP.BXMEXT);
   		DEBUG2("%s btc    =%.2f\n", in->im.imbtc);

		/* calculate age tax credit */

		if (in->id.idage >= 65) {
			DEBUG1("%s age tax credit\n");
			agered = MP.AXPI * minn(MP.AXM, MP.AXRR * nneg(netinc - MP.AXTD));
			adjamt = MP.AXM - agered;
			in->im.imatxc = MP.FNTCR * adjamt;
		}
   		DEBUG2("%s atxc   =%.2f\n", in->im.imatxc);

		/* calculate the Canada Employment Credit */
		/* eligible for the lesser of employment income and EMPTXC * FNTCR */

		if (isnzero(in->id.idiemp)) {
			in->im.imemptxc = MP.FNTCR * minn(MP.EMPTXC, in->id.idiemp);
			DEBUG2("%s emptxc =%.2f\n", in->im.imemptxc);
		} 

		/* calculate amount for infirm dependents age 18 or older */
		/*  if the filer has an amount for idothpe, then give max credit */

		if (isnzero(in->id.idothpe)) {
   		    in->im.imothpe = MP.FNTCR * MP.ODISTC;
            DEBUG2("%s othpe =%.2f\n", in->im.imothpe);
		}

		/* calculate deductions from net income   */

		/* calculate pension income tax credit    */
		in->im.impentxc = MP.FNTCR * minn(MP.YPNDL, in->im.imipnst);
		DEBUG2("%s pentxc =%.2f\n", in->im.impentxc);

		/* compute allowable medical expenses using the value on the database */
		if (isnzero(in->id.idmedgro)) {
			in->im.immeda = nneg(in->id.idmedgro -
				minn(MP.MEDALL, MP.MEDANF * netinc));
			DEBUG3("%s idmedgro= %.2f immeda =%.2f\n",
				in->id.idmedgro, in->im.immeda);
		}

		/* compute allowable charitable expenses using the database value */
		/* Charitable donations are subject to a ceiling of CHATNF % of net income */
		if (isnzero(in->id.idcharit)) {
			in->im.imchara = minn(in->id.idcharit, (MP.CHATNF * netinc));
		}
		if (isnzero(in->id.idgifts)) {
			in->im.imchara += in->id.idgifts;
			DEBUG4("%s idcharit= %.2f idgifts= %.2f imchara =%.2f\n",
				in->id.idcharit, in->id.idgifts, in->im.imchara);
		}
		DEBUG5("%s idcharit= %.2f idgifts= %.2f imicnet= %.2f imchara =%.2f\n",
				in->id.idcharit, in->id.idgifts, in->im.imicnet, in->im.imchara);

		/* calculate medical expenses tax credit */
		if (isnzero(in->im.immeda)) {
			in->im.immedatc = in->im.immeda * MP.FNTCR;
			DEBUG2("%s medatc =%.2f\n", in->im.immedatc);
		}
		/* calculate charitable donations tax credit */
		if (isnzero(in->im.imchara)) {
			in->im.imchartc = minn(MP.CHATL1, in->im.imchara)  * MP.CHATR1
			            + nneg(in->im.imchara - MP.CHATL1) * MP.CHATR2;
			DEBUG2("%s chartc =%.2f\n", in->im.imchartc);
		}

 		/* set disability tax credit for self to maximum    */
 		if (isnzero(in->id.iddisslf)) {
 			in->im.imdisatc = MP.FNTCR * MP.MAXDX;
 		}
 		DEBUG2("%s disatc =%.2f\n", in->im.imdisatc);
 
 		/* set disability tax credit for other to maximum    */
 		if (isnzero(in->id.iddisoth)) {
 			in->im.imdisatc += (MP.FNTCR * MP.MAXDX);
 		}
 		DEBUG2("%s disatc =%.2f\n", in->im.imdisatc);


		/* calculate caregivers tax credit */
		if (MP.CGTCFLG && isnzero(in->id.idcgtc) && MP.CGTCOPT < 3) {
		    /* if CGTCOPT == 1 then */
			/* apply the maximum amount if they have any caregivers amount */
		    if (MP.CGTCOPT == 1) {
				in->im.imcgtc = MP.CGTC * MP.FNTCR;
			}
			else {   /* use the greenbook imputed amount */
			 	in->im.imcgtc = in->id.idcgtc * MP.FNTCR;
			}
			DEBUG2("%s cgtc =%.2f\n", in->im.imcgtc);
		}

		/* calculate education allowance and text book tax credits */
		if (isnzero(in->id.ideducfm)) {
			in->im.imedtxc = MP.FNTCR * (in->id.ideducfm * MP.EDXPM);
			DEBUG2("%s edtxc  =%.2f\n", in->im.imedtxc);
			in->im.imtxttxc = MP.FNTCR * (in->id.ideducfm * MP.TEXTPMFT);
			DEBUG2("%s txttxc =%.2f\n", in->im.imtxttxc);
		}
    	
    	/*  add in the tax credit amount associated with part-time studies */
   	    if (isnzero(in->id.ideducpm)) {
			if (MP.EDDISPT && isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
			    /* give only part time students the full time credit
				   if they also have an amount for disability */
				in->im.imedtxc = MP.FNTCR * (in->id.ideducpm * MP.EDXPM);
				/*  calculate the text book tax credit */
				in->im.imtxttxc = MP.FNTCR * (in->id.ideducpm * MP.TEXTPMFT);
			}
			else {  /* has some part and/or full time and not disabled */
				in->im.imedtxc += MP.FNTCR * (in->id.ideducpm * MP.EDXPMPT);
				in->im.imtxttxc += MP.FNTCR * (in->id.ideducpm * MP.TEXTPMPT);
			}
            DEBUG2("%s edtxc  =%.2f\n", in->im.imedtxc);
		}

		/* calculate the interest paid on student loans tax credit */
		/*  new for version 10.0, apply the Greenbook value idintstu */
		if (MP.INTSLFLG ) {
			if (isnzero(in->id.idintstu)) {
				in->im.imintsl = MP.FNTCR * in->id.idintstu;
		   		DEBUG2("%s intsl =%.2f\n", in->im.imintsl);
		    }
		}
       
        /* apply the reciprocal of the capital gains inclusion rate to 
           stock option deduction */
        in->im.imstkded = in->id.idstkded * (ONE - MP.CAPGIR);
		/* calculate deductions from net income */
		/* note: items that are replaced by tax credits are zero */
		in->im.imdedfn = in->id.idemplo   /* home relocation loan dedn (248) */
                    + in->im.imstkded  /* Stock option deduction (249) */
					+ in->id.idiworkc  /* Workers compensation (147 and 250) */
					+ in->im.imisa     /* Social Assistance (147 and 250) */
					+ in->im.imbcss    /* BC senior supplement (SA) (147 and 250) */
					+ in->im.imigis    /* Guaranteed income supplement (147 and 250) */
					+ in->im.imispa    /* Spouses allowance (147 and 250)  */
                    + in->id.idpartlo  /* limited partnership losses (251) */
                    + in->id.idnclos   /* allowable prior yrs non-capital loss (252) */
                    + in->id.idcloss   /* allowable other yrs capital loss (253)    */
					                   /* capital gains exempt (254) handled below */
					+ in->id.idnorth   /* Northern deductions (255) */
				    + in->id.idaddded  /* additional deductions (256) */
                    ;
		if (!MP.REPNETFLAG) {
			// Add in the social program repayments (calculated previously
			// in txinet).
			in->im.imdedfn += in->im.imrepay;
		}
		/* add in the lifetime capital gains exemption */
		if (isnzero(in->im.imicapgt)) {
			in->im.imcapgex = in->id.idcapgex * MP.CAPGIR;
			in->im.imdedfn += in->im.imcapgex;
		}
  		DEBUG2("%s dedfn  =%.2f\n", in->im.imdedfn);

		/* calculate person's taxable income */
		/* add the forward averaging amount to net income here */
		in->im.imitax = nneg(in->im.imicnet - in->im.imdedfn);
   		DEBUG2("%s itax   =%.2f\n", in->im.imitax);

	}

	DEBUG_OFF("Atxitax");
}
