/***
*cceopt.cpp - Multiple calls of tax/transfer calculator
*
*$Id: cceopt.cpp 6.119 2008/08/22 17:07:44 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/02/22 | Added options for looping through pension income split  |
* | CH  | 07/02/21 | Relabelled cceopt                                       |
* | CH  | 07/02/06 | Added pension income splitting                          |
* | LP  | 07/01/23 | Removed idninco and imninc - no income flags - not used |
* | CH  | 07/01/08 | Further changes due to imicnet change                   |
* | LP  | 06/12/12 | Added imicnet and imictot (tax form concepts in code)   |
* | CH  | 06/10/16 | Changed pass 2 criteria                                 |                                          
* | LP  | 06/06/02 | Added imbcss, imispa, imigis for UI/OAS repay - txinet  |                                          
* | LP  | 06/06/02 | Added imuccb for UI (have to adjust net income in UI)   |                                          
* | LP  | 02/05/02 | added imfcbene and imscb  for SA clawback               |                                          
* | LP  | 01/03/12 | Removed Federal Senior's Benefit                        |                                          
* | CH  | 00/12/19 | Family supplement additions                             |                                          
* | EGA | 99/05/25 | UITARGET by province                                    |
* | EGA | 99/03/25 | Add UITARGET for case selection                         |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | BBM | 98/02/24 | Fix Pass2 to call symbolic drv(ie Adrv) not actual drv.c|
* | CH  | 97/10/02 | added imisa iminet to variables kept in EI loop         |
* | CH  | 97/06/17 | fixed year 2000 problem                                 |
* | CH  | 97/06/02 | changed ui pass recursion code                          |
* | BBM | 96/05/27 | add seniors benefits optimization with OAS/GIS          |
* | BBM | 95/12/22 | add employment insurance reforms (pre-calc imfcben)     |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   cceopt   [global]  Multiple calls of tax/transfer calculator
*   pass2    [static]  recalculate UI with child benefits in place
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */
#include <string.h> /* declarations for string manipulation functions       */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */

/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Scceopt[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;

/* ==================== FUNCTION PROTOTYPES =============================== */

static void pass2(P_hh hh, void (* drv)(P_hh));

/* ==================== STATIC VARIABLE DEFINITIONS ======================= */

static int nocceopt = 0;  /* used to prevent endless recursion by cceopt() */
static int nopass2 = 0;  /* used to prevent endless recursion by pass2() */
static int nopens2 = 0;  /* used to prevent endless recursion by cceopt() */


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef _MSC_VER
#pragma function(strcat,strcmp,strcpy,strlen,strset,memset,memcpy,memcmp)
#endif

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,cceopt,pass2)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */



/*****************************************************************************
*cceopt - Multiple calls of tax/transfer calculator
*
*Synopsis:
*    void cceopt(         
*    P_hh hh,             
*    void (* drv)(P_hh)   
*    )                    
*
*Description:
*   This module identifies families which might benefit by setting child care
*   expenses to zero for children <= 6 years of age.  The complexities of
*   the tax reform situation in Canada make it difficult to know how to
*   optimize many of these cases.  These cases are identified by the
*   following criteria:
*     (1) Children aged <= 6 are present, with associated child care
*         expenses.
*     (2) 
*     (3) 
*	 Once these families have been identified, the tax/transfer system is
*   recalculated, setting CCE for young children to zero.  If the family
*   benefits from this change, this scenario is kept.  Otherwise, the
*   previous scenario is restored.
*
*Implementation:
*   The variables used are as follows:
*     im.imccez  - is CCE zeroed?
*     im.imccend - disposable income with CCE not zeroed
*     im.imccezd - disposable income with CCE zeroed
*   
*Return Value:
*   None.
*
*C-Runtime Calls:
*   memset   Uses a given character to initialize a specified number of bytes in the buffer
*
*Calls:
*   pass2    recalculate UI with child benefits in place
*   round    Round to nearest integer
*   drv      Tax/transfer calculator
*
*Structure Members:
*   MP.CCEZOPT      (i)  CCE young kid optimization rules [1=use, 2=zero, 3=optimize]
*   MP.PENSPLTINCR  (i)  Increment by which to split pension income
*   MP.QPENSPLTINCR (i)  Quebec increment by which to split pension income
*   MP.QSPLTPNSFLAG (i)  Quebec flag to split pension income
*   MP.SPLTPNSFLAG  (i)  Flag to split pension income
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfnpers      (i)  Number of persons in census family
*   hd.hdageeld     (i)  Age of eldest in household
*   hd.hdnkids      (i)  Number of children in household
*   hd.hdprov       (i)  Province
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.idcfrh       (i)  Relationship to census family head
*   id.idipens      (i)  Pension income (115)
*   id.idspoflg     (i)  Person has spouse
*   im.imccec       (i)  Child care expenses claimed on behalf of child
*   im.immicons     (i)  Consumable income
*   in.im           (i)  Individual variant result variables [struct]
*   im.imcceni      (io) Consumable income with CCE not zeroed
*   im.imccez       (io) Is CCE zeroed?
*   im.imccezi      (io) Consumable income with CCE zeroed
*   im.impnsph      (io) Pension income split for head
*   im.impnsps      (io) Pension income split for spouse
*   im.imqpnsph     (io) Quebec pension income split for head
*   im.imqpnsps     (io) Quebec pension income split for spouse
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void cceopt(
	P_hh hh,
	void (* drv)(P_hh)
)
{
	register P_in in;
	register int ini;
	register P_cf cf;
	register int cfi;
	int anyfound;      /* Were there any kids aged <= 6 with CCE in HH ? */
	int anylost;       /* Did any CF's lose by zeroing ? */
	NUMBER cfgain;     /* Amount by which CF gained by zeroing */

    int hdpfound; /* a married head has pension income */
    int sppfound; /* a spouse has pension income */
    NUMBER cfinc; /* census family income */
    NUMBER qcfinc; /* census family income for Quebec optimization */
    NUMBER splitbrk; /* the pension splits we'll be trying */
	/**
	* The following arrays hold `im' variables which would be clobbered
	* by the iminit() function.
	**/

	NUMBER cceni[MAXPERS]; /* Consumable income with no zeroing */
	NUMBER ccezi[MAXPERS]; /* Consumable income with zeroing */
	int    ccez[MAXPERS];  /* Is CCE zeroed ? */

    /* don't lose track of current pension test */
	NUMBER penspec[MAXPERS];  /* Pension income split for head */
	NUMBER penspsc[MAXPERS];  /* Pension income split for spouse */
	NUMBER qpenspec[MAXPERS];  /* Pension income split for head in Quebec  */
	NUMBER qpenspsc[MAXPERS];  /* Pension income split for spouse in Quebec */

    NUMBER bestinc[MAXFAM]; /* Best consummable income for the family */
    NUMBER    bestsplit[MAXFAM]; /* Best pension split for the family */
    int    headbest[MAXFAM]; /* Is splitting head's income best for the family? */
    NUMBER qbestinc[MAXFAM]; /* Best consummable income for the family for Quebec optimization*/
    NUMBER    qbestsplit[MAXFAM]; /* Best pension split for the family for Quebec optimization*/
    int    qheadbest[MAXFAM]; /* Is splitting head's income best for the family? for Quebec optimization*/



	DEBUG_ON("cceopt");



	/**
	* First recalculate ui benefits for families with child tax benefits
	* on our first foray into cceopt
	* This will execute each time a call to drv is made, either in the
	* initial call by call.c or each of the two possible subsequent calls
	* within the cceopt calls.
	* nopass2 is set to 1 in before the call to drv made by pass2 and so
	* will prevent endless recursion
	**/
	if (!nopass2)  {
		pass2(hh, drv);
	}

    if (nopass2 || nopens2 || nocceopt) {
        return;
    }

    /* Split pension income */

    /* Does anyone in the household have pension income to split? */
	if ((MP.SPLTPNSFLAG || MP.QSPLTPNSFLAG) && !nopens2 ) {
        hdpfound = 0;
        sppfound = 0;

        /* Start by checking if there is anyone in the household who could split their
          income (e.g. have pension income and be married) */

	    /* process persons in household */
	    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
            if ( MP.SPLTPNSFLAG || (MP.QSPLTPNSFLAG && hh->hd.hdprov==4)) {
		        /* Set flag for pension income if idcfrh==0 and if there exists a spouse */
   		        if (in->id.idcfrh==0 && in->id.idipens > ZERO && in->id.idspoflg) {
			        hdpfound=1;
		        }
		        /* Set flag for pension income if idcfrh==1 */
		        if (in->id.idcfrh==1 && in->id.idipens > ZERO) {
			        sppfound=1;
		        }
                /* Note that if a person has pension income but is not a head or a spouse, 
                   we won't split since there is no spouse with which to split */
            }
	    }
        /* if there is anyone who could split their pension income, then we will
           need to assign varying portions of their pension income to their spouse */

        if (hdpfound || sppfound) {

            nopens2=1;

            if (MP.SPLTPNSFLAG) {

                /* Start by finding the best federal amount */

                /* Save the initial family consummable income (splitting no pension income) */
                /* we'll save it as the current "best" for that family */

		        /* process each census family in household */
		        for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			        cfinc = ZERO;
			        /* process persons in census family */
			        for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
   				        cfinc += round(in->im.immicons);
			        }
                    /* set the best income for this family right now, with no splitting */
                    bestinc[cfi]=cfinc;
                    /* initialize best split to zero, and best person to head*/
                    bestsplit[cfi]=0;

                    if (in->id.idcfrh==0 && in->id.idipens > ZERO) {
                        headbest[cfi]=1;
                    }
                    else {
                        headbest[cfi]=0;
                    }
		        }


                /* We are going to first split the income of the head, if there is a head
                   with pension income.  Start by splitting 50% of their income, and see if the 
                   resulting consummable income is better.  Then split less of their income until
                   you find the best possible split */
                if (hdpfound>0) {
                    /* start at 50% */
                    splitbrk = 0.5;

                    while ( splitbrk>0.0 ) {
                        /* loop through and zero out variables and then set the split for the head */
			            for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
				            memset(&in->im, 0, sizeof(in->im));
				            in->im.impnsph = splitbrk;
				            in->im.impnsps = 0.0;
			            }
                        /* run through the tax and transfer system */
			            drv(hh);

                        /* now check if income increased for the family */
		                /* process each census family in household */
		                for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			                cfinc = ZERO;
			                /* process persons in census family */
			                for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
   				                cfinc += round(in->im.immicons);
			                }
                            /* check if this is now the best income for federal programs */
                            if (cfinc > bestinc[cfi] ) {
                                /* Keep track of the current best */
                                bestinc[cfi]=cfinc;
                                bestsplit[cfi]=splitbrk;
                                headbest[cfi]=1;
		                    }
                        }
                        /* reduce split by the desired increment */
                        splitbrk -= MP.PENSPLTINCR;
                    }
                }
                /* Now split the income of the spouse, if appropriate.*/
                if (sppfound>0) {
                    /* start at 50% */
                    splitbrk = 0.5;
                    while ( splitbrk>0.0 ) {
                        /* loop through and zero out variables and then set the split for the head */
			            for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
				            memset(&in->im, 0, sizeof(in->im));
				            in->im.impnsph = 0.0;
				            in->im.impnsps = splitbrk;
			            }
                        /* run through the tax and transfer system */
			            drv(hh);
                        /* now check if income increased for the family */
		                /* process each census family in household */
		                for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			                cfinc = ZERO;
			                /* process persons in census family */
			                for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
   				                cfinc += round(in->im.immicons);
			                }
                            /* check if this is now the best income */
                            if (cfinc > bestinc[cfi] ) {
                                /* Keep track of the current best */
                                bestinc[cfi]=cfinc;
                                bestsplit[cfi]=splitbrk;
                                headbest[cfi]=0;
		                    }
                        }
                        /* reduce split by the desired increment */
                        splitbrk -= MP.PENSPLTINCR;
                    }
                }

                /* we now know what the best split is for each family, so run it that way */
		        for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			        /* process persons in census family */
			        for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
				        memset(&in->im, 0, sizeof(in->im));
                        if (headbest[cfi] == 1) {
    				        in->im.impnsph = bestsplit[cfi];
    				        in->im.impnsps = 0.0;
                        }
                        else {
    				        in->im.impnsps = bestsplit[cfi];
    				        in->im.impnsph = 0.0;
                        }
			        }
                }
	            drv(hh);
            }

            if (MP.QSPLTPNSFLAG && hh->hd.hdprov==4) {

                /* If in Quebec, now find the best provincial amount */

                /* now save the initial family consummable income */
                /* we'll save it as the current "best" for that family */

		        /* process each census family in household */
		        for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			        qcfinc = ZERO;
			        /* process persons in census family */
			        for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
   				        qcfinc += round(in->im.immicons);
			        }
                    /* set the best income for this family right now, with no splitting */
                    qbestinc[cfi]=qcfinc;
                    /* initialize best split to zero, and best person to head*/
                    qbestsplit[cfi]=0;

                    if (in->id.idcfrh==0 && in->id.idipens > ZERO) {
                        qheadbest[cfi]=1;
                    }
                    else {
                        qheadbest[cfi]=0;
                    }
		        }
                /* We are going to first split the income of the head */
                if (hdpfound>0) {
                    /* start at 50% */
                    splitbrk = 0.5;
                    while ( splitbrk>0.0 ) {
                        /* loop through and zero out variables and then set the split for the head */
			            for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

    	                    penspec[ini] = in->im.impnsph;
    	                    penspsc[ini] = in->im.impnsps;
				            memset(&in->im, 0, sizeof(in->im));
    	                    in->im.impnsph = penspec[ini];
    	                    in->im.impnsps = penspsc[ini];

				            in->im.imqpnsph = splitbrk;
				            in->im.imqpnsps = 0.0;
			            }
                        /* run through the tax and transfer system */
			            drv(hh);

                        /* now check if income increased for the family */
		                /* process each census family in household */
		                for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			                qcfinc = ZERO;
			                /* process persons in census family */
			                for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
   				                qcfinc += round(in->im.immicons);
			                }
                            if (qcfinc > qbestinc[cfi] ) {
                                qbestinc[cfi]=qcfinc;
                                qbestsplit[cfi]=splitbrk;
                                qheadbest[cfi]=1;
		                    }
                        }
                        /* reduce split by the desired increment */
                        splitbrk -= MP.QPENSPLTINCR;
                    }
                }
                /* Now split the income of the spouse, if appropriate.*/
                if (sppfound>0) {
                    /* Start at 50% */
                    splitbrk = 0.5;
                    while ( splitbrk>0.0 ) {
                        /* loop through and zero out variables and then set the split for the head */
			            for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
    	                    penspec[ini] = in->im.impnsph;
    	                    penspsc[ini] = in->im.impnsps;
				            memset(&in->im, 0, sizeof(in->im));
    	                    in->im.impnsph = penspec[ini];
    	                    in->im.impnsps = penspsc[ini];

				            in->im.imqpnsph = 0.0;
				            in->im.imqpnsps = splitbrk;
			            }
                        /* run through the tax and transfer system */
			            drv(hh);
                        /* now check if income increased for the family */
		                /* process each census family in household */
		                for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			                qcfinc = ZERO;
			                /* process persons in census family */
			                for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
   				                qcfinc += round(in->im.immicons);
			                }
                            if (qcfinc > qbestinc[cfi] ) {
                                /* Keep track of the current best */
                                qbestinc[cfi]=qcfinc;
                                qbestsplit[cfi]=splitbrk;
                                qheadbest[cfi]=0;
		                    }
                        }
                        /* reduce split by the desired increment */
                        splitbrk -= MP.QPENSPLTINCR;
                    }
                }

                /* we now know what the best split is for each family, so run it that way */
		        for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			        /* process persons in census family */
			        for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
    	                penspec[ini] = in->im.impnsph;
    	                penspsc[ini] = in->im.impnsps;
				        memset(&in->im, 0, sizeof(in->im));

    	                in->im.impnsph = penspec[ini];
    	                in->im.impnsps = penspsc[ini];

                        if (qheadbest[cfi] == 1) {
    				        in->im.imqpnsph = qbestsplit[cfi];
    				        in->im.imqpnsps = 0.0;
                        }
                        else {
    				        in->im.imqpnsps = qbestsplit[cfi];
    				        in->im.imqpnsph = 0.0;
                        }
			        }
                }
	            drv(hh);
            }
        }
        nopens2=0;

	}


	/**
	* Just return if
	*   (1) CCE optimization option has not been requested,
	*   (2) the household contains no children at all,
	*   (3) cceopt() has already been called (to prevent endless recursion).
	*   (4) we are in the middle of a pass2 call.
	**/

	DEBUG4("%s CCEZOPT=%d hdnkids=%d nocceopt=%d\n",
		MP.CCEZOPT, hh->hd.hdnkids, nocceopt);

	if (MP.CCEZOPT != 3 || (hh->hd.hdnkids==0 && hh->hd.hdageeld<=55) || nocceopt || nopass2 || nopens2) {
		DEBUG_OFF("cceopt");
		return;
	}


	if (hh->hd.hdnkids!=0) {
		/**
		* At this point, the tax/transfer system has been computed assuming
		* that everyone uses any CCE for young children.
		* We now squirrel away each person's consumable income, and set the
		* variable imccez to 1 for relevant children in preparation for a
		* re-computation of the tax/transfer system.
		* If no relevant kids were found, we simply return.
		**/

		anyfound = 0;

		/* process persons in household */
		for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

			/* Record consumable income (no zeroing of CCE) */
			cceni[ini] = in->im.immicons;
			in->im.imcceni = in->im.immicons;
			in->im.imccezi = in->im.immicons;

			/* Set flag for trial zeroing of CCE */
			if (in->id.idage <= 6 && in->im.imccec > ZERO) {
				anyfound = 1;
				in->im.imccez = 1;
			}
		}


		/* Just return at this point if no relevant children found */

		if (anyfound) {

			/* Re-do the entire tax/transfer system */

			nocceopt = 1;

			/* Zero out all modelled variables, etc., like iminit() does */
			/* Restore ccez variable so that txccea() will do what we want. */

			for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

				ccez[ini] = in->im.imccez;
                /* Don't lost track of pension income */
    	        penspec[ini] = in->im.impnsph;
    	        penspsc[ini] = in->im.impnsps;

    	        qpenspec[ini] = in->im.imqpnsph;
    	        qpenspsc[ini] = in->im.imqpnsps;

				memset(&in->im, 0, sizeof(in->im));

				in->im.imccez = ccez[ini];
                /* Don't lost track of pension income */
    	        in->im.impnsph = penspec[ini];
    	        in->im.impnsps = penspsc[ini];

    	        in->im.imqpnsph = qpenspec[ini];
    	        in->im.imqpnsps = qpenspsc[ini];
			}

			drv(hh);


			for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

				/* Record consumable income (with zeroing of CCE) */
				in->im.imccezi = in->im.immicons;
				ccezi[ini] = in->im.immicons;

				/* Restore consumable income (with no zero-ing of CCE) */
				in->im.imcceni = cceni[ini];
			}

			/**
			* Determine which census families gained by the change.
			* If any lost, then record that fact, and set imccez back to zero
			* for those familes.  Then re-compute the tax/transfer system once again.
			**/

			anylost = 0;

			/* process each census family in household */
			for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {

				cfgain = ZERO;

				/* process persons in census family */
				for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
					
					/* Update CF net gain of zeroing vs. non-zeroing */
					cfgain += in->im.imccezi - in->im.imcceni;
				}

				if (cfgain < ZERO) {
					anylost = 1;   /* sigh! we have to call drv() again. */

					/* process persons in census family */
					for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {

						/* change method back to no-zeroing for kids in this CF */
						in->im.imccez = 0;
					
					}
				}
			}


			/* If we have to call drv() yet again, so be it. */

			if (anylost) {

				/* Re-do the entire tax/transfer system */

				nocceopt = 1;

				/* Zero out all modelled variables, etc., like iminit() does */
				/* Restore ccez variable so that txccea() will do what we want. */

				for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

					ccez[ini] = in->im.imccez;
                    /* Don't lost track of pension income */
    	            penspec[ini] = in->im.impnsph;
    	            penspsc[ini] = in->im.impnsps;

    	            qpenspec[ini] = in->im.imqpnsph;
    	            qpenspsc[ini] = in->im.imqpnsps;

					memset(&in->im, 0, sizeof(in->im));

					in->im.imccez = ccez[ini];
                    /* Don't lost track of pension income */
    	            in->im.impnsph = penspec[ini];
    	            in->im.impnsps = penspsc[ini];

    	            in->im.imqpnsph = qpenspec[ini];
    	            in->im.imqpnsps = qpenspsc[ini];
				}

				drv(hh);
			}


			for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

				/* restore variables */
				in->im.imccez = ccez[ini];
				in->im.imccezi = ccezi[ini];
				in->im.imcceni = cceni[ini];
			}

			/* reset the cceopt flag for the next invocation of drv() */
			nocceopt = 0;
		}
	}
	
	DEBUG_OFF("cceopt");
}

/*****************************************************************************
*pass2 - recalculate UI with child benefits in place
*
*Synopsis:
*    static void pass2(   
*    P_hh hh,             
*    void (* drv)(P_hh)   
*    )                    
*
*Description:
*   This module recalculates UI benefits under reform options which 
*   require that federal child tax benefits be known.  
*   Only households containing census families where the head or spouse is
*   collecting UI and the family receives some child benefit need to
*   have this recalculation performed.
*	 Once these families have been identified, the tax/transfer system is
*   recalculated, setting child tax benefits to the value obtained in the 
*   first pass of drv.
*
*C-Runtime Calls:
*   memset   Uses a given character to initialize a specified number of bytes in the buffer
*
*Calls:
*   drv      Tax/transfer calculator
*
*Structure Members:
*   MP.UITARGET     (i)  UI/EI target for the number of cases (UI and EI)
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfnpers      (i)  Number of persons in census family
*   hd.hdprov       (i)  Province
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idcfrh       (i)  Relationship to census family head
*   id.iduirank     (i)  Predicted ranked likelihood of getting UI
*   in.im           (i)  Individual variant result variables [struct]
*   uc.ucstat       (i)  Claim status flag
*   im.imbcss       (io) British Columbia Seniors Supplement
*   im.imccez       (io) Is CCE zeroed?
*   im.imfcben      (io) Total Federal Child Benefits
*   im.imfcbene     (io) Total Federal Child Benefit Supplement
*   im.imicnet      (io) Net Income (line 236)
*   im.imigis       (io) GIS benefits
*   im.iminet       (io) Net income
*   im.imisa        (io) Social assistance (or replacement program)
*   im.imispa       (io) Spouse's allowance
*   im.impnsph      (io) Pension income split for head
*   im.impnsps      (io) Pension income split for spouse
*   im.imqpnsph     (io) Quebec pension income split for head
*   im.imqpnsps     (io) Quebec pension income split for spouse
*   im.imscb        (io) Saskatchewan Child Benefit
*   im.imuccb       (io) Universal Child Care Benefit amount
*
*Called by:
*   cceopt   Multiple calls of tax/transfer calculator
*
*****************************************************************************/
static void pass2(
	P_hh hh,
	void (* drv)(P_hh)
)
{
	register P_in in;
	register int ini;
	register P_cf cf;
	register int cfi;

	int uifound;      /* Were there any UI claimants with CTB in household? */
	int safound;      /* Were there any SA recipients in the household? */
   	int gisorspa;     /* Were there any GIS or SPA recipients in the household? */
    int uccbfnd;      /* Were there any UCCB recipients in the household? */
	/**
	* The following arrays hold `im' variables which would be clobbered
	* by the iminit() function.
	**/

	NUMBER fcben[MAXPERS]; /* Federal Child tax benefit */
	NUMBER fcbene[MAXPERS];  /* Federal Child Benefit Supplement */
	NUMBER scb[MAXPERS];    /* Saskatchewan Child Benefit */
	NUMBER inet[MAXPERS]; /* Net income */
	NUMBER icnet[MAXPERS];  /* Net income (line 236) */
	NUMBER isa[MAXPERS]; /* Social assistance */
	NUMBER uccb[MAXPERS]; /* Universal Child Care Benefit  */
	NUMBER gis[MAXPERS];  /* Federal GIS benefits */
	NUMBER spa[MAXPERS];  /* Federal SPA benefits */
	NUMBER bcss[MAXPERS];  /* British Columbia Senior Supplement */
	int uiccez[MAXPERS]; /* Zero out cc*/
    /* don't lose track of current pension test */
	NUMBER penspe[MAXPERS];  /* Pension income split for head */
	NUMBER pensps[MAXPERS];  /* Pension income split for spouse */

	NUMBER qpenspe[MAXPERS];  /* Pension income split for head */
	NUMBER qpensps[MAXPERS];  /* Pension income split for spouse */


	/**
	* Just return if
	*   (1) pass2() has already been called (to prevent endless recursion).
	**/

	DEBUG4("%s UIBASOPT=%d hdnkids=%d nopass2=%d\n",
		MP.UIBASOPT, hh->hd.hdnkids, nopass2);
/*
	if (!MP.UIEIFSFLG || MP.UIEIOPT != 2 || hh->hd.hdnkids==0  
		  	|| hh->hd.hdageeld < 54 || nopass2) {  
		return;
	}*/

	if (nopass2) {  
		return;
	}

	/**
	* At this point, the UI has been computed assuming
	* that everyone received no child tax benefits.
	* We now squirrel away each person's child tax benefits
	* If no relevant EI recipients were found, we simply return.
	**/

	uifound = 0;
	safound = 0;
 	gisorspa = 0; 
    uccbfnd = 0;

	/**
	* Test for uifound flag to see if we need to re-calculate the household:
	*	(1) Households containing census families with both children
	*      and some UI claim data for either the head or spouse if present
	*/
	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {

		/* process persons in census family */
		for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {

			if ((in->id.idcfrh<=1 && cf->cfnkids>0 && (in->id.uc1.ucstat>0 || in->id.uc2.ucstat>0))&& (in->id.iduirank/1000 < MP.UITARGET[hh->hd.hdprov]))  {
				uifound = 1;
			}
			if (in->im.imisa > 0 || in->im.imbcss > 0) {
				safound = 1;
			}
		   	if (in->im.imigis > 0 || in->im.imispa > 0) {
				gisorspa=1;
			}  
			if (in->im.imuccb > 0)  {
				uccbfnd = 1;
			}
		}
	}


	/* Just return at this point if no UI claimants found */

    if (!uifound && !safound && !gisorspa && !uccbfnd) {   
		return;
	}

	/**
	* Re-do the entire tax/transfer system for:
	*	(1) Households containing census families with both a child tax benefit
	*      and some UI claim data for the head or spouse
	*/

	nopass2 = 1;

	/* Zero out all modelled variables, etc., like iminit() does */
	/* Restore ccez variable so that txccea() will do what we want. */

	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

        /* keep both the income variables which are needed, as well as the
              other variables which we are currently testing out */

		fcben[ini] = in->im.imfcben;
		fcbene[ini] = in->im.imfcbene;
		scb[ini] = in->im.imscb;
		inet[ini] = in->im.iminet;
		icnet[ini] = in->im.imicnet; 
		isa[ini] = in->im.imisa;
        uiccez[ini] = in->im.imccez;  
		uccb[ini] = in->im.imuccb;
		gis[ini] = in->im.imigis;
		spa[ini] = in->im.imispa;
		bcss[ini] = in->im.imbcss;
    	penspe[ini] = in->im.impnsph;
    	pensps[ini] = in->im.impnsps;
    	qpenspe[ini] = in->im.imqpnsph;
    	qpensps[ini] = in->im.imqpnsps;

		memset(&in->im, 0, sizeof(in->im));

		in->im.imfcben = fcben[ini];
		in->im.imfcbene = fcbene[ini];
		in->im.iminet = inet[ini];
		in->im.imicnet = icnet[ini];
		in->im.imisa = isa[ini];
		in->im.imccez = uiccez[ini];
		in->im.imscb = scb[ini];
		in->im.imuccb = uccb[ini];
		in->im.imigis = gis[ini];
		in->im.imispa = spa[ini];
		in->im.imbcss = bcss[ini];
    	in->im.impnsph = penspe[ini];
    	in->im.impnsps = pensps[ini];
    	in->im.imqpnsph = qpenspe[ini];
    	in->im.imqpnsps = qpensps[ini];
	}

	drv(hh);

	/* reset the nopass2 flag for the next invocation of drv() */
	nopass2 = 0;

	return;
}




