/***
*gist.cpp - Compute Provincial GIS top-ups for elderly
*
*$Id: gist.cpp 6.123 2008/08/22 17:07:44 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 08/08/19 | bug fix to NL - one senior couples use family income    |
* | LP  | 07/07/04 | Added senior couple NL parameter, changed to net income |
* | LP  | 07/02/13 | Income test for MAN GIST now uses imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/19 | Removed AUCCBITF-UCCB removed from income as per forms  |
* | LP  | 06/12/21 | Changed iminet to imicnet income concept for NB test    |
* | LP  | 06/06/14 | Added imbcss - B.C. Senior Supplement (part of SA)      |
* | LP  | 06/06/13 | Added AUCCBITF for uccb to reduce inc for Alta Sen. Ben |
* |     |          |  and added ASBCYFLG to calculate for calendar year      |
* | LP  | 06/01/23 | Added SASKTECA - Sask. temporary energy cost allowance  |
* | LP  | 04/03/09 | Added ALTA senior supplemental accommodation benefit    |
* | LP  | 04/02/20 | Deleted ALTA Widows Pension and B.C. sen ben for < 65   |
* | LP  | 04/02/18 | Deducted idialimo from income concept for ALTA Sen.Ben  |
* | CH  | 03/10/17 | Deleted iditogv, replaced idiint + idioinv by idiinvrd  |
* | LP  | 01/06/11 | Changed the options for idtenur due to database change  | 
* | LP  | 01/04/05 | Added NBSBSPA NB seniors' benefit SPA eligibility flag  |
* | LP  | 01/02/28 | Removed Federal Seniors Benefit program                 |
* | CH  | 00/05/24 | Added iditrrsp                                          |
* | LP  | 99/04/27 | Added NFLD Low-Income Seniors' Benefit                  |
* | BBM | 99/02/11 | Drop idiroom references (now in idisenf as of 7.0)      |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/04/07 | Added NB Low-Income Senior's Benefit                    |
* | CH  | 98/02/12 | Changed code for Alberta seniors benefit and widdows    |
* | EGA | 97/10/07 | Fixed mixed cased (gis-nogis) in various provinces      |
* | CH  | 97/06/17 | Fixed year 2000 problem                                 |
* | BBM | 96/08/20 | Allow Federal Seniors benefits to trigger GIST programs |
* | RSE | 94/04/07 | Add Alberta seniors benefit for 1994 and beyond         |
* | BBM | 92/06/04 | Change filter to renters on Ont. seniors grant          |
* | SFG | 90/10/05 | reactivate use of fx data                               |
* | SFG | 90/09/18 | disable use of fx data                                  |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   gist     [global]  Compute Provincial GIS top-ups for elderly
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== MANIFEST CONSTANTS ================================ */

/* the following codes are placed in nfoldtyp. */
/* They are computed in gis.                 */

#define GISSINGLE   1    /* single, with OAS                        */
#define SPAWIDOW    2    /* widow(er) aged 60-64                    */
#define GISCOUPLE   3    /* couple, both have OAS                   */
#define GISSPA      4    /* couple, elder has OAS, younger is 60-64 */
#define GISYNG      5    /* couple, only younger has OAS            */
#define GISELD      6    /* couple, only elder has OAS              */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Sgist[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,gist)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*gist - Compute Provincial GIS top-ups for elderly
*
*Synopsis:
*    void gist(           
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   txhhexp  Compute and pro-rate household taxes, rent, etc.
*   isnzero  Is argument non-zero ?
*   nneg     Change negative numbers to zero
*   taxbak1  One level tax-back function
*
*Structure Members:
*   MP.ALTAMIN      (i)  Alberta GIS supplement minimum annual benefit
*   MP.ALTASC       (i)  Alberta GIS supplement maximum annual benefit
*   MP.ALTASOPT     (i)  Alberta seniors option [1=GIST,2=new seniors benefit 1994]
*   MP.ASBBASIC     (i)  Alberta seniors benefit annual basic benefit
*   MP.ASBBASICP    (i)  Alberta seniors benefit annual basic benefit for 1st half of year
*   MP.ASBBOASP     (i)  Alberta seniors benefit Basic OAS from previous year
*   MP.ASBCYFLG     (i)  Alberta seniors benefit calendar year calculation flag
*   MP.ASBEMP       (i)  Alberta seniors benefit employment inclusion rate
*   MP.ASBEMPP      (i)  Alberta seniors benefit employment inclusion rate for 1st half of year
*   MP.ASBNOAS      (i)  Alberta seniors benefit reduction if no OAS [senior type]
*   MP.ASBNOASP     (i)  Alberta seniors benefit reduction if no OAS for 1st half of year [senior type]
*   MP.ASBNSS       (i)  Alberta seniors benefit non-senior spouse supplement
*   MP.ASBNSSP      (i)  Alberta seniors benefit non-senior spouse supplement for 1st half of year
*   MP.ASBRENT      (i)  Alberta seniors benefit renter supplement
*   MP.ASBRENTP     (i)  Alberta seniors benefit renter supplement for 1st half of year
*   MP.ASBRR        (i)  Alberta seniors benefit reduction rate [senior type]
*   MP.ASBRRP       (i)  Alberta seniors benefit reduction rate for 1st half of year [senior type]
*   MP.ASBSAB       (i)  Alberta seniors benefit supplemental accommodation benefit
*   MP.ASBSABP      (i)  Alberta seniors benefit supplemental accommodation benefit for 1st half of year
*   MP.ASBSABRR     (i)  Alberta seniors benefit supplemental accommodation reduction rate
*   MP.ASBSABRRP    (i)  Alberta seniors benefit supplemental accommodation reduction rate for 1st half of year
*   MP.ASBSS        (i)  Alberta seniors benefit senior spouse supplement
*   MP.ASBSSOPT     (i)  Alberta seniors benefit 1 senior couple option [1=model separately,2=model as senior couples]
*   MP.ASBSSOPTP    (i)  Alberta seniors benefit 1 senior couple option for 1st half of year
*   MP.ASBSSP       (i)  Alberta seniors benefit senior spouse supplement for 1st half of year
*   MP.BCC          (i)  British Columbia GIS supplement: married pensioners
*   MP.BCS          (i)  British Columbia GIS supplement: single pensioners
*   MP.BOAS         (i)  Basic OAS
*   MP.GISRLM       (i)  Basic GIS reduction level: married pensioners
*   MP.GISTFLAG     (i)  Provincial GIS top-up flag
*   MP.MANC         (i)  Manitoba GIS supplement: married pensioners
*   MP.MANCNPF      (i)  Manitoba GIS supplement reduction point: married
*   MP.MANS         (i)  Manitoba GIS supplement: single pensioners
*   MP.MANSNPF      (i)  Manitoba GIS supplement reduction point: single
*   MP.NBSB         (i)  New Brunswick Low-Income Seniors' Benefit
*   MP.NBSBSPA      (i)  N.B. Low-Income Seniors' Benefit - SPA eligibility
*   MP.NFSBBAS      (i)  NFLD Low-Income Seniors' Benefit base amount
*   MP.NFSBCBAS     (i)  NFLD Low-Income Seniors' Benefit base amount for senior couples
*   MP.NFSBCRR      (i)  NFLD Low-Income Seniors' Benefit reduction rate for senior couples
*   MP.NFSBCTD      (i)  NFLD Low-Income Seniors' Benefit turndown for senior couples
*   MP.NFSBRR       (i)  NFLD Low-Income Seniors' Benefit reduction rate
*   MP.NFSBTD       (i)  NFLD Low-Income Seniors' Benefit turndown
*   MP.NS13         (i)  Nova Scotia GIS supplement for 1/3 GIS
*   MP.NS23         (i)  Nova Scotia GIS supplement for 2/3 GIS
*   MP.NSLT13       (i)  Nova Scotia GIS supplement for less than 1/3 GIS
*   MP.NSMAX        (i)  Nova Scotia maximum GIS supplement level
*   MP.ONTC         (i)  Ontario GIS supplement: married pensioners
*   MP.ONTFPTG      (i)  Ontario property tax grant (fraction of rent)
*   MP.ONTPTG       (i)  Ontario property tax grant for seniors
*   MP.ONTS         (i)  Ontario GIS supplement: single pensioners
*   MP.ONTSTG       (i)  Ontario sales tax grant for seniors
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   MP.SASKC        (i)  Saskatchewan GIS supplement: married pensioners
*   MP.SASKMINC     (i)  Saskatchewan GIS supplement minimum benefits: married
*   MP.SASKMINS     (i)  Saskatchewan GIS supplement minimum benefits: single
*   MP.SASKRR1      (i)  Saskatchewan GIS supplement reduction rate: regular
*   MP.SASKRR2      (i)  Saskatchewan GIS supplement reduction rate: 1 GIS
*   MP.SASKRR3      (i)  Saskatchewan GIS supplement reduction rate: SPA
*   MP.SASKS        (i)  Saskatchewan GIS supplement: single pensioners
*   MP.SASKTECA     (i)  Saskatchewan GIS supplement temporary energy cost allowance
*   MP.TARGETYEAR   (i)  Year of analysis
*   hd.hdageeld     (i)  Age of eldest in household
*   hd.hdprov       (i)  Province
*   hd.hdtenur      (i)  Tenure
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.idcfrh       (i)  Relationship to census family head
*   id.idialimo     (i)  Alimony income received
*   id.idieflag     (i)  Cloned institutionalized elderly person
*   id.idiemp       (i)  Wages & salaries
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idspoflg     (i)  Person has spouse
*   im.imgisinc     (i)  Individual's income for GIS/SPA reduction
*   im.imgismax     (i)  Maximum amount of GIS
*   im.imgistyp     (i)  Type of GIS entitlement
*   im.imicnet      (i)  Net Income (line 236)
*   im.imigis       (i)  GIS benefits
*   im.imioas       (i)  OAS benefits
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imispa       (i)  Spouse's allowance
*   im.imitot       (i)  Total income
*   im.imoasres     (i)  Partial OAS fraction
*   im.imrentpd     (i)  Imputed rent paid
*   im.imspamax     (i)  Maximum amount of SPA
*   im.imspatyp     (i)  Type of SPA entitlement
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imiasb       (io) Alberta seniors benefit
*   im.imigist      (io) GIS provincial top-up
*   im.impptg       (io) Provincial property tax grant for seniors
*   im.impstg       (io) Provincial sales tax grant for seniors
*   im.imbcss       (o)  British Columbia Seniors Supplement
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void gist(
	P_hh hh
)
{
	register P_in in;     /* pointer to data for current person */
	register P_in inspo;  /* pointer to spouse */
	int ini;

	P_in partin;     /* pointer to partner's INDAT (for married couple) */
	NUMBER nfgisinc = ZERO;	/* nuclear family income for GIS/SPA tax-back */
	NUMBER ratio = ZERO;
	NUMBER asbfinc1 = ZERO;	/* Alberta seniors benefit family income for 1st half of year */
	NUMBER asbfinc2 = ZERO;	/* Alberta seniors benefit family income */
	NUMBER asbben = ZERO;	/* Alberta seniors benefit maximum benefit */
	NUMBER asbsab = ZERO;   /* Alberta seniors benefit supplementary accommodation benefit */
	NUMBER asbrr = ZERO;	/* Alberta seniors benefit reduction rate */
	NUMBER asbben1 = ZERO;	/* Alberta seniors benefit maximum benefit */
	NUMBER asbsab1 = ZERO;   /* Alberta seniors benefit supplementary accommodation benefit */
	NUMBER asbrr1 = ZERO;	/* Alberta seniors benefit reduction rate */
	NUMBER asbben2 = ZERO;	/* Alberta seniors benefit maximum benefit */
	NUMBER asbsab2 = ZERO;   /* Alberta seniors benefit supplementary accommodation benefit */
	NUMBER asbrr2 = ZERO;	/* Alberta seniors benefit reduction rate */
    NUMBER nooas = ZERO;    /* Alberta seniors benefit not qualified for OAS */
    NUMBER spnooas = ZERO;  /* Alberta seniors benefit spouse not qualified for OAS */
	int oastype;            /* Alberta seniors benefit oas type */
	NUMBER fedben = ZERO;	/* Actual Federal Elderly supplements */
	NUMBER fedspa = ZERO;	/* Actual Federal SPA benefits */
	NUMBER maxben = ZERO;	/* Maximum Federal Elderly supplements */
	NUMBER qualify = ZERO;  /* Alberta seniors benefit non-senior dependent */
	NUMBER nfgist = ZERO;   /* Newfoundland seniors benefit */
	NUMBER nbfedben = ZERO;  /* New Brunswick low-income seniors benefit eligibility */
	NUMBER nbspben = ZERO;   /* New Brunswick low-income seniors benefit spouse's eligibility */
	NUMBER faminet = ZERO;   /* family net income concept for NL Seniors Benefit */

	DEBUG_ON("gist");



	if (!MP.GISTFLAG) {
		DEBUG1("%s turned off by GISTFLAG\n");
		DEBUG_OFF("gist");
		return;
	}

	if (hh->hd.hdageeld < 55) {
		DEBUG1("%s eldest in household is under 55\n");
		DEBUG_OFF("gist");
		return;
	}
	/* call this function to perform the imputation of rent paid */
	/* which is required for Ontario */
	txhhexp(hh);
     

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		/* initialize spouse pointer */
	   inspo = in->id.idinspo;

		/* compute family GIS tax-back income */
		nfgisinc = in->im.imgisinc;
		if (in->id.idspoflg) {
			nfgisinc += inspo->im.imgisinc;
		}


		/**
		* calculate GIST eligibility variables 
		**/
		fedben=ZERO;
		fedspa=ZERO;
		maxben=ZERO;

		if (isnzero(in->im.imigis))  {
		  	fedben=in->im.imigis;
			maxben=in->im.imgismax;
		}
		else if (isnzero(in->im.imispa))  {
		  	fedspa=in->im.imispa;
			maxben=in->im.imgismax;
		}

		/* for N.B., eligibility required receipt of GIS. */
		/* beginning in 1999, eligibility was broadened to include receipt of SPA */

		nbfedben = ZERO;
		nbspben =  ZERO;
		if (isnzero(in->im.imigis)) {
			nbfedben = in->im.imigis ;
		}
		if (MP.NBSBSPA && isnzero(in->im.imispa)) {
			nbfedben += in->im.imispa ;
		}
		/* calculate spouse's eligibility */
		if (in->id.idspoflg) {
			if (isnzero(inspo->im.imigis)) {
				nbspben = inspo->im.imigis ;
			}
			if (MP.NBSBSPA && isnzero(inspo->im.imispa)) {
				nbspben += inspo->im.imispa ;
			}
		}

		/* calculate GIST entitlement */
		DEBUG2("%s prov=%d\n", hh->hd.hdprov);
		in->im.imigist = ZERO;
		switch (hh->hd.hdprov) {
			
			case PRNFLD:
				/* Newfoundland Low-Income Seniors' Benefit*/
				
			      /*  Give benefit to those 65 and older only */
				  /*  This benefit begins in 1999  */
				  /*  Initially this program was based on individual incomes only */
				  /*  Change introduced in 2007, where both couples are seniors, */
				  /*  then the income test is based on family net income with higher thresholds */
				faminet = ZERO;
				if (in->id.idage >= 65) {
					  /*  Maximum benefit given if income is less than turndown */
					  /*  benefit is reduced by reduction rate */
					  nfgist = MP.NFSBBAS;
					  faminet = nneg(in->im.imicnet - in->im.imuccb) * MP.PYINC;
					  /* if no spouse, or spouse under 65 then base it on the individual's income */
					  if (!in->id.idspoflg ) {
					  	  if (isnzero(nfgist)) {
							taxbak1(faminet, MP.NFSBTD, &nfgist, MP.NFSBRR);
						  }
					  }
					  else if (in->id.idspoflg && inspo->id.idage < 65) {
						 faminet += (nneg(inspo->im.imicnet - inspo->im.imuccb) * MP.PYINC);
					  	 if (isnzero(nfgist)) {
			  	  			taxbak1(faminet, MP.NFSBCTD, &nfgist, MP.NFSBRR);
						 }
					  }
					  
					  /* where an eligible spouse (over 65) exists, income test will be based */
					  /* on previous years' family net income (adjusted for UCCB) */
					  
					  else if (in->id.idspoflg && inspo->id.idage >= 65)  {
						 nfgist = MP.NFSBCBAS;
						 faminet += (nneg(inspo->im.imicnet - inspo->im.imuccb) * MP.PYINC);
					  	 if (isnzero(nfgist)) {
				  	  		taxbak1(faminet, MP.NFSBCTD, &nfgist, MP.NFSBCRR);
							/* since the maximum is combined for the couple, */
							/* give each spouse half of the calculated benefit */
							nfgist = nfgist/2;
						 }
					  }

				  	  in->im.imigist = nneg(nfgist);
				}	  
				break;

			case PRNS:
				/* Nova Scotia special social assistance (SSA)*/
				if (isnzero(fedben) && isnzero(maxben)) {
					ratio = fedben / maxben;
					if (ratio >= ONE) in->im.imigist = MP.NSMAX;
					else if (ratio >= (NUMBER) .666) in->im.imigist = MP.NS23;
					else if (ratio >= (NUMBER) .333) in->im.imigist = MP.NS13;
					else                    in->im.imigist = MP.NSLT13;
					DEBUG3("%s ratio=%6.4f, igist=%.2f\n",
					        ratio, in->im.imigist);
				}
				else {
					DEBUG1("%s did not receive GIS\n");
				}
				break;

			case PRNB:
				/* New Brunswick Low-Income Seniors' Benefit*/
				/* benefit is given to people with GIS */
				if (isnzero(nbfedben)) {
				/* only give benefit to people who have GIS.  When both spouses
				   receive GIS, only give to one spouse.  Give to spouse with lower
				   net income */
					if (!in->id.idspoflg) {
						in->im.imigist = MP.NBSB;
					}
					/* both spouses receive GIS - only one can claim */
					else if (isnzero(nbspben) ) {
						/* give to spouse with lower net income.  If both spouses 
						   have same income, then give to first spouse in household (eldest) */
						if ((in->im.imicnet <= inspo->im.imicnet) && inspo->im.imigist==0) {
							in->im.imigist = MP.NBSB;
						}
	  				}
					/* spouse did not receive GIS */
					else {
						in->im.imigist = MP.NBSB;
					}
				}
   			    break;


			case PRONT:
				/* Ontario GAINS-A */
				if (isnzero(fedben)) {
					switch(in->im.imgistyp) {  
					case 1:
						DEBUG1("%s one GIS recipient alone or in couple\n");
						in->im.imigist = nneg(MP.ONTS - (NUMBER) .50 * nfgisinc);
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
						break;
					case 2:
						if (in->id.idage >= 65)  {
						DEBUG1("%s one GIS recipient alone or in couple\n");
						in->im.imigist = nneg(MP.ONTS - (NUMBER) .25 * nneg(nfgisinc-
							MP.BOAS-1.5*MP.GISRLM));
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
					    break;	
					case 3: 
						DEBUG1("%s both GIS recipients in couple\n");
						in->im.imigist = nneg(MP.ONTC - (NUMBER) .25 * nfgisinc);
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
					    break;  
					case 4:
						if (in->id.idage >= 65) {
						DEBUG1("%s one GIS recipient alone or in couple\n");
						in->im.imigist = nneg(MP.ONTS - (NUMBER) .25 * nneg(nfgisinc-
							MP.BOAS-1.5*MP.GISRLM));
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
					    break;	
					}
				}
				else {
				     /* do not assign gist to unidentified gistyp */
		             if (in->im.imgistyp == 0) in->im.imigist = ZERO;
	                 DEBUG1("%s did not receive GIS\n");
				}
				     
				
				/* Ontario tax grants */
				if (in->id.idage >= 65) {

					/* Sales tax grant */
					in->im.impstg = MP.ONTSTG;

					/* Property tax grant (only family head gets it) */
					if (!in->id.idieflag && in->id.idcfrh==0) {
						if (hh->hd.hdtenur == 1) {
							/* renters get 20% of rental expenses up to the maximum */
							in->im.impptg = MP.ONTFPTG * in->im.imrentpd/*hh->fx.fxio[IO_GPRENT]*/;
							if (in->im.impptg > MP.ONTPTG) {
								in->im.impptg = MP.ONTPTG;
							}
						}
						else {
							/* homeowners get full amount */
							in->im.impptg = MP.ONTPTG;
						}
					}
					/* add to imigist for now */
					in->im.imigist += in->im.impstg
							            + in->im.impptg;
				}
				break;

			case PRMAN:
				/* Manitoba supplement for pensioners (MSP for GIS/SPA) */
				if (isnzero(fedben) || isnzero(fedspa)) {
					DEBUG1("%s person received GIS or SPA\n");
					if (in->im.imgistyp == 1 || in->im.imgistyp == 2 || in->im.imspatyp == 1) {
						DEBUG1("%s one GIS/SPA recipient alone or in couple\n");
						in->im.imigist = nneg(MP.MANS - .50 * nfgisinc);
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
					}
					else {
						DEBUG1("%s two GIS/SPA recipients in couple\n");
						in->im.imigist = nneg(MP.MANC - .25 * nfgisinc);
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
					}
				}

				else if (in->id.idage >= 55) {
					DEBUG1("%s person (aged 55+) received no GIS/SPA\n");
					/* Special Manitoba top-ups  (MSP for non-GIS/SPA).  */
					if ((in->im.imgisinc * .50) <= (MP.PYINC * in->im.imipnst)) {
						DEBUG1("%s over 1/2 of income is pension\n");
						if (!in->id.idspoflg) {
						   in->im.imigist = nneg(MP.MANS - .50 * nneg(in->im.imgisinc - MP.MANSNPF));
							DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
						else if (in->id.idspoflg) {
							in->im.imigist = nneg(MP.MANC - .25 * nneg(nfgisinc - MP.MANCNPF));
							DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
					}
				}
				break;

			case PRSASK:
				/* Saskatchewan income plan */
				/* This program involves tax-backs that are tied to the reductions */
				/* in the Federal GIS & SPA payments */
				if (isnzero(fedben)) {
					switch (in->im.imgistyp) {
						case 1:       /* single pensioner receiving GIS */
								in->im.imigist = nneg(MP.SASKS - MP.SASKRR1 * (maxben - fedben));
							    if (in->im.imigist < MP.SASKMINS) in->im.imigist = ZERO;
							    DEBUG2("%s single pensioner with GIS, reduced by %.2f\n",
									in->im.imgismax - in->im.imigis);
								/* Announced in 2005, a temporary energy cost allowance for benefit 
						  		 recipients as long as they are not institutionalized. */
								if (in->im.imigist > 0 && in->id.idieflag == 0) {
									in->im.imigist += MP.SASKTECA;
								}
								break;
						case 2:       /* married pensioner receiving GIS */
								in->im.imigist = nneg(MP.SASKS - MP.SASKRR2 * (maxben - fedben));
							    if (in->im.imigist < MP.SASKMINS) in->im.imigist = ZERO;
							    DEBUG2("%s married pensioner with GIS, was reduced by %.2f\n",
									in->im.imgismax - in->im.imigis);
								/* Announced in 2005, a temporary energy cost allowance for benefit 
						  		 recipients as long as they are not institutionalized. */
								if (in->im.imigist > 0 && in->id.idieflag == 0) {
									in->im.imigist += MP.SASKTECA;
								}
							break;
						case 3:       /* two pensioners both receiving GIS */
								in->im.imigist = nneg(MP.SASKC - MP.SASKRR1 * (maxben - fedben));
							    if (in->im.imigist < MP.SASKMINC) in->im.imigist = ZERO;
							    DEBUG2("%s married pensioner, both with GIS, GIS was reduced by %.2f\n",
									in->im.imgismax - in->im.imigis);
								/* Announced in 2005, a temporary energy cost allowance for benefit 
						  		 recipients as long as they are not institutionalized. */
								if (in->im.imigist > 0 && in->id.idieflag == 0) {
									in->im.imigist += MP.SASKTECA;
								}
							break;
						case 4:       /* GIS/SPA family */
								partin = in->id.idinspo;
							    in->im.imigist = nneg(MP.SASKC - MP.SASKRR3 *
										(partin->im.imspamax - partin->im.imispa) );
							    if (in->im.imigist < MP.SASKMINC) in->im.imigist = ZERO;
							    DEBUG2("%s GIS/SPA Family, SPA was reduced by %.2f\n",
										(partin->im.imspamax - partin->im.imispa) );
								/* Announced in 2005, a temporary energy cost allowance for benefit 
						  		 recipients as long as they are not institutionalized. */
								if (in->im.imigist > 0 && in->id.idieflag == 0) {
									in->im.imigist += MP.SASKTECA;
								}
							break;
					}
					DEBUG2("%s gist = %.2f\n", in->im.imigist);
				}
				else {
					DEBUG1("%s did not receive GIS\n");
				}
				break;

			case PRALTA:

				switch (MP.ALTASOPT){ /* Alberta seniors have 2 options */
					case 1:  /* Alberta Assured Income Plan */
						if (isnzero(fedben)
						    || (isnzero(fedspa) && in->im.imspatyp == 1
	                        && ((MP.TARGETYEAR - in->id.idage) <= 1922))) {

							in->im.imigist = nneg(MP.ALTASC - .5 * (maxben - fedben));

							if (in->im.imigist < MP.ALTAMIN) {
								in->im.imigist = MP.ALTAMIN;
							}
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
						else {
							DEBUG1("%s did not receive GIS or SPA\n");
						}
					break;

					case 2: 	/* Alberta seniors benefit  1994 and beyond */

						/* find individuals over 65 in household*/
						/* seniors benefit only applies to persons aged 65 or more */
						if (in->id.idage>=65) {
						/* start by splitting seniors by type: single, one-senior couple, two
						   senior couple, and if renter or home owner */
							if (!in->id.idspoflg) {
								/* single and renter */
								/* also include institutionalized elderly as renters and */
								/*  give the supplementary accomodation benefit later */
								if (hh->hd.hdtenur == 1 || in->id.idieflag == 1) {
								    asbben2 = MP.ASBBASIC + MP.ASBRENT;
									asbrr2  = MP.ASBRR[0];
									oastype=0;
								}
								/* single and homeowner */
								else {
								    asbben2 = MP.ASBBASIC ;
									asbrr2  = MP.ASBRR[1];
									oastype=1;
								}
							}
							/* if differentiating between 1 senior couple and 2 senior couples
							   and if spouse is less than 65 */
							else if (MP.ASBSSOPT==1 && inspo->id.idage < 65) {
								/* one-senior couple and renter  and institutional elderly */
								if (hh->hd.hdtenur == 1 || in->id.idieflag == 1) {
								    asbben2 = MP.ASBBASIC + MP.ASBNSS + MP.ASBRENT;
									asbrr2  = MP.ASBRR[4];
									oastype=4;
								}
								/* one-senior couple and homeowner */
								else {
								    asbben2 = MP.ASBBASIC + MP.ASBNSS;
									asbrr2  = MP.ASBRR[5];
									oastype=5;
								}
							}
							else {  
								/* two-senior couple and renter  */
								if (hh->hd.hdtenur == 1) {
								    asbben2 = MP.ASBBASIC + MP.ASBSS + MP.ASBRENT;
									asbrr2  = MP.ASBRR[2];
									oastype=2;
								}
								/* two-senior couple and homeowner */
								else {
								    asbben2 = MP.ASBBASIC + MP.ASBSS;
									asbrr2  = MP.ASBRR[3];
									oastype=3;
								}
							}

							/* calculate income used in program.  It is total income (line 150 of tax guide)
							   minus OAS/GIS/SPA, CPP Death and Disabiliy, alimony received, worker's compensation,
							   social assistance, RRSP contributions, and 5% of employment.  since imitot already
							   exludes GIS/SPA, social assistance, and worker's compensation, it is not necessary
							   to deduct them.  Currently, CPP Death and disability are not included
							   as separate items in the model */

							asbfinc2 = MP.PYINC * (nneg(in->im.imitot 
							               - (MP.ASBEMP * in->id.idiemp)
										   - in->id.idialimo
							               - in->im.imioas 
							               - in->id.idrpp
							               - in->id.idrrsp));
							/* flag if not eligible for oas */
							if (in->im.imoasres==0) {
							   nooas=1;
							}

                            /* note: should also exclude CPP disability, CPP death and employment commission.
							   since imitot does not include worker's comp, sa, or the fed. sups., they
							   do not need to be deducted */
							if (in->id.idspoflg) {
								asbfinc2 += MP.PYINC * (nneg(inspo->im.imitot
							               - (MP.ASBEMP * inspo->id.idiemp)
										   - inspo->id.idialimo
							               - inspo->im.imioas 
							               - inspo->id.idrpp
							               - inspo->id.idrrsp ));

								/* flag if spouse not eligible for oas */
								/* note that some spouses under teh age of 65, the rule does not apply
								   to them */
							    if (inspo->im.imoasres==0 && inspo->id.idage>=65) {
							       spnooas=1;
							    }
								/* if both spouses not eligible for oas and if 
								   income is greater than 10000 then oas is deducted 
								   from income, and total benefit will later be reduced */
								if (nooas && spnooas) {
								     if (asbfinc2>10000) {
									     asbfinc2 = nneg(asbfinc2-(2*MP.BOAS));
									 }
								}
								else if (MP.ASBSSOPT==1 && (nooas || spnooas) && inspo->id.idage>=65) {
								/* if only one spouse is inelibible for oas, then treat as
								   a one-senior spouse, if modeling single senior spouse */
									/* one-senior couple and renter */
									if (hh->hd.hdtenur == 1) {
									    asbben2 = MP.ASBBASIC + MP.ASBNSS + MP.ASBRENT;
										asbrr2  = MP.ASBRR[4];
									}
									/* one-senior couple and homeowner */
									else {
									    asbben2 = MP.ASBBASIC + MP.ASBNSS;
										asbrr2  = MP.ASBRR[5];
									}
								}
							}
							/* if single, not eligible for oas, and with income greater than
							   10000, then oas is deducted from income and the total benefit will
							   later be reduced */
							else if (nooas && asbfinc2>10000) {
							    asbfinc2 = nneg(asbfinc2 - MP.BOAS);
							}
							/* assign basic benefit and reduction rate according to type of
							   senior and then tax back */
							taxbak1(asbfinc2, 0, &asbben2, asbrr2);

							/* if a single person did not receive OAS, or if both persons in couple
							   are not eligible for oas, then benefit is reduced */
							if (in->id.idspoflg && spnooas && nooas) {
								asbben2 = asbben2 * MP.ASBNOAS[oastype];
							}
							else if (!in->id.idspoflg && nooas) {
							    asbben2 = asbben2 * MP.ASBNOAS[oastype];
							}

							if (MP.ASBCYFLG) {
								/* start by splitting seniors by type: single, one-senior couple, two
							    	senior couple, and if renter or home owner */
								if (!in->id.idspoflg) {
									/* single and renter */
									/* also include institutionalized elderly as renters and */
									/*  give the supplementary accomodation benefit later */
									if (hh->hd.hdtenur == 1 || in->id.idieflag == 1) {
									    asbben1 = MP.ASBBASICP + MP.ASBRENTP;
										asbrr1  = MP.ASBRRP[0];
									}
									/* single and homeowner */
									else {
									    asbben1 = MP.ASBBASICP ;
										asbrr1  = MP.ASBRRP[1];
									}
								}
								/* if differentiating between 1 senior couple and 2 senior couples
								   and if spouse is less than 65 */
								else if (MP.ASBSSOPTP==1 && inspo->id.idage < 65) {
									/* one-senior couple and renter  and institutional elderly */
									if (hh->hd.hdtenur == 1 || in->id.idieflag == 1) {
									    asbben1 = MP.ASBBASICP + MP.ASBNSSP + MP.ASBRENTP;
										asbrr1  = MP.ASBRRP[4];
									}
									/* one-senior couple and homeowner */
									else {
									    asbben1 = MP.ASBBASICP + MP.ASBNSSP;
										asbrr1  = MP.ASBRRP[5];
									}
								}
								else {  
									/* two-senior couple and renter  */
									if (hh->hd.hdtenur == 1) {
									    asbben1 = MP.ASBBASICP + MP.ASBSSP + MP.ASBRENTP;
										asbrr1  = MP.ASBRRP[2];
									}
									/* two-senior couple and homeowner */
									else {
									    asbben1 = MP.ASBBASICP + MP.ASBSSP;
										asbrr1  = MP.ASBRRP[3];
									}
								}

								/* calculate income used in program.  It is total income (line 150 of tax guide)
								   minus OAS/GIS/SPA, CPP Death and Disabiliy, alimony received, worker's compensation,
								   social assistance, RRSP contributions, and 5% of employment.  since imitot already
								   exludes GIS/SPA, social assistance, and worker's compensation, it is not necessary
								   to deduct them.  Currently, CPP Death and disability are not included
								   as separate items in the model */

								asbfinc1 = MP.PYINCP * (nneg(in->im.imitot 
								               - (MP.ASBEMPP * in->id.idiemp)
											   - in->id.idialimo
								               - in->im.imioas 
								               - in->id.idrpp
								               - in->id.idrrsp ));

	                            /* note: should also exclude CPP disability, CPP death and employment commission.
								   since imitot does not include worker's comp, sa, or the fed. sups., they
								   do not need to be deducted */
								if (in->id.idspoflg) {
									asbfinc1 += MP.PYINCP * (nneg(inspo->im.imitot
								               - (MP.ASBEMPP * inspo->id.idiemp)
											   - inspo->id.idialimo
								               - inspo->im.imioas 
								               - inspo->id.idrpp
								               - inspo->id.idrrsp ));
									/* if both spouses not eligible for oas and if 
									   income is greater than 10000 then oas is deducted 
									   from income, and total benefit will later be reduced */
									if (nooas && spnooas) {
									     if (asbfinc1>10000) {
										     asbfinc1 = nneg(asbfinc1-(2*MP.ASBBOASP));
										 }
									}
									else if (MP.ASBSSOPTP==1 && (nooas || spnooas) && inspo->id.idage>=65) {
									/* if only one spouse is inelibible for oas, then treat as
									   a one-senior spouse, if modeling single senior spouse */
										/* one-senior couple and renter */
										if (hh->hd.hdtenur == 1) {
										    asbben1 = MP.ASBBASICP + MP.ASBNSSP + MP.ASBRENTP;
											asbrr1  = MP.ASBRRP[4];
										}
										/* one-senior couple and homeowner */
										else {
										    asbben1 = MP.ASBBASICP + MP.ASBNSSP;
											asbrr1  = MP.ASBRRP[5];
										}
									}
								}
								/* if single, not eligible for oas, and with income greater than
								   10000, then oas is deducted from income and the total benefit will
								   later be reduced */
								else if (nooas && asbfinc1>10000) {
								    asbfinc1 = nneg(asbfinc1 - MP.ASBBOASP);
								}
								/* assign basic benefit and reduction rate according to type of
								   senior and then tax back */
								taxbak1(asbfinc1, 0, &asbben1, asbrr1);

								/* if a single person did not receive OAS, or if both persons in couple
								   are not eligible for oas, then benefit is reduced */
								if (in->id.idspoflg && spnooas && nooas) {
									asbben1 = asbben1 * MP.ASBNOASP[oastype];
								}
								else if (!in->id.idspoflg && nooas) {
								    asbben1 = asbben1 * MP.ASBNOASP[oastype];
								}

								/* in calculate for calendar year loop */
								/*  take half of each calculation */

								asbben = (asbben1/2) + (asbben2/2);

							}
							else {
								/* Use the July to June calculation for the year */
								asbben = asbben2;
							}
							/* if two-senior couple, split benefit among both parties */
							if (in->id.idspoflg && inspo->id.idage>=65) {
								in->im.imiasb = asbben/2;
							}
							else {
								in->im.imiasb = asbben;
							}

							/* Now calculate the supplementary accomodation benefit */
							/* This extra benefit goes to seniors who reside in long-term */
							/* care and are eligible to receive the Alberta Senior Benefit */

							if (in->id.idieflag == 1 && isnzero(asbben) ) {
								asbsab2 = MP.ASBSAB;
								taxbak1(asbfinc2,0,&asbsab2,MP.ASBSABRR);
								
								if (MP.ASBCYFLG) {  /* calculate for the calendar year */
									asbsab1 = MP.ASBSABP;
									taxbak1(asbfinc1,0,&asbsab1,MP.ASBSABRRP);
									/* take half of each calculation for calendar year equivalent */
									asbsab = (asbsab1/2) + (asbsab2/2);
								}
								else {  /* Use the July to June amount for the year */
									asbsab = asbsab2;
								}
								 
								in->im.imiasb += asbsab;
							}
							in->im.imigist = in->im.imiasb;
							
						}
					break;
				}	
				break;

			case PRBC:
				/* British Columbia gain for seniors supplement */
				if (isnzero(fedben) || isnzero(fedspa)) {
					if (in->im.imgistyp == 1        /* single GIS */
					    &&  in->id.idage >=65 ) {  /* widow(er) SPA */
		 				DEBUG1("%s single rate\n");
						in->im.imigist = nneg(MP.BCS - .50 * nfgisinc);
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
                       
					}
					if (in->im.imgistyp == 2 ) {/* married one pensioner */	
						DEBUG1("%s married one pensioner\n");
						if (in->id.idage >=65 ) {
							in->im.imigist = nneg(MP.BCS - .50 *  nneg(nfgisinc-
							MP.BOAS-1.5*MP.GISRLM));
						    DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
					}
					if(in->im.imgistyp == 4){ /*married one 60-64*/
						if (in->id.idage >=65 )	{
						    in->im.imigist = nneg(MP.BCC - .25 *  nneg(nfgisinc-
							(MP.BOAS+MP.GISRLM)*0.5));
						    DEBUG2("%s igist=%.2f\n", in->im.imigist);
						}
					}
					if (in->im.imgistyp == 3 ) {
						DEBUG1("%s couple rate\n");
						in->im.imigist = nneg(MP.BCC - .25 * nfgisinc);
						DEBUG2("%s igist=%.2f\n", in->im.imigist);
					}
					/* do not assign gist to unidentified gistyp */
     		        if (in->im.imgistyp == 0) in->im.imigist = ZERO;
					DEBUG1("%s did not receive GIS or SPA\n");

				}
				else {
				/* do not assign gist to unidentified gistyp */
		        if (in->im.imgistyp == 0) in->im.imigist = ZERO;
					DEBUG1("%s did not receive GIS or SPA\n");
				}
				if (isnzero(in->im.imigist)) {
					in->im.imbcss = in->im.imigist;
				}


		}
	}

	DEBUG_OFF("gist");
	return;
}
