/***
*samod.cpp - Compute social assistance or guarantees
*
*$Id: samod.cpp 6.112 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/02/23 | Added clawback algorithms for calendar year calculation |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 06/12/21 | Zero out imisa since it is saved in cceopt              |
* | CH  | 06/05/11 | Added Ontario flow through, changed Alta flow through   |
* | LP  | 05/11/29 | Fixed ANCBSFT - was commented out but should be used    |
* | LP  | 05/08/10 | Revised clawbacks, no longer remove clawback from sa    |
* | LP  | 05/07/08 | Readded clawbacks for version 11, use idsamnths         |
* | LP  | 04/02/20 | Removed clawbacks                                       |
* | CH  | 03/05/22 | Changed clawbacks                                       |
* | LP  | 02/05/02 | Added SA clawback provincial schemes (SACLAWFLAG etc.)  |
* | SFG | 99/12/09 | Change fa->famod and sa->samod                          |
* | EGA | 99/05/25 | SATARGET by province                                    |
* | EGA | 99/03/25 | Add SAPFLAG and SATARGET for case selection of SA       |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   samod    [global]  Compute social assistance or guarantees
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ssamod[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,samod)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*samod - Compute social assistance or guarantees
*
*Synopsis:
*    void samod(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   isnzero  Is argument non-zero ?
*   iszero   Is argument zero ?
*   nneg     Change negative numbers to zero
*   minn     Find the minimum of two numbers
*
*Structure Members:
*   MP.ANCBSFT1     (i)  Alberta social assistance NCBS flow-through amount for first child
*   MP.ANCBSFT1P    (i)  Alberta SA NCBS flow-through amount for first child for 1st half of year
*   MP.ANCBSFT2     (i)  Alberta social assistance NCBS flow-through amount for second child
*   MP.ANCBSFT2P    (i)  Alberta SA NCBS flow-through amount for second child for 1st half of year
*   MP.ANCBSFT3     (i)  Alberta social assistance NCBS flow-through amount for third (or more) child
*   MP.ANCBSFT3P    (i)  Alberta SA NCBS flow-through amount for third (or more) child for 1st half of year
*   MP.ASHELTER     (i)  Alberta Shelter Increase by number of persons in household
*   MP.ASHELTERP    (i)  Alberta Shelter Increase by number of persons in household for 1st half of year
*   MP.ASHELTFLG    (i)  Alberta Shelter Increase under NCBS clawback flag
*   MP.ASHELTFLGP   (i)  Alberta Shelter Increase under NCBS clawback flag for 1st half of year
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.MCLAWPCT     (i)  Manitoba SA Clawback percentage kids over flow through age in families [#kids][#kids<age]
*   MP.MCLAWPCTP    (i)  Manitoba SA Clawback percentage kids over flow through age for 1st half of year [#kids][#kids<age]
*   MP.MCLAWSA12    (i)  Manitoba Social assistance NCBS clawback for children 12 and under
*   MP.MCLAWSA12P   (i)  Manitoba SA NCBS clawback for children 12 and under for 1st half of year
*   MP.MCLAWSA6     (i)  Manitoba Social assistance NCBS clawback for children 6 and under
*   MP.MCLAWSA6P    (i)  Manitoba SA NCBS clawback for children 6 and under for 1st half of year
*   MP.NCLAWSA1     (i)  Newfoundland social assistance clawback amount for first child
*   MP.NCLAWSA1P    (i)  Newfoundland SA clawback amount for first child for 1st half of year
*   MP.NCLAWSA2     (i)  Newfoundland social assistance clawback amount for each additional child
*   MP.NCLAWSA2P    (i)  Newfoundland SA clawback amount for each additional child for 1st half of year
*   MP.ONCBSFT1     (i)  Ontario social assistance NCBS flow-through amount for first child
*   MP.ONCBSFT1P    (i)  Ontario SA NCBS flow-through amount for first child for 1st half of year
*   MP.ONCBSFT2     (i)  Ontario social assistance NCBS flow-through amount for second child
*   MP.ONCBSFT2P    (i)  Ontario SA NCBS flow-through amount for second child for 1st half of year
*   MP.ONCBSFT3     (i)  Ontario social assistance NCBS flow-through amount for third (or more) child
*   MP.ONCBSFT3P    (i)  Ontario SA NCBS flow-through amount for third (or more) child for 1st half of year
*   MP.PNCBSFT      (i)  P.E.I. social assistance NCBS flow-through amount per child
*   MP.PNCBSFTP     (i)  P.E.I. SA NCBS flow-through amount per child for 1st half of year
*   MP.QNCBSFT1     (i)  Quebec social assistance NCBS flow-through amount for first child
*   MP.QNCBSFT1P    (i)  Quebec SA NCBS flow-through amount for first child for 1st half of year
*   MP.QNCBSFT2     (i)  Quebec social assistance NCBS flow-through amount for second child
*   MP.QNCBSFT2P    (i)  Quebec SA NCBS flow-through amount for second child for 1st half of year
*   MP.QNCBSFT3     (i)  Quebec social assistance NCBS flow-through amount for third (or more) child
*   MP.QNCBSFT3P    (i)  Quebec SA NCBS flow-through amount for third (or more) child for 1st half of year
*   MP.SACLAWFLAG   (i)  Social assistance NCBS clawback flag
*   MP.SACLAWPR     (i)  Social assistance clawback flag [prov]
*   MP.SAELDOPT     (i)  SA for elderly calculation method [1=~SA,2=GIST->~SA,3=diff]
*   MP.SAFLAG       (i)  Social assistance flag
*   MP.SAFS         (i)  Federal share of social assistance [prov]
*   MP.SAPFLAG      (i)  Social assistance predicted value flag
*   MP.SATARGET     (i)  Social assistance target for predicted SA
*   MP.SFAOUT       (i)  Proportion of social assistance to eliminate
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfnpers      (i)  Number of persons in census family
*   hd.hdprov       (i)  Province
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   id.idage        (i)  Age
*   id.idisa        (i)  Social assistance income
*   id.idsamths     (i)  Imputed number of months on SA
*   id.idsapred     (i)  Predicted amount of SA received
*   id.idsarank     (i)  Predicted ranked likelihood of getting SA
*   im.imfcbene     (i)  Total Federal Child Benefit Supplement
*   im.imigist      (i)  GIS provincial top-up
*   im.imscb        (i)  Saskatchewan Child Benefit
*   im.imfsa        (io) Federal social assistance
*   im.imisa        (io) Social assistance (or replacement program)
*   im.imashelt     (o)  Alberta shelter increase under NCBS clawback
*   im.impsa        (o)  Provincial social assistance
*   im.imsaclaw     (o)  Social Assistance NCB Clawback Amount
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void samod(
	P_hh hh
)
{
	register P_in in;
	P_cf cf;
	int prov;
	NUMBER saval;
	NUMBER fnctbs;	   /* calculated value of NCBS */
	NUMBER clawamt;	   /* amount of SA clawback */
	NUMBER clawamt1;   /* amount of SA clawback for 1st half of year */
	NUMBER clawamt2;   /* amount of SA clawback for 2nd half of year */
	NUMBER yngamt1;	   /* amount of SA clawback when <6 not clawed back for 1st half of year */
	NUMBER oldamt1;	   /* amount of SA clawback when <12 not clawed back for 1st half of year */
	NUMBER yngamt2;	   /* amount of SA clawback when <6 not clawed back for 2nd half of year */
	NUMBER oldamt2;	   /* amount of SA clawback when <12 not clawed back for 2nd half of year */

	NUMBER ftamt;	   /* SA flow through amount */
	NUMBER ftamt1;     /* SA flow through amount for 1st half of year */
	NUMBER ftamt2;     /* SA flow through amount for 2nd half of year */
	NUMBER scb;		   /* calculated value of Sask. child benefit */
	NUMBER ashelt1;    /* Alberta shelter increase for 1st half of year */
	NUMBER ashelt2;    /* Alberta shelter increase for 2nd half of year */
	int nch0_6;		   /* number of children up to age 6 */
	int nch0_12;       /* number of children up to age 12 */
	int nchfam;        /* number of children in the familiy for table lookup */
	register int ini;
	register int cfi;

	DEBUG_ON("samod");

	if (!MP.SAFLAG) {
		DEBUG1("%s social assistance turned off by SAFLAG\n");
		DEBUG_OFF("samod");
		return;
	}

	/* store hhlds province in a temporary */
	prov = hh->hd.hdprov;

	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

		fnctbs = ZERO;
		scb = ZERO;
		clawamt1 = ZERO;
		ftamt1 = ZERO;
		clawamt2 = ZERO;
		ftamt2 = ZERO;
		clawamt = ZERO;
		ftamt = ZERO;
		ashelt1 = ZERO;
		ashelt2 = ZERO;
		oldamt1 = ZERO;
		yngamt1 = ZERO;
		oldamt2 = ZERO;
		yngamt2 = ZERO;
		nch0_6 = 0;
		nch0_12 = 0;
		nchfam = 0;
		
		/* process persons in census family with kids and calculate family 
			level benefits for social assistance clawback schemes */
		if (cf->cfnkids > 0) {
			for (ini=0, in=cf->cfin; ini<cf->cfnpers; ini++, in++) {
				fnctbs += in->im.imfcbene;
				scb += in->im.imscb;
				if (in->id.idage < 7 ) {
					nch0_6++;
				}
				if (in->id.idage < 13 ) {
					nch0_12++;
				}
			}
		}

		/* process persons in census family */
		for (ini=0, in=cf->cfin; ini<cf->cfnpers; ini++, in++) {
			saval = ZERO;

			/* setting imisa to zero to start */
			in->im.imisa = ZERO;

			if (MP.SAPFLAG) {
			    if (in->id.idsarank/1000 < MP.SATARGET[hh->hd.hdprov]) saval =in->id.idsapred;
			}
			else saval = in->id.idisa;     
			     
			    DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
				         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

				if (isnzero(saval)) {
					if (in->id.idage >= 65) {
						switch (MP.SAELDOPT) {
							case 1:
								in->im.imisa = ZERO;
								break;
							case 2:
								in->im.imisa = ZERO;
								if (iszero(in->im.imigist)) {
									in->im.imisa = saval * (ONE - MP.SFAOUT);
								}
								break;
							case 3:
								in->im.imisa = nneg(saval - in->im.imigist);
								break;
						}
					}
					else {
						in->im.imisa = saval * (ONE - MP.SFAOUT);
					}

					/* split social assistance between federal and provincial govts */
					in->im.imfsa = saval * MP.SAFS[prov];
					in->im.impsa = saval - in->im.imfsa;
				}

				/* Claw Back the provincial Social Assistance if SACLAWFLAG set to 1. */

				if (MP.SACLAWFLAG == 1 && in->im.imisa>0 && fnctbs>0){
				
		
					if (MP.SACLAWPR[prov] == 1) {
						if ( prov == 0 ) {
							/* Newfoundland - SA amounts reduced by NCLAWSA1 for the first child */
							/* and NCLAWSA2 for each additional child */
							if (cf->cfnkids == 1 ) {
								clawamt1 = MP.NCLAWSA1 *  (in->id.idsamths / 12);
							}
							else {
								clawamt1 = ((MP.NCLAWSA2 * (cf->cfnkids - 1)) + MP.NCLAWSA1)
									         * (in->id.idsamths / 12);
							}
							if (MP.FCBENCAL) {	 /* calculate for the calendar year */ 
								if (cf->cfnkids == 1 ) {
									clawamt2 = MP.NCLAWSA1P * (in->id.idsamths/12);
								}
								else {
									clawamt2 = ((MP.NCLAWSA2P * (cf->cfnkids-1)) + MP.NCLAWSA1P)
												* (in->id.idsamths/12);
								}
								clawamt = (clawamt1/2) + (clawamt2/2);
							}
							else {    /* use the july to june calculation */
								clawamt = clawamt1;
							}
							in->im.imsaclaw = minn(clawamt, fnctbs);
						   /*	in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa;  */
						}
						if ( prov == 1 ) {
				  			/* P.E.I.- SA amounts reduced by NCBS amounts */
							clawamt = fnctbs;
							/* P.E.I. flowed through the 2000 increase to the NCBS */
							/* An amount per child is not clawed back */
							if ( cf->cfnkids > 0 ) {
							  	ftamt1 = MP.PNCBSFT * cf->cfnkids;
							}
							
							if (MP.FCBENCAL) { /* calculate for the calendar year */
								if (cf->cfnkids > 0 ) {
								   ftamt2 = MP.PNCBSFTP * cf->cfnkids;
								}
								ftamt = (ftamt1/2) + (ftamt2/2);
							}
							else { /* use the july to june calculation */
								 ftamt = ftamt1;
							}

							in->im.imsaclaw = nneg(clawamt - ftamt) * (in->id.idsamths / 12 );
							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa;  */
						}
						if ( prov == 2 ) {
							/* Nova Scotia  */
							/* SA amounts reduced by NCBS amounts */
							in->im.imsaclaw = fnctbs * (in->id.idsamths / 12 );
							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa; */
						}
						if ( prov == 4 ) {
							/* Quebec - SA amounts reduced by the NCBS  */
							clawamt = fnctbs ;
							/* Quebec flowed through the NCBS increases which followed 1998*/
                            /* Note that increases in QFA were also clawed back though this 
                               is not yet implemented in the model */
  							ftamt1 = MP.QNCBSFT1 + 
  							        + ( minn(nneg(cf->cfnkids-1),1) * MP.QNCBSFT2 ) 
  							        + ( nneg(cf->cfnkids-2) * MP.QNCBSFT3 );

							if ( MP.FCBENCAL ) {  /* calculate for the calendar year */
	  							ftamt2 = MP.QNCBSFT1P + 
	  							        + ( minn(nneg(cf->cfnkids-1),1) * MP.QNCBSFT2P ) 
	  							        + ( nneg(cf->cfnkids-2) * MP.QNCBSFT3P );
								ftamt = (ftamt1/2) + (ftamt2/2);
							}
							else { /* use the july to june calculation */
								ftamt = ftamt1;
							}
								
							in->im.imsaclaw = nneg(clawamt - ftamt) * (in->id.idsamths / 12);

							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa;  */
						}
						if ( prov == 5 ) {
							clawamt = fnctbs;
							/* Ontario started flowing through increases to the NCBS in July 2004 */
							if ( cf->cfnkids > 0 ) {
							  	ftamt1 = MP.ONCBSFT1 + 
  							        + ( minn(nneg(cf->cfnkids-1),1) * MP.ONCBSFT2 ) 
  							        + ( nneg(cf->cfnkids-2) * MP.ONCBSFT3 );

							}
							if ( MP.FCBENCAL ) { /* calculate for the calendar year */
								ftamt2 =  MP.ONCBSFT1P + 
	  							        + ( minn(nneg(cf->cfnkids-1),1) * MP.ONCBSFT2P ) 
	  							        + ( nneg(cf->cfnkids-2) * MP.ONCBSFT3P );
								ftamt = (ftamt1/2) + (ftamt2/2);
							}
							else { /* use the july to june calculation */
								ftamt = ftamt1;
							}

							in->im.imsaclaw = nneg(clawamt - ftamt) * (in->id.idsamths / 12 );
							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa;  */
						}
						if ( prov == 6 ) {
							/* Manitoba - SA amounts reduced by NCBS amounts */
							/* if MCLAWSA6 = 0.0 then there is no time in the year where kids
							                     under 6 are exempt from the clawback*/
							/* if MCLAWSA12 = 0.0 then there is no time in the year where kids
							                     under 12 are exempt from the clawback*/
							if (MP.MCLAWSA6==0.0 && MP.MCLAWSA12==0.0) {
								clawamt1 = fnctbs;
							}
							else {
								/* Determine the number of children under age 7 and under age 12 */
								
								if (nch0_6 > 8)  {
									nch0_6 = 8;
								}
								if (nch0_12 > 8 ) {
									 nch0_12 = 8;
								}
								if (cf->cfnkids > 10)  {
									nchfam = 10;
								}
								else {
									nchfam = cf->cfnkids;
								}
								/* Manitoba does not claw back NCBS benefits */
								/* Applicable to children 6 years of age and younger */
									 
								if ( MP.MCLAWSA6 > 0.0) {
									yngamt1 = MP.MCLAWSA6 * MP.MCLAWPCT[nchfam][nch0_6] * fnctbs;
								}
								/* Beginning in January 2003 Manitoba will not claw back NCBS benefits */
								/* Applicable to children 12 years of age and younger */

                                /* In January 2004, Manitoba will not claw back any NCBS benefits.
                                   Since these are half years, only take half */
								if ( MP.MCLAWSA12 > 0.0 ) {
									oldamt1 = MP.MCLAWSA12 * MP.MCLAWPCT[nchfam][nch0_12] * fnctbs;
								}
                                clawamt1=yngamt1 + oldamt1;

							} 
							if ( MP.FCBENCAL ) { /* calculate for the calendar year */
								if (MP.MCLAWSA6P==0.0 && MP.MCLAWSA12P==0.0) {
									clawamt2 = fnctbs;
								}
								else {
									/* Determine the number of children under age 7 and under age 12 */
									
									if (nch0_6 > 8)  {
										nch0_6 = 8;
									}
									if (nch0_12 > 8 ) {
										 nch0_12 = 8;
									}
									if (cf->cfnkids > 10)  {
										nchfam = 10;
									}
									else {
										nchfam = cf->cfnkids;
									}
									/* Manitoba does not claw back NCBS benefits */
									/* Applicable to children 6 years of age and younger */
										 
									if ( MP.MCLAWSA6P > 0.0) {
										yngamt2 = MP.MCLAWSA6P * MP.MCLAWPCTP[nchfam][nch0_6] * fnctbs;
									}
									/* Beginning in January 2003 Manitoba will not claw back NCBS benefits */
									/* Applicable to children 12 years of age and younger */

	                                /* In January 2004, Manitoba will not claw back any NCBS benefits.
	                                   Since these are half years, only take half */
									if ( MP.MCLAWSA12P > 0.0 ) {
										oldamt2 = MP.MCLAWSA12P * MP.MCLAWPCTP[nchfam][nch0_12] * fnctbs;
									}
	                                clawamt2=yngamt2 + oldamt2;

								}
								clawamt = (clawamt1/2) + (clawamt2/2);
							}
							else { /* use the july to june calculation */
								clawamt = clawamt1;
							}
							 
							in->im.imsaclaw = clawamt * (in->id.idsamths / 12 );
							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa; */
				  		}
						if ( prov == 7 ) {
							/* Saskatchewan - SA amounts reduced by the NCBS and SCB amounts */
							in->im.imsaclaw = (fnctbs + scb) * (in->id.idsamths / 12) ;
							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							in->im.imisa = in->im.impsa; */
						}
						if ( prov == 8 ) {
							/* Alberta flowed through the 2003 increase to the NCBS */
							/* An amount per child is not clawed back */
						   	ftamt1 =  MP.ANCBSFT1 + 
	  							        + ( minn(nneg(cf->cfnkids-1),1) * MP.ANCBSFT2 ) 
	  							        + ( nneg(cf->cfnkids-2) * MP.ANCBSFT3 );
							
							if (MP.FCBENCAL) {
								ftamt2 =  MP.ANCBSFT1P + 
	  							        + ( minn(nneg(cf->cfnkids-1),1) * MP.ANCBSFT2P ) 
	  							        + ( nneg(cf->cfnkids-2) * MP.ANCBSFT3P );
								ftamt = (ftamt1/2) + (ftamt2/2);
							}
							else { /* use the july to june calculation */
								ftamt = ftamt1;
							}

							/* Alberta SA amounts reduced by NCBS amounts minus flow through*/
							in->im.imsaclaw = nneg(fnctbs-ftamt) * (in->id.idsamths / 12 );

							/* in->im.impsa = nneg(in->im.imisa - clawamt);
							   in->im.imisa = in->im.impsa;  */

							/* No increase to shelter allowances */
						  	if (MP.ASHELTFLG == 0) {
								ashelt1 = ZERO;
							}
							else {	/* Give a monthly shelter increase */
								ashelt1 = MP.ASHELTER[cf->cfnpers] * in->id.idsamths;
							}
							if (MP.FCBENCAL) { /* calculate for the calendar year */
								if (MP.ASHELTFLGP == 0 ) {
									ashelt2 = 0;
								}
								else {
									ashelt2 = MP.ASHELTERP[cf->cfnpers] * in->id.idsamths;
								}
								in->im.imashelt = (ashelt1/2) + (ashelt2/2);
							}
							else { /* use the july to june calculation */
								in->im.imashelt = ashelt1;
							}	  
						}
					}
				}

			DEBUG2("%s recipients's isa=%.2f\n", in->im.imisa);
	 		/* indicate that person has income */

		   
		}
	}
	DEBUG_OFF("samod");
}

