/***
*txctc.cpp - Compute child tax credit
*
*$Id: txctc.cpp 6.114 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | CH  | 06/10/16 | Minor bug fix to imuccb inclusion                       |
* | LP  | 06/06/20 | Added imbcss (BC sen supp as part of sa) to income tests|
* | LP  | 06/06/12 | Added phase in rates for Federal Child Tax Benefit      |
* | LP  | 06/06/05 | Revised the application of PYINCP                       |
* | LP  | 06/06/02 | Added FCBUCCB, UCCB interaction with NCB, adjust net inc|
* | LP  | 06/05/23 | Added code and parameters for federal child benefits to |
* |     |          | be calculated on a calendar year basis                  |
* | LP  | 04/02/18 | Added idiworkc to net income concepts                   |
* | LP  | 01/03/12 | Removed pre-1988 program parameter CTCIFLAG             |
* | CH  | 00/03/30 | Added flag for federal calculation of Quebec benefit    |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/03/03 | Added code to allow no phase in for WIS                 |
* | CH  | 97/09/16 | Changed WIS enrichment                                  |
* | CH  | 97/03/04 | Added WIS enrichment for 1997, fixed Quebec 3rd & subs  |
* | BBM | 92/07/14 | Add CTC fudge parameters CTCTUR CTCINC                  |
* | BBM | 92/07/07 | Federal Child Benefit program implemented               |
* | SFG | 91/05/29 | Don't give ctc to non-child teens (pre-reform)          |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txctc    [global]  Compute child tax credit
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxctc[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txctc)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txctc - Compute child tax credit
*
*Synopsis:
*    void txctc(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   taxbak1  One level tax-back function
*   lkup2    Table look-up without interpolation
*   minn     Find the minimum of two numbers
*   isnzero  Is argument non-zero ?
*
*Structure Members:
*   MP.ACBC1        (i)  Alberta Child benefit per child aged 0 - 6
*   MP.ACBC1P       (i)  Alberta Child benefit per child aged 0 - 6 for 1st half of year
*   MP.ACBC2        (i)  Alberta Child benefit per child aged 7 - 11
*   MP.ACBC2P       (i)  Alberta Child benefit per child aged 7 - 11 for 1st half of year
*   MP.ACBC3        (i)  Alberta Child benefit per child aged 12- 15
*   MP.ACBC3P       (i)  Alberta Child benefit per child aged 12- 15 for 1st half of year
*   MP.ACBC4        (i)  Alberta Child benefit per child aged 16- 17
*   MP.ACBC4P       (i)  Alberta Child benefit per child aged 16- 17 for 1st half of year
*   MP.CTCERF       (i)  Refundable CTC child care expense reduction fraction
*   MP.CTCINC       (i)  Refundable CTC family income scaling factor
*   MP.CTCOPT       (i)  Refundable Child tax credit option
*   MP.CTCPC        (i)  Refundable Child tax credit per child
*   MP.CTCREF       (i)  Refundable CTC post-reform rules flag
*   MP.CTCRR        (i)  Refundable Child tax credit reduction rate
*   MP.CTCSUP       (i)  Refundable CTC young child supplement
*   MP.CTCTD        (i)  Family income refundable child tax credit turndown
*   MP.CTCTUR       (i)  Refundable CTC take up rate table [benefit,rate]
*   MP.CTCTURrows   (i)  number of rows in CTCTUR
*   MP.FCBBAS       (i)  Basic child benefit (per child)
*   MP.FCBBASP      (i)  Basic child benefit (per child) for 1st half of year
*   MP.FCBECI       (i)  Child benefits earning supplement cut-in level
*   MP.FCBECIP      (i)  Child benefits earning supplement cut-in level for 1st half of year
*   MP.FCBEIE       (i)  Flag for WIS dependent on number of children
*   MP.FCBEIEP      (i)  Flag for WIS dependent on number of children for 1st half of year
*   MP.FCBEIS       (i)  Child benefits earned income supplement
*   MP.FCBEIS1      (i)  WIS/NCBS for first child
*   MP.FCBEIS1P     (i)  WIS/NCBS for first child for 1st half of year
*   MP.FCBEIS2      (i)  WIS/NCBS for second child
*   MP.FCBEIS2P     (i)  WIS/NCBS for second child for 1st half of year
*   MP.FCBEIS3      (i)  WIS/NCBS for each additional child
*   MP.FCBEIS3P     (i)  WIS/NCBS for each additional child for 1st half of year
*   MP.FCBEISP      (i)  Child benefits earned income supplement for 1st half of year
*   MP.FCBEMX       (i)  Earnings where WIS phase-in is at maximum
*   MP.FCBEMXP      (i)  Earnings where WIS phase-in is at maximum for 1st half of year
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.FCBERR       (i)  Child benefits earning supplement reduction rate
*   MP.FCBERR1      (i)  WIS/NCBS reduction rate for families with 1 child
*   MP.FCBERR1P     (i)  WIS/NCBS reduction rate for families with 1 child for 1st half of year
*   MP.FCBERR2      (i)  WIS/NCBS reduction rate for families with 2 children
*   MP.FCBERR2P     (i)  WIS/NCBS reduction rate for families with 2 children for 1st half of year
*   MP.FCBERR3      (i)  WIS/NCBS reduction rate for families with 3+ children
*   MP.FCBERR3P     (i)  WIS/NCBS reduction rate for families with 3+ children for 1st half of year
*   MP.FCBERRP      (i)  Child benefits earning supplement reduction rate for 1st half of year
*   MP.FCBESR       (i)  Child benefits earned income supplement rate
*   MP.FCBESRP      (i)  Child benefits earned income supplement rate for 1st half of year
*   MP.FCBETD       (i)  WIS/NCBS turndown level
*   MP.FCBETDP      (i)  WIS/NCBS turndown level for 1st half of year
*   MP.FCBLRG       (i)  Supplement for 3rd and subsequent kids
*   MP.FCBLRGP      (i)  Supplement for 3rd and subsequent kids for 1st half of year
*   MP.FCBPI        (i)  Federal Child Tax Benefit phase-in rate
*   MP.FCBPIP       (i)  Federal Child Tax Benefit phase-in rate for 2nd half of year
*   MP.FCBRR        (i)  Multi-children family income reduction rate
*   MP.FCBRRP       (i)  Multi-children family income reduction rate for 1st half of year
*   MP.FCBRRS       (i)  Single-child family income reduction rate
*   MP.FCBRRSP      (i)  Single-child family income reduction rate for 1st half of year
*   MP.FCBTD        (i)  Federal child benefits family income turndown
*   MP.FCBTDP       (i)  Federal child benefits family income turndown for 1st half of year
*   MP.FCBUCCB      (i)  Flag for Supplement for children age 6
*   MP.FCBUCCBP     (i)  Flag for Supplement for children age 6 for 1st half of year
*   MP.FCBYNG       (i)  Supplement for children under age 7
*   MP.FCBYNGP      (i)  Supplement for children under age 7 for 1st half of year
*   MP.FCBYNGR      (i)  Child care expense reduction rate
*   MP.FCBYNGRP     (i)  Child care expense reduction rate for 1st half of year
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   MP.QCBC1        (i)  Quebec Child benefit for 1st child
*   MP.QCBC1P       (i)  Quebec Child benefit for 1st child for 1st half of year
*   MP.QCBC2        (i)  Quebec Child benefit for 2nd child
*   MP.QCBC2P       (i)  Quebec Child benefit for 2nd child for 1st half of year
*   MP.QCBC3        (i)  Quebec Child benefit for 3rd child
*   MP.QCBC3P       (i)  Quebec Child benefit for 3rd child for 1st half of year
*   MP.QCBCFLG      (i)  Quebec Child benefit flag for Quebec calculation methodology
*   MP.QCBCFLGP     (i)  Quebec Child benefit flag for Quebec calculation methodology for 1st half of year
*   MP.QCBCS        (i)  Quebec Child benefit supplement child aged 12- 17
*   MP.QCBCSP       (i)  Quebec Child benefit supplement child aged 12- 17 for 1st half of year
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfspoflg     (i)  Census family contains married couple
*   hd.hdprov       (i)  Province
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idiemp       (i)  Wages & salaries
*   id.idise        (i)  Self-employed income
*   id.idrand       (i)  Random numbers [array]
*   id.idsex        (i)  Sex
*   im.imccec       (i)  Child care expenses claimed on behalf of child
*   im.imicnet      (i)  Net Income (line 236)
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   nf.nfinch       (i)  First child in nuclear family [pointer]
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imctc        (o)  Child tax credit
*   im.imfcben      (o)  Total Federal Child Benefits
*   im.imfcbenb     (o)  Total Federal Child Benefits Base
*   im.imfcbene     (o)  Total Federal Child Benefit Supplement
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void txctc(
	P_hh hh
)
{
	register P_in ineld;
	register P_in inspo;
	register P_cf cf;
	register int cfi;
	register	P_nf nf;
	register int nfi;
	int ini;
	P_in in;   
	int nch0_6;
	int nch0_17;
	int nch0_18;
	NUMBER faminet = ZERO;
	NUMBER ctc = ZERO;
	NUMBER ccec = ZERO;
	NUMBER cben = ZERO;
	NUMBER cbeneis = ZERO;
	NUMBER cben1 = ZERO;
	NUMBER cbeneis1 = ZERO;
	NUMBER cben2 = ZERO;
	NUMBER cbeneis2 = ZERO;
	NUMBER nfinet = ZERO;
	NUMBER nfiemp = ZERO;
	NUMBER ccex = ZERO;
	NUMBER eismax = ZERO;
	NUMBER eispirr = ZERO; 
	NUMBER eisporr = ZERO;

	DEBUG_ON("txctc");

	if (MP.CTCOPT==1 || MP.CTCOPT == 3) {
		DEBUG1("%s child tax credit system in effect\n");

      /* process each census family in household */
	   for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {

		   if (cf->cfnchild == 0) {
			   DEBUG1("%s skipping census family with no `children'\n");
			   continue;
		   }

		   /* initialise elder's pointer */
		   ineld = cf->cfineld;

		   /* calculate elder's contribution to family net income */
		   faminet = nneg(ineld->im.imicnet - ineld->im.imuccb);


		   if (cf->cfspoflg) {
			   DEBUG1("%s spouse present\n");
			   inspo = cf->cfinspo;    /* spouse's in pointer */

			   /* add spouse's net income to family net income */
			   faminet += nneg(inspo->im.imicnet - inspo->im.imuccb);
		   }

         /* Scale family net income with user specified rates */
         faminet = faminet * MP.CTCINC;
       
		   /* do other stuff if reform option activated */
		   if (MP.CTCREF) {
			   nch0_6 = 0;
			   nch0_18 = 0;
			   ccec = ZERO;
			   ctc = ZERO;
			   /* process children in census family */
			   for (ini=0, in=cf->cfinch;  ini<cf->cfnchild;  in++) {
				   ini++;

				   if (in->id.idage >= 19) {
					   DEBUG2("%s discarding old child, aged %d\n", in->id.idage);
					   continue;
				   }

				   /* Count up remaining children */
				   nch0_18++;

				   /**
				   * Count up little kids (for supplement)
				   * and their associated child care expenses.
				   **/
				   if (in->id.idage <= 6) {
					   nch0_6++;
					   ccec += in->im.imccec;
				   }
			   }

			   /* compute supplement for young children */
			   /* subtract CTCERF of CCEA claimed on behalf of young children */
			   if (nch0_6) {
				   ctc = nneg((nch0_6 * MP.CTCSUP) - MP.CTCERF * ccec);
			   }

			   /* add basic child tax credit for children */
			   ctc += nch0_18 * MP.CTCPC;
		   }

		   else /* MP.CTCREF ==0 (non-reform) */ {
			   nch0_17 = 0;
			   /* calculate child tax credit */
			   /* process children in census family */
			   for (ini=0, in=cf->cfinch;  ini<cf->cfnchild;  in++,ini++) {

				   if (in->id.idage >= 18) {
					   DEBUG2("%s discarding old child, aged %d\n", in->id.idage);
					   continue;
				   }

				   /* Count up remaining children */
				   nch0_17++;
			   }

			   ctc = nch0_17 * MP.CTCPC;
		   }

		   /* tax back credit (ctc) at reduction rate CTCRR on faminet */
		   /* in excess of CTCTD                                       */

		   taxbak1(faminet, MP.CTCTD, &ctc, MP.CTCRR);

		   DEBUG4("%s nkids=%d, faminet=%.2f ctc=%.2f\n", cf->cfnkids, faminet, ctc);

         /**
			* Apply takeup rates
			**/
			if (ineld->id.idrand[2] > (NUMBER) lkup2(MP.CTCTUR, MP.CTCTURrows, (double) ctc)) {
				DEBUG1("%s failed random take-up test\n");
            ctc = ZERO;
			}

         /* assign CTC to mother if present */
		   if (cf->cfspoflg && (inspo->id.idsex == FEMALE)) {
			   DEBUG1("%s spouse is the mother, gets CTC\n");
			   inspo->im.imctc = ctc;
			   DEBUG2("%s ctc=%.2f\n", inspo->im.imctc);

		   }
		   else {
			   DEBUG1("%s elder receives CTC\n");
			   ineld->im.imctc = ctc;
			   DEBUG2("%s ctc=%.2f\n", ineld->im.imctc);

		   }

	   }
	}


   /**
    * FEDERAL CHILD BENEFIT PROGRAM ANNOUNCED IN 1992 BUDGET
    **/

   if (MP.CTCOPT==2 || MP.CTCOPT == 3) {
		DEBUG1("%s child benefit program in effect\n");

       /* process each nuclear family in household */
	   for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
		   DEBUG4("%s cben: nf %d spoflg=%d, nkids=%d\n",
		    	nfi, nf->nfspoflg, nf->nfnkids);

           if (nf->nfnkids == 0) {
			   DEBUG1("%s cben: skipping nuclear family with no `children'\n");
			   continue;
		   }

           /* Initialize local variables and head and spouse pointers */
		   nch0_6 = 0;
		   ineld = nf->nfineld;
		   inspo = nf->nfinspo;

           /* Compute Family net income and family employment income */
			 nfinet = MP.PYINC * nneg( ineld->im.imicnet - ineld->im.imuccb);
          	 nfiemp = MP.PYINC *
				    (ineld->id.idiemp + ineld->id.idise );

           if (nf->nfspoflg) {
			  nfinet += MP.PYINC * nneg( inspo->im.imicnet   - inspo->im.imuccb);
	          nfiemp += MP.PYINC *
						  (inspo->id.idiemp + inspo->id.idise);
           }

           /* Scale incomes with user specified rates */
           nfinet = nfinet * MP.CTCINC;
           nfiemp = nfiemp * MP.CTCINC;

		  DEBUG3("%s cben: neti=%.2f empi=%.2f\n", nfinet, nfiemp);


		  /* initialize base benefit*/
		  cben2 = ZERO;

		  /* Derive base amount which is dependent on province */
		  switch (hh->hd.hdprov) {
			  case PRQUE:
                  /* When the QCBCFLG is set, Quebec has a specialized child tax 
                     benefit system in place which takes into account a child's parity */
			  	 if (MP.QCBCFLG) {
					 /* Loop through kids to sort into parity groups */
					 for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
					    if (ini == 0) cben2 += MP.QCBC1; /* 1st child */
					    else if (ini == 1) cben2 += MP.QCBC2; /* 2nd child */
					    else   cben2 += MP.QCBC3; /* 3rd and subsequent child  */
					    /* supplement for ages 12 to 17 */
					    if ((in->id.idage >= 12) && (in->id.idage <= 17)) cben2 += MP.QCBCS;
					 }
                 }
                 /* The benefit is calculated in the same way as the federal benefit */
                 else {
        		     cben2 = MP.FCBBAS * nf->nfnkids;
                 }
				 break;

			  case PRALTA:
					/* Loop through kids to sort by age group */
					for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
						if (in->id.idage < 7) cben2 += MP.ACBC1;		/* age 0-6*/
						else if (in->id.idage < 12) cben2 += MP.ACBC2;/* age 7-11*/
						else if (in->id.idage < 16) cben2 += MP.ACBC3;/* age 12-15*/
						else if (in->id.idage <= 17) cben2 += MP.ACBC4;/* age 16-17*/
					}
					break;

				default:
		         cben2 = MP.FCBBAS * nf->nfnkids;
					break;
			}
			/**
            * process kids in nuclear family
			* Count the number of children who are under age seven and
			* count the total amount of child care expenses in the household.
			**/

			for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
				DEBUG4("%s cben kid %d: age=%d, cex=%.2f\n",
							ini, in->id.idage, in->im.imccec);
				if (MP.FCBUCCB) { /* phase out of supplement with UCCB */
					if (in->id.idage == 6) {
						nch0_6 ++;
					}
				}
				else { 
            		if (in->id.idage < 7) { 
						DEBUG1("%s cben: increment young kids counter\n");
            			nch0_6 ++;
					}
				}
				ccex += in->im.imccec;
			}



         	/** Assign young child, and large family benefits **/
			/* apply young child amount reduced by portion of child care expense */
         	cben2 += nneg((MP.FCBYNG * nch0_6) - (MP.FCBYNGR * ccex));
			DEBUG2("%s cben: basic cben w/cce top-up  cben=%.2f\n", cben);

         	if (nf->nfnkids > 2) { 
            	cben2 += MP.FCBLRG * (nf->nfnkids-2);
				DEBUG2("%s cben: increment for 3+ kids  cben=%.2f\n", cben2);
         	}


         	/***
         	** Tax Back basic, young child, and large family benefits
         	** providing a different(reduced) tax rate for
         	** single children families
         	***/
         	if (nf->nfnkids==1) {
   		   		taxbak1(nfinet, MP.FCBTD, &cben2, MP.FCBRRS);
         	}
         	else {
	   	   		taxbak1(nfinet, MP.FCBTD, &cben2, MP.FCBRR);
         	}

			DEBUG4("%s cben: cben after reduction cben=%.2f neti=%.2f empi=%.2f\n",
					cben2, nfinet, nfiemp);

         	/***
         	** calc gross earned income supplement
         	** then use taxbak to reduce the benefit 
         	** for families with higher net incomes
         	***/

         	/***
         	** IMPORTANT NOTE: The British Columbia Family Benefit program
         	** was amended in 1998 to represent a combined B.C. Family Benefit/	
		 	** Federal Earned Income Supplement payment
		 	** The formula according to the regulations includes the same 
		 	** parameters in the calculation of the Federal Earned Income Supplement
		 	**  for 1998.  The formula for subsequent years should be verified to ensure
		 	**  the proper calculation and parameters for the B.C. Family Benefit program
         	***/


			if ( nfiemp > MP.FCBECI || MP.FCBECI==0) {
			
			   if (MP.FCBEIE == 0) {
			       /* original Working Income Supplement */
				   eismax = MP.FCBEIS ;
			  	   DEBUG1("%s cben: earned inc benefit calc\n");

         	       cbeneis2 = minn( eismax,  MP.FCBESR * (nfiemp - MP.FCBECI) );
				   DEBUG2("%s cben: gross earned inc benefit cbeneis=%.2f\n",
						cbeneis2);

				   if (isnzero(cbeneis2)) {
   		   	            taxbak1(nfinet, MP.FCBETD, &cbeneis2, MP.FCBERR);
                   
				        DEBUG2("%s cben: reduced earned inc benefit cbeneis=%.2f\n",
						cbeneis2);
			       }
			 
			   }

               else {
			       /* different benefits according to number of children */
			       eismax =  (MP.FCBEIS1) + (MP.FCBEIS2 * minn(nneg(nf->nfnkids-1),1)) +
			                 (MP.FCBEIS3 * nneg(nf->nfnkids-2));

				   if (MP.FCBECI > 0) {
					   /* the phase-in rate now depends on number of children */
					   eispirr  =  eismax / (MP.FCBEMX - MP.FCBECI);
    				   cbeneis2 = minn( eismax, eispirr * (nfiemp - MP.FCBECI));
				   }
				   else {
					   /* no phase in */
    				   cbeneis2 = eismax;
				   }

				   /* the phase-out rate also depends on number of children, up to 3 */
                   if (nf->nfnkids == 1) eisporr = MP.FCBERR1;
				   else if (nf->nfnkids == 2) eisporr = MP.FCBERR2;
				   else if (nf->nfnkids > 2) eisporr = MP.FCBERR3;

				   if (isnzero(cbeneis2)) {
   		   	            taxbak1(nfinet, MP.FCBETD, &cbeneis2, eisporr);
                   
				        DEBUG2("%s cben: reduced earned inc benefit cbeneis=%.2f\n",
						cbeneis2);
			       }
			   }
			}
			else {
			   DEBUG1("%s cben: earned inc to low for supplement\n");
			   cbeneis2 = ZERO;
			}

			
 		   /* While still in nuclear family loop, 
	  	   /* Enhancement to the Federal Child Tax Benefit calculation 
	       now includes a flag to allow users to calculate the benefit 
		   as per the calendar year, using the parameter values for the first half
		   and parameter values for the last half of year */

	      if (MP.FCBENCAL) {
			
			 /*  First need to adjust net and employment income for the year
			     that is 2 years prior to current year. 
			 */

             nfinet = ( nfinet / MP.PYINC ) * MP.PYINCP;
             nfiemp = ( nfiemp / MP.PYINC ) * MP.PYINCP;

			 /* initialize base benefit and local variables */
		  	 cben1 = ZERO;
			 eismax = ZERO;
			 eispirr = ZERO;
			 eisporr = ZERO;
			 nch0_6 = 0;
			 ccex = ZERO;

			 /* Derive base amount which is dependent on province */
			 switch (hh->hd.hdprov) {
			   	case PRQUE:
	                 /* When the QCBCFLG is set, Quebec has a specialized child tax 
	                  benefit system in place which takes into account a child's parity */
					if (MP.QCBCFLGP) {
					   /* Loop through kids to sort into parity groups */
					   for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
					       if (ini == 0) cben1 += MP.QCBC1P; /* 1st child */
					    	    else if (ini == 1) cben1 += MP.QCBC2P; /* 2nd child */
							    else   cben1 += MP.QCBC3P; /* 3rd and subsequent child  */
							    /* supplement for ages 12 to 17 */
						   if ((in->id.idage >= 12) && (in->id.idage <= 17)) cben1 += MP.QCBCSP;
					   }
	                }
	                     /* The benefit is calculated in the same way as the federal benefit */
	                else {
	        		    cben1 = MP.FCBBASP * nf->nfnkids;
	                }
					break;

			 	case PRALTA:
			 		/* Loop through kids to sort by age group */
			 		for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
			 			if (in->id.idage < 7) cben1 += MP.ACBC1P;		/* age 0-6*/
							else if (in->id.idage < 12) cben1 += MP.ACBC2P; /* age 7-11*/
							else if (in->id.idage < 16) cben1 += MP.ACBC3P; /* age 12-15*/
							else if (in->id.idage <= 17) cben1 += MP.ACBC4P; /* age 16-17*/
					}
					break;

				default:
			        cben1 = MP.FCBBASP * nf->nfnkids;
					break;
			 }
		     /**
             * process kids in nuclear family
			 * Count the number of children who are under age seven and
			 * count the total amount of child care expenses in the household.
			 **/

			 for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
			     DEBUG4("%s cben kid %d: age=%d, cex=%.2f\n",
							ini, in->id.idage, in->im.imccec);
				 if (MP.FCBUCCBP) { /* phase out of supplement with UCCB */
					 if (in->id.idage == 6) {
						nch0_6 ++;
					 }
				 }
				 else { 
            		 if (in->id.idage < 7) { 
						DEBUG1("%s cben: increment young kids counter\n");
            			nch0_6 ++;
					 }
				 }
				ccex += in->im.imccec;
			 }
			
	         /** Assign young child, and large family benefits **/
				/* apply young child amount reduced by portion of child care expense */
	         cben1 += nneg((MP.FCBYNGP * nch0_6) - (MP.FCBYNGRP * ccex));
				DEBUG2("%s cben: basic cben w/cce top-up  cben=%.2f\n", cben1);

	         if (nf->nfnkids > 2) { 
	            cben1 += MP.FCBLRGP * (nf->nfnkids-2);
					DEBUG2("%s cben: increment for 3+ kids  cben=%.2f\n", cben1);
	         }

	         /***
	         ** Tax Back basic, young child, and large family benefits
	         ** providing a different(reduced) tax rate for
	         ** single children families
	         ***/

	         if (nf->nfnkids==1) {
	   		   taxbak1(nfinet, MP.FCBTDP, &cben1, MP.FCBRRSP);
	         }
	         else {
		   	   taxbak1(nfinet, MP.FCBTDP, &cben1, MP.FCBRRP);
	         }

			 DEBUG4("%s cben: cben after reduction cben=%.2f neti=%.2f empi=%.2f\n",
						cben1, nfinet, nfiemp);

	         /***
	         ** calc gross earned income supplement
	         ** then use taxbak to reduce the benefit 
	         ** for families with higher net incomes
	         ***/

	         /***
	         ** IMPORTANT NOTE: The British Columbia Family Benefit program
	         ** was amended in 1998 to represent a combined B.C. Family Benefit/	
			 ** Federal Earned Income Supplement payment
			 ** The formula according to the regulations includes the same 
			 ** parameters in the calculation of the Federal Earned Income Supplement
			 **  for 1998.  The formula for subsequent years should be verified to ensure
			 **  the proper calculation and parameters for the B.C. Family Benefit program
	         ***/

			 if ( nfiemp > MP.FCBECIP || MP.FCBECIP==0) {
				
				if (MP.FCBEIEP == 0) {
				   /* original Working Income Supplement */
				   eismax = MP.FCBEISP ;
				   DEBUG1("%s cben: earned inc benefit calc\n");

	         	   cbeneis1 = minn( eismax,  MP.FCBESRP * (nfiemp - MP.FCBECIP) );
				   DEBUG2("%s cben: gross earned inc benefit cbeneis=%.2f\n",
							cbeneis1);

				   if (isnzero(cbeneis1)) {
	   		   	      taxbak1(nfinet, MP.FCBETDP, &cbeneis1, MP.FCBERRP);
	                   
				      DEBUG2("%s cben: reduced earned inc benefit cbeneis=%.2f\n",
							cbeneis1);
				   }
				 
				}

	            else {
				       /* different benefits according to number of children */
				    eismax =  (MP.FCBEIS1P) + (MP.FCBEIS2P * minn(nneg(nf->nfnkids-1),1)) +
				              (MP.FCBEIS3P * nneg(nf->nfnkids-2));

				    if (MP.FCBECIP > 0) {
				   	   /* the phase-in rate now depends on number of children */
				   	   eispirr  =  eismax / (MP.FCBEMXP - MP.FCBECIP);
	    		   	   cbeneis1 = minn( eismax, eispirr * (nfiemp - MP.FCBECIP));
				    }
				    else {
				   	   /* no phase in */
	    		   	   cbeneis1 = eismax;
				    }

				    /* the phase-out rate also depends on number of children, up to 3 */
	                if (nf->nfnkids == 1) eisporr = MP.FCBERR1P;
					   else if (nf->nfnkids == 2) eisporr = MP.FCBERR2P;
					   else if (nf->nfnkids > 2) eisporr = MP.FCBERR3P;

				    if (isnzero(cbeneis1)) {
	   		   	         taxbak1(nfinet, MP.FCBETDP, &cbeneis1, eisporr);
	                   
				         DEBUG2("%s cben: reduced earned inc benefit cbeneis=%.2f\n",
							cbeneis1);
				    }
				}
			 }
			 else {
			    DEBUG1("%s cben: earned inc to low for supplement\n");
			    cbeneis1 = ZERO;
			 }

			 /* if modeling calendar year, take half of each calculation */
			 cben = (cben1/2) + ((cben2/2) * MP.FCBPIP);
			 cbeneis = (cbeneis1/2) + ((cbeneis2/2) * MP.FCBPIP);
 		  }

		  else {
		    	/* using the July to June amounts for entire year */
			  cben = cben2 * MP.FCBPI;
			  cbeneis = cbeneis2 * MP.FCBPI;
		  }
	      /* assign child benefits to mother if present */

	      if (nf->nfspoflg && (inspo->id.idsex == FEMALE)) {
		      DEBUG1("%s cben: spouse is the mother, gets CTC\n");
		      inspo->im.imfcbenb = cben;
		      inspo->im.imfcbene = cbeneis;
		      inspo->im.imfcben = cben + cbeneis;
		      DEBUG3("%s cben: eldcben=%.2f spocben=%.2f\n",
		 			ineld->im.imfcben, inspo->im.imfcben);

		  }
		  else {
		     DEBUG1("%s cben: elder receives CTC\n");
		     ineld->im.imfcbenb = cben;
		     ineld->im.imfcbene = cbeneis;
		     ineld->im.imfcben = cben + cbeneis;
		     DEBUG2("%s cben: eldcben=%.2f\n", ineld->im.imfcben);

		  }

      /* end of nuclear family loop. */
      }

   }
	DEBUG_OFF("txctc");
}
