/***
*txfstc.cpp - Compute federal sales tax credit
*
*$Id: txfstc.cpp 6.111 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/20 | Added imbcss (BC sen supp as part of sa) to income tests|
* | LP  | 06/06/05 | Added GST credit parameters and code for calendar calc. |
* | LP  | 06/05/30 | Subtract Universal Child Care Benefit from net income   |
* | LP  | 05/10/21 | Added Federal Energy Cost Benefit program (FECBFLAG+)   |
* | LP  | 04/02/17 | Added idiworkc to income test for Fed. Sales Tax Credit |
* | LP  | 01/03/12 | Removed pre-1988 parameters                             |
* | CH  | 00/10/24 | Added relief for heating expenses                       |
* | LP  | 00/06/05 | Added GSTCTUNC - GST take up by number of elig. children|
* | CH  | 00/04/10 | Added GSTAXM                                            |
* | CH  | 99/04/16 | Single parent families can optionally get the full      |
* |     |          |     additional amount                                   |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 97/10/02 | FSTC credit can be claimed for kids up to 19            |
* | CH  | 97/10/02 | GST credit is calculated using last year's income       |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txfstc   [global]  Compute federal sales tax credit
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxfstc[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txfstc)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txfstc - Compute federal sales tax credit
*
*Synopsis:
*    void txfstc(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   minn     Find the minimum of two numbers
*   taxbak1  One level tax-back function
*   lkup2    Table look-up without interpolation
*
*Structure Members:
*   MP.FECBFLAG     (i)  Federal Energy Cost Benefit activation flag
*   MP.FECBGIS      (i)  Federal Energy Cost Benefit for GIS recipients
*   MP.FECBNCBS     (i)  Federal Energy Cost Benefit for NCBS recipients
*   MP.FSTCC        (i)  Federal sales tax credit amount for dependant
*   MP.FSTCCP       (i)  Federal sales tax credit amount for dependant for 1st half of year
*   MP.FSTCF        (i)  Federal sales tax credit amount for filer
*   MP.FSTCFP       (i)  Federal sales tax credit amount for filer for 1st half of year
*   MP.FSTCL        (i)  Federal sales tax credit reduction level
*   MP.FSTCLP       (i)  Federal sales tax credit reduction level for 1st half of year
*   MP.FSTCR        (i)  Federal sales tax credit reduction rate
*   MP.FSTCRP       (i)  Federal sales tax credit reduction rate for 1st half of year
*   MP.FSTCS        (i)  Federal sales tax credit amount for spouse
*   MP.FSTCSP       (i)  Federal sales tax credit amount for spouse for 1st half of year
*   MP.GSTAC        (i)  GST additional credit amount
*   MP.GSTACP       (i)  GST additional credit amount for 1st half of year
*   MP.GSTAR        (i)  GST additional credit rate of net income
*   MP.GSTARP       (i)  GST additional credit rate of net income for 1st half of year
*   MP.GSTASPFLAG   (i)  Maximum GST additional credit for single parents flag
*   MP.GSTASPFLAGP  (i)  Maximum GST additional credit for single parents flag for 1st half of year
*   MP.GSTAXM       (i)  GST additional credit exemption
*   MP.GSTAXMP      (i)  GST additional credit exemption for 1st half of year
*   MP.GSTCTUNC     (i)  GST credit take up by number of children
*   MP.GSTCTUNCrows (i)  number of rows in GSTCTUNC
*   MP.GSTCYFLG     (i)  GST credit calendar year calculation activation flag
*   MP.GSTFLAG      (i)  GST credit activation flag
*   MP.HEATFAM      (i)  Federal relief for heating expenses for families
*   MP.HEATRLFLG    (i)  Federal relief for heating expenses activation flag
*   MP.HEATSNG      (i)  Federal relief for heating expenses for singles
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   cf.cfageeld     (i)  Age of eldest in census family
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfspoflg     (i)  Census family contains married couple
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.idrand       (i)  Random numbers [array]
*   im.imfcbene     (i)  Total Federal Child Benefit Supplement
*   im.imicnet      (i)  Net Income (line 236)
*   im.imigis       (i)  GIS benefits
*   im.imispa       (i)  Spouse's allowance
*   im.imitax       (i)  Taxable income
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imfecb       (o)  Federal Energy Cost Benefit
*   im.imfstc       (o)  Federal sales tax credit
*   im.imheatrl     (o)  Federal relief for heating expenses
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void txfstc(
	P_hh hh
)
{
	register P_in ineld;
	register P_in inspo;
	register P_cf cf;
	register int cfi;
	int ini;
	P_in in;
	int nelich;


	register NUMBER faminet = ZERO;
	NUMBER fstc = ZERO;
	NUMBER fstc1 = ZERO;
	NUMBER fstc2 = ZERO;
	NUMBER gsta = ZERO;     /* GST additional amount for singles */
	NUMBER kidfstc = ZERO;	   /* sales tax credit for children over age limit */
	NUMBER kidfstc1 = ZERO;	   /* sales tax credit for children over age limit */
	NUMBER kidfstc2 = ZERO;	   /* sales tax credit for children over age limit */
	NUMBER kidgsta = ZERO;     /* GST additional amount for single children over age limit */
	NUMBER kidgsta1 = ZERO;     /* GST additional amount for single children over age limit */
	NUMBER kidgsta2 = ZERO;     /* GST additional amount for single children over age limit */
	NUMBER kidinet = ZERO;     /* income for children over age limit */


	DEBUG_ON("txfstc");

    /* process each census family in household */
	for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {

    	nelich = 0 ;  /* eligible children in household */
		  

		/* initialise elder's pointer */
		ineld = cf->cfineld;

    	/* add in the federal sales tax credit of the filer */
	    fstc2 = MP.FSTCF;
		DEBUG2("%s filer's fstc=%.2f\n", fstc);

		/* calculate contribution of eldest to family net income */
		faminet = nneg(ineld->im.imicnet - ineld->im.imuccb);
		DEBUG2("%s faminet=%.2f\n", faminet);
		DEBUG1("%s including elder's social assistance etc. in faminet\n");
		DEBUG4("%s imigis=%.2f imispa=%.2f imisa=%.2f\n",
			ineld->im.imigis, ineld->im.imispa, ineld->im.imisa);
		
		if (cf->cfspoflg) {
			DEBUG1("%s spouse present\n");
			inspo = cf->cfinspo;

			/* add in the federal sales tax credit of the spouse */
			fstc2 += MP.FSTCS;
			DEBUG2("%s spouses's fstc=%.2f\n", MP.FSTCS);

			/* add spouse's net income to family net income */
			faminet += nneg(inspo->im.imicnet - inspo->im.imuccb);
		}


		/* if reform activated, count kids and assign benefit to persons over the age of 19*/
			/* process children in census family */
		for (ini=0, in=cf->cfinch;  ini<cf->cfnchild;  in++) {

	 	   ini++;

		   kidinet = ZERO;
		   kidfstc2 = ZERO;
		   kidgsta2 = ZERO;
		   kidfstc1 = ZERO;
		   kidgsta1 = ZERO;

		   if (in->id.idage >= 19) {
			   DEBUG2("%s discarding old child, aged %d\n", in->id.idage);
			   /* Old children qualify for their own benefit */
		       kidfstc2 = MP.FSTCF;
		       kidinet = nneg(in->im.imicnet - in->im.imuccb);
		   	   /* since these are children in census family, they cannot have a 
			      spouse or children of their own */

			   /* if GST, then credit is calculated using previous year's income 
		          and since single with no kids, they qualify for additional amount */
		       if (MP.GSTFLAG) {
		            kidinet *= MP.PYINC;
		            kidgsta2 = minn(MP.GSTAR * nneg(kidinet - MP.GSTAXM), MP.GSTAC);
		            kidfstc2 += kidgsta2;
		   	   }

 		       /* tax back credit (fstc) at reduction rate FSTCR on kidinet */
		       /* in excess of FSTCL.                                       */

		       taxbak1(kidinet, MP.FSTCL, &kidfstc2, MP.FSTCR);

			   if (MP.GSTCYFLG) {  /* calculate credit for calendar year */
					kidfstc1 = MP.FSTCFP;
					/* adjust income to represent the 2 years prior */
				   	kidinet = (kidinet / MP.PYINC) * MP.PYINCP;	 
					kidgsta1 = minn(MP.GSTARP * nneg(kidinet - MP.GSTAXMP), MP.GSTACP);
					kidfstc1 += kidgsta1;
					taxbak1(kidinet, MP.FSTCLP, &kidfstc1, MP.FSTCRP);

					kidfstc = (kidfstc2/2) + (kidfstc1/2);
					in->im.imfstc = kidfstc;
			   }
			   else {  /* use the July to June amounts for the year */

					kidfstc = kidfstc2;
		       		in->im.imfstc = kidfstc2;
			   }

               /* if gst credit is greater than zero, then give the heating credit.
                   In theory, it should be gst credit of previous year. 
                   Census family children always qualify as singles
               */
               if ( MP.HEATRLFLG && kidfstc > ZERO ) {
                   in->im.imheatrl = MP.HEATSNG;
               }

		   }

		   /* if child is young enough, count for additional deduction for parent */
		   else nelich++;

		}

		if (!cf->cfspoflg && nelich==0 && cf->cfageeld < 19) {
			DEBUG1("%s skipping census families who are too young to qualify for credit'\n");
			continue;
		}
		

        /* if single parent */
		if (!cf->cfspoflg && MP.GSTFLAG && nelich>0) {
			DEBUG2("%s single parent family, adding %.2f\n",
				MP.FSTCF - MP.FSTCC);
			fstc2 += MP.FSTCF - MP.FSTCC;
		}

	    /* if GST, then credit is calculated using previous years income */
		if (MP.GSTFLAG) {
		    faminet *= MP.PYINC;
		}

		/* add in the federal sales tax credit of the children */
		if (nelich != 0) {
			DEBUG3("%s %d kids present, adding %.2f\n",
				nelich, nelich * MP.FSTCC);
			fstc2 += nelich * MP.FSTCC;
		}

		/* Add amount for single parent families when they get maximum
		   additional amount as per the 1999 Budget */
		if (MP.GSTFLAG && !cf->cfspoflg && nelich != 0 && MP.GSTASPFLAG) {
				gsta = MP.GSTAC;
				DEBUG3("%s gsta=%.2f faminet=%.2f\n", gsta, faminet);
				fstc2 += gsta;
		}
		/* Add amount for single parent families (when it depends on net income)
		   and for other single persons */
		else if (MP.GSTFLAG && !cf->cfspoflg) {
			gsta = minn(MP.GSTAR * nneg(faminet - MP.GSTAXM), MP.GSTAC);
			DEBUG3("%s gsta=%.2f faminet=%.2f\n", gsta, faminet);
			fstc2 += gsta;
		}


		/* tax back credit (fstc) at reduction rate FSTCR on faminet */
		/* in excess of FSTCL.                                       */

		DEBUG2("%s faminet for reduction=%.2f\n", faminet);
		DEBUG2("%s fstc before reduction=%.2f\n", fstc2);
		taxbak1(faminet, MP.FSTCL, &fstc2, MP.FSTCR);
		DEBUG2("%s fstc after  reduction=%.2f\n", fstc2);

		if (MP.GSTCYFLG) {
			gsta = ZERO;
	    	/* add in the federal sales tax credit of the filer */
		    fstc1 = MP.FSTCFP;				   5
			DEBUG2("%s filer's fstc=%.2f\n", fstc1);


			if (cf->cfspoflg) {

				/* add in the federal sales tax credit of the spouse */
				fstc1 += MP.FSTCSP;
				DEBUG2("%s spouses's fstc=%.2f\n", MP.FSTCSP);

			}

	        /* if single parent */
			if (!cf->cfspoflg && nelich>0) {
				DEBUG2("%s single parent family, adding %.2f\n",
					MP.FSTCFP - MP.FSTCCP);
				fstc1 += MP.FSTCFP - MP.FSTCCP;
			}

		    /* if GST, then credit is calculated using 2 years prior income */
		    faminet = (faminet / MP.PYINC) * MP.PYINCP;	

			/* add in the federal sales tax credit of the children */
			if (nelich != 0) {
				DEBUG3("%s %d kids present, adding %.2f\n",
					nelich, nelich * MP.FSTCCP);
				fstc1 += nelich * MP.FSTCCP;
			}

			/* Add amount for single parent families when they get maximum
			   additional amount as per the 1999 Budget */
			if ( !cf->cfspoflg && nelich != 0 && MP.GSTASPFLAGP) {
					gsta = MP.GSTACP;
					DEBUG3("%s gsta=%.2f faminet=%.2f\n", gsta, faminet);
					fstc1 += gsta;
			}
			/* Add amount for single parent families (when it depends on net income)
			   and for other single persons */
			else if ( !cf->cfspoflg) {
				gsta = minn(MP.GSTARP * nneg(faminet - MP.GSTAXMP), MP.GSTACP);
				DEBUG3("%s gsta=%.2f faminet=%.2f\n", gsta, faminet);
				fstc1 += gsta;
			}


			/* tax back credit (fstc) at reduction rate FSTCR on faminet */
			/* in excess of FSTCL.                                       */

			DEBUG2("%s faminet for reduction=%.2f\n", faminet);
			DEBUG2("%s fstc before reduction=%.2f\n", fstc1);
			taxbak1(faminet, MP.FSTCLP, &fstc1, MP.FSTCRP);
			DEBUG2("%s fstc after  reduction=%.2f\n", fstc1);

			/*  Apply a take up rate for the GST credit based on number of    *
			*   eligible children in the family.                               * 
			*   The take up rate is used to denote what proportion of families*
			*   based on number of children would not apply for their entitled*
			*   credit.  Thus a take up of 1.000 results in everyone taking it*
			*   while a take up of less than 1.000 (ex. .888) means that (88.8%) * 
			*   proportion would get the credit and the remainder (11.2%) would *
			*   have the GST credit converted to 0.   */
			  
			if (fstc1 > 0 &&
			    (ineld->id.idrand[5] > (NUMBER) lkup2(MP.GSTCTUNC, MP.GSTCTUNCrows, (double) nelich))) {
				 fstc1 = ZERO;
			}

			fstc = (fstc2/2) + (fstc1/2);

		}
		else {  /* use the July to June amounts for the year */
			fstc = fstc2;
		}

		/*  Apply a take up rate for the GST credit based on number of    *
		*   eligible children in the family.                               * 
		*   The take up rate is used to denote what proportion of families*
		*   based on number of children would not apply for their entitled*
		*   credit.  Thus a take up of 1.000 results in everyone taking it*
		*   while a take up of less than 1.000 (ex. .888) means that (88.8%) * 
		*   proportion would get the credit and the remainder (11.2%) would *
		*   have the GST credit converted to 0.   */
			  
		if (fstc > 0 &&
		    (ineld->id.idrand[5] > (NUMBER) lkup2(MP.GSTCTUNC, MP.GSTCTUNCrows, (double) nelich))) {
			 fstc = ZERO;
		}


		/**
		* In the absence of any other information,
		* assign FSTC to spouse with higher net income.
		**/

		if (cf->cfspoflg && (inspo->im.imitax > ineld->im.imitax)) {
			DEBUG1("%s spouse has higher taxable income and gets fstc\n");
			inspo->im.imfstc = fstc;
            /* if gst credit is greater than zero, then give the heating credit.
                In theory, it should be gst credit of previous year. 
                Since there is a spouse, give the amount for the family 
            */
            if ( MP.HEATRLFLG && fstc > ZERO ) {
                inspo->im.imheatrl = MP.HEATFAM;
            }
		}
		else {
			DEBUG1("%s elder has higher taxable income and gets fstc\n");
			ineld->im.imfstc = fstc;
            /* if gst credit is greater than zero, then give the heating credit.
                In theory, it should be gst credit of previous year. 
            */
            if ( MP.HEATRLFLG && fstc > ZERO ) {
                /* check to see if family amount should be given */
                if (cf->cfspoflg || nelich != 0) {
                    ineld->im.imheatrl = MP.HEATFAM;
                }
                else {
                    ineld->im.imheatrl = MP.HEATSNG;
                }
            }
		}
  	}
		/* Federal Energy Cost Benefit program */

	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
		if (MP.FECBFLAG) {
			if (in->im.imigis > 0 || in->im.imispa > 0) {
				in->im.imfecb = MP.FECBGIS;
			}

			/* if an individual is in receipt of both the NCBS and GIS/SPA
			   then the higher amount is awarded (the NCBS amount). */

			if (in->im.imfcbene > 0) {
				in->im.imfecb = MP.FECBNCBS;
			}

		}
		else {
			in->im.imfecb = ZERO;
		}
    }


	DEBUG_OFF("txfstc");

}

