/***
*txqccea.cpp - Compute child care expense allowance (Quebec)
*
*$Id: txqccea.cpp 6.105 2008/08/22 17:07:46 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 07/01/16 | Deleted imuccb from net inc test, imuccb only in tax inc|
* | LP  | 06/06/09 | Removed QCCEA1, QCCEA2, QCCEA3 - no longer required     |
* | LP  | 06/06/01 | Removed imuccb from income tested credits               |
* | LP	| 04/06/18 | Added QCAFLAG to child care expense calculation         |
* | LP	| 04/02/09 | Added CCEAOAG - increased age limit for CCE -same as Fed|
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | BBM | 92/07/28 | Release 4.0                                             |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txqccea  [global]  Compute child care expense allowance (Quebec)
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxqccea[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txqccea)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txqccea - Compute child care expense allowance (Quebec)
*
*Synopsis:
*    void txqccea(        
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   isnzero  Is argument non-zero ?
*   iszero   Is argument zero ?
*   nneg     Change negative numbers to zero
*
*Structure Members:
*   MP.CCEAOAGE     (i)  CCEA Maximum eligible age for oldest child
*   MP.CCEATFLG     (i)  CCEA eligible->claimable transformation activation flag
*   MP.CCEATFR      (i)  CCEA eligible->claimable fraction
*   MP.CCEATLL      (i)  CCEA eligible->claimable lower limit
*   MP.QBXM         (i)  Quebec basic personal exemption/amount
*   MP.QCAFLAG      (i)  Quebec Child Assistance refundable tax credit flag
*   MP.QCCEAOLD     (i)  Quebec CCEA old child limit
*   MP.QCCEAYNG     (i)  Quebec CCEA young child limit
*   MP.QCCEOPT      (i)  Quebec CCE calculation option [1=deduction,2=refundable credit]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   id.idage        (i)  Age
*   id.idccet       (i)  Child care expenses associated with child
*   id.idccett      (i)  Child care expenses (Limit A, Form T778)
*   id.idiemp       (i)  Wages & salaries
*   id.idise        (i)  Self-employed income
*   im.imqinet      (io) Quebec net income
*   im.imqccea      (o)  Quebec child care expenses allowed (dedn)
*   im.imqdedft     (o)  Quebec deductions from total income
*
*Called by:
*   txqinet  Compute net income (Quebec)
*
*****************************************************************************/
void txqccea(
	P_hh hh
)
{
	int spogets;    /* 0=elder gets, 1=spouse gets */
	P_in ineld;
	P_in inspo;
	int cfi;
	P_in in;
	int ini;
	register P_cf cf;
	int spoflg;
	int kidsclmd;         /* number of children eligible for CCE */
	NUMBER ccee = ZERO;   /* eligible child care expenses for child */
	NUMBER ccec = ZERO;   /* child care expenses claimed for child */
	NUMBER ccea = ZERO;   /* child care expenses claimed for entire family */
	NUMBER limitA = ZERO; /* limitation `A' in form (amount paid) */
	NUMBER limitB = ZERO; /* limitation `B' in form (earned income) */
	NUMBER limitC = ZERO; /* limitation `C' in form (amounts per young/old child) */
	NUMBER earneld = ZERO;  /* elder's earned income */
	NUMBER earnspo = ZERO;  /* spouse's earned income */


	DEBUG_ON("txqccea");

	if (MP.QCCEOPT==EXEM) {
		/**
		* Child care expenses are calculated as deduction from total income
		* to arrive at net income
		**/
		
		/* process each census family in household */
		for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {

			/* skip to next family if no kids here */
			if (!cf->cfnkids) {
				continue;
			}

			limitA = ZERO;
			limitC = ZERO;

			/* Use FAMEX-derived SPSD amount that varies with child, and fudge */

			/* first determine allowed amount for family */
			/* process kids in census family */
			kidsclmd = 0;
			for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++) {
				ini++;

	         /* determine total amount of child care expenses per child */
			 if (MP.CCEATFLG) {
	   			/* fudge eligible amount from FAMEX total child care expenses */
	            if (in->id.idccet < MP.CCEATLL) {
					   ccee = ZERO;
	   			}
		   		else {
			   		ccee = MP.CCEATFR * in->id.idccet;
				   }
				}
	         else {  /* Use Greenbook imputed amount per child */
		   		ccee = in->id.idccett;
	         }

				DEBUG4("%s processing child aged %d, ccet=%.2f ccee=%.2f\n",
					in->id.idage, in->id.idccet, ccee);

				/* subject to limits as a function of age */
				if (in->id.idage <= 6) {
					limitC += MP.QCCEAYNG;
					ccec = ccee;
					kidsclmd++;
				}
				else if (in->id.idage <= MP.CCEAOAGE) {
					   /* Once the universal Child Assistance Program is activated 
					      eligible children are those with income less than QBXM	*/
					   if (MP.QCAFLAG && in->im.imqinet > MP.QBXM ) {
					       ccec = 0;
					   }
					   else {
							limitC += MP.QCCEAOLD;
							ccec = ccee;
					   }
					   if (isnzero(ccec)) {
						kidsclmd++;
					   }
				}
				else {
					/* child too old to be eligible */
					continue;
				}

				/* cumulate amount for family */
				limitA += ccec;
			}

			/* let ccea be the minimum of limitations A and C */
			ccea = (limitA < limitC) ? limitA : limitC;


			/* continue with next family if ccea is zero */
			if (iszero(ccea)) {
				continue;
			}

			/* get elder's pointer */
			ineld = cf->cfineld;


			/* compute elder's earned income */
			earneld = ineld->id.idiemp
				     + nneg(ineld->id.idise);
			DEBUG2("%s elder's earned income=%.2f\n", earneld);

			/**
			* Determine if spouse is present in census family,
			* and perform appropriate calculations.
			**/

			/* initialise who claims CCE to elder */
			spogets = 0;

			spoflg = cf->cfspoflg;
			if (spoflg) {
				DEBUG1("%s spouse present\n");

				/* get spouse's pointer */
				inspo = cf->cfinspo;

				/* compute spouse's earned income */
				earnspo = inspo->id.idiemp
				     	+ nneg(inspo->id.idise);
				DEBUG2("%s spouse's earned income=%.2f\n", earnspo);

				/* compute limit B based on lower earned income */
				if (earnspo < earneld) {
					limitB = earnspo;
				}
				else {
					limitB = earneld;
				}

				/**
				* Optimize by assigning CCE to spouse with higher net income,
				* since in this way the family as a whole optimizes their
				* tax situation.
				**/

				DEBUG1("%s partner with higher net income receives ccea\n");
				if (inspo->im.imqinet >= ineld->im.imqinet) {
					DEBUG2("%s spouse claims ccea=%.2f\n", ccea);
					spogets = 1;
				}
				else {
					DEBUG2("%s elder claims ccea=%.2f\n", ccea);
				}
			}

			else {
			limitB = earneld;
				DEBUG2("%s elder claims ccea=%.2f\n", ccea);
			}

			DEBUG2("%s limit `B'=%.2f\n", limitB);

			/* Apply limit B */
			if (ccea > limitB) {
				ccea = limitB;
			}

			DEBUG1("%s assigning child care expenses deduction\n");
			if (spogets) {

				DEBUG2("%s original qinet =%.2f\n", inspo->im.imqinet);
				DEBUG2("%s original qdedft=%.2f\n", inspo->im.imqdedft);
				inspo->im.imqinet = nneg(inspo->im.imqinet - ccea);
				inspo->im.imqccea = ccea;
				inspo->im.imqdedft += ccea;
				DEBUG2("%s revised qinet =%.2f\n", inspo->im.imqinet);
				DEBUG2("%s revised qdedft=%.2f\n", inspo->im.imqdedft);
			}
			else {

				DEBUG2("%s original inet =%.2f\n", ineld->im.imqinet);
				DEBUG2("%s original dedft=%.2f\n", ineld->im.imqdedft);
				ineld->im.imqinet = nneg(ineld->im.imqinet - ccea);
				ineld->im.imqccea = ccea;
				ineld->im.imqdedft += ccea;
				DEBUG2("%s revised inet =%.2f\n", ineld->im.imqinet);
				DEBUG2("%s revised dedft=%.2f\n", ineld->im.imqdedft);
			}
		}
	}

	DEBUG_OFF("txqccea");
	return;
}
