/***
*txsask.cpp - Compute provincial taxes for Saskatchewan
*
*$Id: txsask.cpp 6.140 2008/08/22 17:07:46 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 08/06/11 | Added SCGTCTD and a model option to SCGTCOPT and code   |
* | LP  | 08/06/03 | Removed caregivers (impcgtc) from transfer of credits   |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/19 | Removed SUCCBITF-UCCB removed from income as per forms  |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/12 | Added Sask sales tax credit by calendar year code/parms |
* | LP  | 06/06/09 | Removed SAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added SUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/06/05 | Revised the application of PYINCP                       |
* | LP  | 06/05/26 | Added Sask child benefit by calendar year code/parms    |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate SDTCRLC|
* | LP  | 05/11/29 | Fixed bug in part-time months of education tax credit   |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 04/06/22 | Added Sask. Child Benefit Single Parent Amount - SCBSPA |
* | LP  | 04/02/20 | Removed Sask. Employment Supplement - SA program        |
* | LP  | 04/02/17 | Added idiworkc to income test for sask stc and child ben|
* | LP  | 03/12/22 | Added SMEDALL and SMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/09 | Added impothpe, SODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added SCGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/12 | Added ideducpm and SEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | LP  | 03/03/25 | Moved imssstxc,imsctxc to transferable between spouses  |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/07/10 | Added take up rate to Sask. Employment Supplement       |
* | LP  | 01/06/21 | Added Employment Supplementary Benefit for under 13     |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred.|
* | LP  | 01/05/09 | Added SAMTPCTM and added another option to SAMTOPT      |
* | LP  | 01/04/10 | Added SPTC, SPTCBEN Sask political contrib. tax credit  |
* | LP  | 01/04/05 | Added Saskatchewan Home Heating Assistance Rebate       |
* | LP  | 01/03/12 | Removed pre-1988 programs (SRDOPT and exemptions)       |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/05/05 | Modified CGTC                                           |
* | CH  | 00/04/07 | Removed unused code                                     |
* | CH  | 00/04/07 | Added child and senior tax credits                      |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/05/21 | Added Child benefit phase in                            |
* | CH  | 98/04/30 | Added Child benefit and Employment supplement           |
* | SFG | 97/10/23 | Change overlay definitions to avoid breaking DOS limits |
* | BBM | 97/05/27 | Drop idsaskti -- no data in 1992                        |
* | ER  | 96/09/12 | Sask. deficit surtax reduction amount (SDSRA)           |
* | GJC | 93/04/16 | fix up problem with deficit surtax calculation          |
* | BBM | 92/09/02 | Removed Saskatchewan family tax credit proposal         |
* | BBM | 92/07/22 | Incorporated non-refundable provincial tax credit var   |
* | BBM | 92/07/08 | Modeled provincial tax incentives, drop idptc ref       |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txsask   [global]  Compute provincial taxes for Saskatchewan
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxsask[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(ROOT_TEXT,txsask)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txsask - Compute provincial taxes for Saskatchewan
*
*Synopsis:
*    void txsask(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   minn     Find the minimum of two numbers
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   lkup1    Table look-up with interpolation/extrapolation
*
*Structure Members:
*   MP.CTCINC       (i)  Refundable CTC family income scaling factor
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.GSTCYFLG     (i)  GST credit calendar year calculation activation flag
*   MP.HEATSNG      (i)  Federal relief for heating expenses for singles
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   MP.SAMTOPT      (i)  Sask. alternative minimum tax option
*   MP.SAMTPCTF     (i)  Sask. amt rate as pct of additional fed tax due to minimum tax
*   MP.SAMTPCTM     (i)  Sask. amt rate as pct of federal minimum tax amount
*   MP.SAMTTX       (i)  Sask. amt rate as tax on adjusted income
*   MP.SAXM         (i)  Sask. Age Amount
*   MP.SAXRR        (i)  Sask. Age Amount credit reduction rate
*   MP.SAXTD        (i)  Sask. Age Amount net income turndown
*   MP.SBXM         (i)  Sask. Basic Personal Exemption/amount
*   MP.SCB1         (i)  Sask Child Benefit max for 1st child
*   MP.SCB1P        (i)  Sask Child Benefit max for 1st child for 1st half of year
*   MP.SCB2         (i)  Sask Child Benefit max for 2nd child
*   MP.SCB2P        (i)  Sask Child Benefit max for 2nd child for 1st half of year
*   MP.SCB3         (i)  Sask Child Benefit max for 3rd and additional child
*   MP.SCB3P        (i)  Sask Child Benefit max for 3rd and additional child for 1st half of year
*   MP.SCBFLG       (i)  Sask Child Benefit activation flag
*   MP.SCBRR1       (i)  Sask Child Benefit 1st reduction rate [number of kids]
*   MP.SCBRR1P      (i)  Sask Child Benefit 1st reduction rate [number of kids] for 1st half of year
*   MP.SCBRR2       (i)  Sask Child Benefit 2nd reduction rate [number of kids]
*   MP.SCBRR2P      (i)  Sask Child Benefit 2nd reduction rate [number of kids] for 1st half of year
*   MP.SCBSPA       (i)  Sask Child Benefit Single Parent Amount
*   MP.SCBSPAP      (i)  Sask Child Benefit Single Parent Amount for 1st half of year
*   MP.SCBTD1       (i)  Sask Child Benefit 1st turndown
*   MP.SCBTD1P      (i)  Sask Child Benefit 1st turndown for 1st half of year
*   MP.SCBTD2       (i)  Sask Child Benefit 2nd turndown
*   MP.SCBTD2P      (i)  Sask Child Benefit 2nd turndown for 1st half of year
*   MP.SCGTC        (i)  Sask. Caregiver Tax Credit
*   MP.SCGTCFLG     (i)  Sask. Caregiver Tax Credit activation flag
*   MP.SCGTCOPT     (i)  Sask. Caregiver Tax Credit Option (1=max, 2=greenbook, 3=model)
*   MP.SCGTCTD      (i)  Sask. Caregiver Tax Credit income turndown
*   MP.SCHATL1      (i)  Sask. Charitable Donations amount level 1
*   MP.SCHATR1      (i)  Sask. Charitable Donations tax credit rate 1
*   MP.SCHATR2      (i)  Sask. Charitable Donations tax credit rate 2
*   MP.SCNTC        (i)  Sask. non-refundable child tax credit amt per child
*   MP.SDSF         (i)  Saskatchewan provincial deficit surtax fraction
*   MP.SDSRA        (i)  Saskatchewan deficit surtax reduction rate
*   MP.SDTCR        (i)  Sask. dividend tax credit rate
*   MP.SDTCRLC      (i)  Sask. dividend tax credit rate from large (eligible) corporations
*   MP.SEDXPM       (i)  Sask. Education Amount per month of full-time studies
*   MP.SEDXPMPT     (i)  Sask. Education Amount per month of part-time studies
*   MP.SEMXM        (i)  Sask. equivalent to married amount
*   MP.SEMXMT       (i)  Sask. equivalent to married turndown level
*   MP.SFTAX        (i)  Saskatchewan provincial flat surtax rate on net income
*   MP.SHEATFAM     (i)  Saskatchewan Home Heating Assistance Rebate for families
*   MP.SHEATFLG     (i)  Saskatchewan Home Heating Assistance Rebate activation flag
*   MP.SHEATSNG     (i)  Saskatchewan Home Heating Assistance Rebate for singles
*   MP.SLVCMAX      (i)  Maximum Sask. labour-sponsored funds tax credit allowed
*   MP.SLVCRT       (i)  Percent of Sask. labour-sponsored funds cost allowed as credit
*   MP.SMAXDX       (i)  Sask. Maximum Disability deduction/amount
*   MP.SMAXET       (i)  Sask. maximum on transfer of education and tuition amount
*   MP.SMEDALL      (i)  Sask. Medical allowance maximum lower limit
*   MP.SMEDANF      (i)  Sask. Medical allowance lower limit net income fraction
*   MP.SMXM         (i)  Sask. married amount
*   MP.SMXMT        (i)  Sask. married amount turndown level
*   MP.SODISTC      (i)  Sask. Maximum infirm dependent 18 or older tax credit
*   MP.SPNTCR       (i)  Sask. provincial non-refundable tax credit rate
*   MP.SPTC         (i)  Saskatchewan political contribution table [total donations,donation allowed]
*   MP.SPTCBEN      (i)  Maximum Saskatchewan political tax credit allowed
*   MP.SPTCrows     (i)  number of rows in SPTC
*   MP.SPTF         (i)  Saskatchewan provincial tax fraction
*   MP.SPTX         (i)  Sask. tax table [taxable income,basic provincial tax]
*   MP.SPTXrows     (i)  number of rows in SPTX
*   MP.SSCI         (i)  Saskatchewan surtax cut-in
*   MP.SSF          (i)  Saskatchewan provincial high income surtax fraction
*   MP.SSSTC        (i)  Sask. senior supplement tax credit
*   MP.SSTC1KIDTD   (i)  Sask. sales tax credit child amount turndown if only 1 child
*   MP.SSTC1KIDTDP  (i)  Sask. sales tax credit child amount turndown if only 1 child for 1st half of year
*   MP.SSTC2KIDTD   (i)  Sask. sales tax credit child amount turndown if more than 1 child
*   MP.SSTC2KIDTDP  (i)  Sask. sales tax credit child amount turndown if more than 1 child for 1st half of year
*   MP.SSTCBAS      (i)  Sask. sales tax credit base amount
*   MP.SSTCBASP     (i)  Sask. sales tax credit base amount for 1st half of year
*   MP.SSTCBASPIR   (i)  Sask. sales tax credit base amount phase in rate
*   MP.SSTCBASPIRP  (i)  Sask. sales tax credit base amount phase in rate for 1st half of year
*   MP.SSTCBASTD    (i)  Sask. sales tax credit base amount turndown
*   MP.SSTCBASTDP   (i)  Sask. sales tax credit base amount turndown for 1st half of year
*   MP.SSTCFLAG     (i)  Sask. sales tax credit activation flag
*   MP.SSTCKID      (i)  Sask. sales tax credit child amount
*   MP.SSTCKIDP     (i)  Sask. sales tax credit child amount for 1st half of year
*   MP.SSTCPI       (i)  Sask. sales tax credit phase in
*   MP.SSTCPI1P     (i)  Sask. sales tax credit phase in for 1st half of year
*   MP.SSTCPI2P     (i)  Sask. sales tax credit phase in for 2nd half of year
*   MP.SSTCRR       (i)  Sask. sales tax credit reduction rate
*   MP.SSTCRRP      (i)  Sask. sales tax credit reduction rate for 1st half of year
*   MP.SSTCSP       (i)  Sask. sales tax credit spousal amount
*   MP.SSTCSPP      (i)  Sask. sales tax credit spousal amount for 1st half of year
*   MP.SSTCSPTD     (i)  Sask. sales tax credit spousal amount turndown
*   MP.SSTCSPTDP    (i)  Sask. sales tax credit spousal amount turndown for 1st half of year
*   MP.SSTR         (i)  Saskatchewan spousal & married equivalent tax reduction
*   MP.STRBA        (i)  Saskatchewan tax reduction base amount
*   MP.STRBR        (i)  Saskatchewan basic provincial tax reduction
*   MP.STRCL        (i)  Saskatchewan child tax reduction limit
*   MP.STRPC        (i)  Saskatchewan tax reduction per child
*   MP.STRRR        (i)  Saskatchewan tax reduction reduction rate
*   MP.STRSC        (i)  Saskatchewan tax reduction for senior citizens
*   MP.STXFLG       (i)  Sask. tax on taxable income activation flag
*   MP.SYPNDL       (i)  Sask. Pension Income Deduction Amount
*   MP.TARGETYEAR   (i)  Year of analysis
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   ef.efin         (i)  First person in economic family [pointer]
*   ef.efnpers      (i)  Number of persons in economic family
*   hh.cf           (i)  Census family data [array]
*   hh.ef           (i)  Economic family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnef        (i)  Number of economic families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.iddisab      (i)  Disability status (age 16+)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.ididiv       (i)  Dividend income (actual)
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idsex        (i)  Sex
*   id.idspoflg     (i)  Person has spouse
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imamtfg      (i)  Minimum tax flag
*   im.imamtinc     (i)  Net adjusted income used for minimum tax
*   im.imamtprv     (i)  Federal minimum tax concept used for provincial minimum tax
*   im.imbft        (i)  Basic federal tax
*   im.imchara      (i)  Allowable charitable donations and gifts (calculated)
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedfn      (i)  All deductions from net income
*   im.imdepni      (i)  Dependant's net income
*   im.imheatrl     (i)  Federal relief for heating expenses
*   im.imicnet      (i)  Net Income (line 236)
*   im.imigis       (i)  GIS benefits
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.iminet       (i)  Net income
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imispa       (i)  Spouse's allowance
*   im.imitax       (i)  Taxable income
*   im.immartxc     (i)  Married tax credit claimed
*   im.imminamt     (i)  Minimum Amount due to Federal Minimum Tax
*   im.imnfach      (i)  Number of family allowance children claimed
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imbpt        (io) Basic provincial tax
*   im.imnptc       (io) Non-refundable provincial tax credits
*   im.impatxc      (io) Prov Age amount tax credit
*   im.impbtc       (io) Prov Basic amount
*   im.impcgtc      (io) Prov Caregiver Tax Credit Level
*   im.impcgtcp     (io) Provincial Dependent Caregiver Tax Credit Amount
*   im.impchrtc     (io) Prov Charitable Donations tax credit
*   im.impcpptc     (io) Prov CPP/QPP contributions tax credit
*   im.impdistc     (io) Prov Disability tax credit
*   im.impdtxc      (io) Provincial dividend tax credit
*   im.impedtxc     (io) Prov Education tax credit
*   im.impintsl     (io) Provincial interest on student loans tax credit
*   im.implvctc     (io) Provincial labour-sponsored funds tax credit
*   im.impmartc     (io) Prov Married and Equivalent to Married tax credit
*   im.impmeda      (io) Prov Medical expenses allowed (computed)
*   im.impmedtc     (io) Prov Medical Expenses tax credit
*   im.impminco     (io) Prov Minimum tax carryover
*   im.impnit       (io) Provincial net income tax
*   im.impothpe     (io) Provincial infirm dependents 18 or older tax credit
*   im.imppentc     (io) Prov Pension Income tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.imprvtax     (io) Prov gross provincial tax
*   im.impsur       (io) Provincial surtax
*   im.imptaxcr     (io) Prov total tax credits
*   im.imptr        (io) Provincial tax reduction
*   im.impttxct     (io) Prov total tax credits transferred
*   im.imptutxc     (io) Prov Tuition tax credit
*   im.impuictc     (io) Prov UI/EI contributions tax credit
*   im.imsctxc      (io) Saskatchewan Non-refundable child tax credit
*   im.imssstxc     (io) Saskatchewan senior supplement tax credit
*   im.imtxp        (io) Provincial income tax payable
*   im.impatxcr     (o)  Prov total tax credits applied
*   im.impctxct     (o)  Prov tax credits transferred from children
*   im.impheatr     (o)  Provincial Home Heating Fuel/Energy Rebate
*   im.impstxct     (o)  Prov tax credits transferred from spouse
*   im.imptc        (o)  Refundable provincial tax credits
*   im.imscb        (o)  Saskatchewan Child Benefit
*   im.imsstc       (o)  Saskatchewan refundable sales tax credit
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void txsask(
    P_hh hh
)
{
    register int ini;
    register P_in in;
    register P_in ineld;
    register P_in inspo;
    register    P_nf nf;
    register int nfi;
    register P_cf cf;
    register int cfi;
	P_ef ef;
			  
	register int efi;
    NUMBER nfinet = ZERO;
    NUMBER nfinetcy = ZERO;
    NUMBER nfiemp = ZERO;
    NUMBER cben = ZERO;
    NUMBER cben1 = ZERO;
    NUMBER cben2 = ZERO;
    NUMBER poltcr = ZERO;    /* Political tax credit (raw) */
    NUMBER poltc = ZERO;     /* Political tax credit */
    NUMBER lvctc = ZERO;   /* Labour-sponsored venture capital tax cred */
    NUMBER eldnet = ZERO;   /* Net income of ineld */
    NUMBER sponet = ZERO;   /* Net income of inspo */
    NUMBER fmnet = ZERO;    /* Family net income */
    NUMBER highnet = ZERO;  /* Biggest net income in family */
    NUMBER stceld = ZERO;   /* Sales tax credit of ineld */
    NUMBER stcspo = ZERO;   /* Sales tax credit of spouse */
    NUMBER stckid = ZERO;   /* Sales tax credit of kids */
    NUMBER stckid2 = ZERO;   /* Sales tax credit of second child */
    NUMBER famstc = ZERO;   /* Family Sales tax credit */
    NUMBER famstc2 = ZERO;   /* Family Sales tax credit */
    NUMBER stceldp = ZERO;   /* Sales tax credit of ineld */
    NUMBER stcspop = ZERO;   /* Sales tax credit of spouse */
    NUMBER stckidp = ZERO;   /* Sales tax credit of kids */
    NUMBER stckid2p = ZERO;   /* Sales tax credit of second child */
    NUMBER famstcp = ZERO;   /* Family Sales tax credit */
	NUMBER adjnet = ZERO;    /* Adjusted net income concept */
	NUMBER efimcgtcp = ZERO;  /* combined cgtc potential amounts for ef */   

    DEBUG_ON("txsask");

    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.STXFLG) { 

       /* Calculate the Caregiver tax credit */
       if (MP.SCGTCFLG  && MP.SCGTCOPT == 3) {
       
          int cgassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimcgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 65 and either a head or spouse or other family member who could be a parent (with takeup) */
                if (in->id.idage >=65 || (in->id.iddisab == 1 && in->id.idage>= 18 ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.impcgtcp = MP.SPNTCR * minn(nneg(MP.SCGTCTD -in->im.iminet),MP.SCGTC);
                }
			   efimcgtcp += in->im.impcgtcp;
             }

			 cgassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (cgassign == 0 && in->id.idcgtc > 0 ) {
				   	if (in->id.idspoflg && inspo->im.impcgtcp > 0) {
				   		in->im.impcgtc = nneg(efimcgtcp - inspo->im.impcgtcp - in->im.impcgtcp );
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.impcgtc = nneg(efimcgtcp - in->im.impcgtcp);
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (cgassign == 0 && efimcgtcp > 0 && in->id.idcgtc > 0 && in->im.impcgtc == 0 ) {
		 			in->im.impcgtc = efimcgtcp;
		 			cgassign = 1;
		 	 	}
		    }
		  } 	 /* end of ef hh loop */
	   }  /* end of cgtc loop */


       /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;

          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */

          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni; 
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.SPNTCR * nneg(MP.SEMXM - nneg(cinet - MP.SEMXMT));
             }
          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption as well as the child tax credit
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.SPNTCR * nneg(MP.SMXM - nneg(inspo->im.imdepni - MP.SMXMT));
                ineld->im.imsctxc = cf->cfnkids * MP.SPNTCR * MP.SCNTC;
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.SPNTCR * nneg(MP.SMXM - nneg(ineld->im.imdepni - MP.SMXMT)); 
                inspo->im.imsctxc = cf->cfnkids * MP.SPNTCR * MP.SCNTC;
             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
             ineld->im.imsctxc = cf->cfnkids * MP.SPNTCR * MP.SCNTC;
          }
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/
	
			 /* calculate caregivers tax credit */
			 if (MP.SCGTCFLG && isnzero(in->id.idcgtc) && MP.SCGTCOPT < 3) {
				/* if SGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.SCGTCOPT == 1) {
					in->im.impcgtc = MP.SCGTC * MP.SPNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.SPNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }


             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.SPNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.SPNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.SPNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.SPNTCR * MP.SBXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc = in->im.imicnet;

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.SAXM, MP.SAXRR * nneg(netinc - MP.SAXTD));
                adjamt = MP.SAXM - agered;
                in->im.impatxc = MP.SPNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.SPNTCR * minn(MP.SYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.SMEDALL, MP.SMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.SPNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

			 /* calculate amount for infirm dependents age 18 or older */
			 /*  if the filer has an amount for idothpe, then give max credit */

			 if (isnzero(in->id.idothpe)) {
   				in->im.impothpe = MP.SPNTCR * MP.SODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
			 }

             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.SCHATL1, in->im.imchara)  * MP.SCHATR1
                            + nneg(in->im.imchara - MP.SCHATL1) * MP.SCHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.SPNTCR * MP.SMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.SPNTCR * MP.SMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.SPNTCR * (in->id.ideducfm * MP.SEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.SPNTCR * (in->id.ideducpm * MP.SEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.SPNTCR * (in->id.ideducpm * MP.SEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }

			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.SPNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }

             /* calculate federal dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.SDTCR)
								+ (in->im.imildivt * MP.SDTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }

			 /* calculate the Saskatchewan minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.SAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use SAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.SAMTPCTF;
				 }
				 else if (MP.SAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using SAMTTX. */
						in->im.impminco = in->id.idmincar * MP.SAMTTX;
					  }
				 else if (MP.SAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using SAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.SAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }

             /* calculate senior supplement */
             if (in->id.idage >= 65 ) {
                in->im.imssstxc = MP.SPNTCR * MP.SSSTC;
             }
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
            
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */
			 if (in->im.imamtfg && MP.SAMTOPT == 3) {
                in->im.imprvtax = MP.SAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.SPTX, MP.SPTXrows, in->im.imitax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
                        + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
                        + in->im.impminco    /* provincial minimum tax carryover credit */
						+ in->im.impintsl   /* provincial interest on student loans credit */
                        + in->im.imssstxc     /* supplemental age tax credit                  */
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.SPNTCR * MP.SMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.SPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.SPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                       + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco   /* provincial minimum tax carryover credit */
					   + in->im.impintsl   /* provincial interest on student loans credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
                    + in->im.imppentc   /* pension income tax credit             */
                    + in->im.imssstxc     /* supplemental age tax credit                  */
                    + in->im.imsctxc     /* child age tax credit                  */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.SPNTCR * MP.SMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.SPNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.SBXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic ));
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

    /* process each person in household */
    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
           ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

        /* original tax as percentage of federal tax */
        if (!MP.STXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.SPTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if STXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.SAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.SAMTPCTF;
			}
			else {
				if ( in->im.imamtfg && MP.SAMTOPT == 4 ) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.SAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}

        /* compute net income flat tax on adjusted net income */
		if (MP.TARGETYEAR == 1991) {
			adjnet = in->im.imicnet;
		}
		else { /* have to adjust net income line 236 less 250 */
			adjnet = nneg(in->im.imicnet - in->im.imisa - in->im.imispa
						 - in->im.imigis - in->id.idiworkc);
		}
        in->im.impnit = MP.SFTAX * adjnet;

      /* apply net income tax */
        in->im.imtxp += in->im.impnit;

        /* compute deficit surtax */
        in->im.impsur = nneg((MP.SDSF * in->im.imtxp) - MP.SDSRA);

        /* compute high income surtax surtax and add to deficit surtax */
        in->im.impsur += MP.SSF * nneg(in->im.imtxp - MP.SSCI);

        /* apply surtax */
        in->im.imtxp += in->im.impsur;

        /* compute reduction - add basic or sales tax reduction */
        in->im.imptr = MP.STRBR;

        /* compute reduction - add spousal or married equivalent tax reduction */
        if (isnzero(in->im.immartxc)) {
            DEBUG1("%s eligible for married or equivalent reduction\n");
            in->im.imptr += MP.SSTR;
        }

        /* compute reduction - add child tax reduction */
        in->im.imptr += minn(MP.STRCL, in->im.imnfach * MP.STRPC);

        /* compute reduction - add senior citizens tax reduction */
        if (in->id.idage >= 65) {
            in->im.imptr += MP.STRSC;
        }

        /* compute reduction based on net income - iminet less line 250 (adjnet) */
        in->im.imptr = nneg(in->im.imptr - MP.STRRR * nneg(adjnet - MP.STRBA ));
       
        /* ensure that reduction does not make tax negative */
        if (in->im.imptr > in->im.imtxp) {
            in->im.imptr = in->im.imtxp;
        }

        /* apply reduction */
        in->im.imtxp -= in->im.imptr;

        /* calculate and apply provincial foreign tax credit */
        in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
        in->im.imtxp -= in->im.imnptc;

        DEBUG3("%s SASK age=Z%d, chclm=%.2f\n", in->id.idage, in->im.imchclm);

        /* Non-refundable Saskatchewan Political Contribution Tax Credit */
        if (isnzero(in->id.idprvpol)) {
            poltcr = (NUMBER) lkup1(MP.SPTC, MP.SPTCrows, in->id.idprvpol);
            poltc = minn(MP.SPTCBEN, poltcr);
            in->im.impptc = minn(poltc, in->im.imtxp);
            DEBUG4("%s alt pol: idpolcon=%.2f poltcr=%.2f poltc=%.2f\n",
               in->id.idprvpol, poltcr, poltc);

            /* apply political contribution tax credit */
            in->im.imnptc += in->im.impptc;
            in->im.imtxp  -= in->im.impptc;

            DEBUG3("%s idpolcon =%.2f importc =%.2f\n",
               in->id.idprvpol, in->im.importc);
        }


        DEBUG4("%s SASK bpt=%.2f, txp=%.2f, ptc=%.2f\n",
            in->im.imbpt, in->im.imtxp, in->im.imptc);

        /* Sask labour-sponsored venture capital credit */
        if ( isnzero(in->id.idlabtxg) ) {
           lvctc = minn(MP.SLVCMAX, MP.SLVCRT * in->id.idlabtxg);
           in->im.implvctc = minn(lvctc, in->im.imtxp);
           in->im.imnptc += in->im.implvctc;
           in->im.imtxp  -= in->im.implvctc;
        }

		/* Saskatchewan Home Heating Assistance Rebate */

		/* Give the one time rebate to recipients of the federal home heating rebate */

		if (MP.SHEATFLG) {
			/* check if this individual is to receive the federal rebate */
			if (isnzero(in->im.imheatrl)) {
				/* if this amount is equal to the individual amount, 
				   give the individual provincial top-up. */
				if ( in->im.imheatrl == MP.HEATSNG ) {
					in->im.impheatr = MP.SHEATSNG ;
				}
				else { /* federal amount must then be equal to family credit */
					/* give the individual the provincial family rebate top-up. */
					in->im.impheatr = MP.SHEATFAM ;
				}
			}
		}

      }

    /**
    * Saskatchewan Child Benefit and Employment Supplement 
    *           AND
    * Saskatchewan refundable sales tax credit
    **/

   if (MP.SCBFLG==1 || MP.SSTCFLAG==1) {
        DEBUG1("%s child benefit program in effect\n");

      /* process each nuclear family in household */
       for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
            DEBUG4("%s cben: nf %d spoflg=%d, nkids=%d\n",
            nfi, nf->nfspoflg, nf->nfnkids);

         /* Initialize local variables and head and spouse pointers */
         ineld = nf->nfineld;
         inspo = nf->nfinspo;

         /* Sales tax credit */
         if (MP.SSTCFLAG == 1) {
            /* initialize local variables to zero */
            eldnet = ZERO;   /* Net income of ineld */
            sponet = ZERO;   /* Net income of inspo */
            fmnet = ZERO;    /* Family net income */
            highnet = ZERO;  /* Biggest net income in family */
            stceld = ZERO;   /* Sales tax credit of ineld */
            stcspo = ZERO;   /* Sales tax credit of spouse */
            stckid = ZERO;   /* Sales tax credit of kids */
            stckid2 = ZERO;   /* Sales tax credit of second child */
            famstc = ZERO;   /* Family Sales tax credit */
            famstc2 = ZERO;   /* Family Sales tax credit for July onwards */

            /* calculate family net income */
            eldnet = MP.PYINC * nneg (ineld->im.imicnet - ineld->im.imuccb );
            fmnet = eldnet;
            if (nf->nfspoflg) {
                sponet = MP.PYINC * nneg(inspo->im.imicnet - inspo->im.imuccb );
                fmnet += sponet ;
            }
            if (eldnet>=sponet) highnet=eldnet;
            else highnet=sponet;

            /* assign credit for adult with greatest income */
            /* note: since we are calculating the results at the family level, it doesn't
               matter if this is ineld or inspo  */
            /* credit is phased in on individual net income */
            stceld = minn(MP.SSTCBAS, MP.SSTCBASPIR * highnet);
            /* credit is phased out on family income */
            stceld = nneg(stceld - MP.SSTCRR * nneg(fmnet - MP.SSTCBASTD));
            /* add to family credit */
            famstc2 = stceld;

            if (nf->nfspoflg) {
                /* assign credit for spouse */

                /* credit is not phased in for the second adult */
                stcspo = (MP.SSTCSP);
                /* credit is phased out on family income */
                stcspo = nneg(stcspo - MP.SSTCRR * nneg(fmnet - MP.SSTCSPTD));
                /* add to family credit */
                famstc2 += stcspo;
                
                /* assign credit for kids */

                /* if one child */
                if (nf->nfnkids == 1) {
                    stckid = MP.SSTCKID;
                    /* credit is phased out on family income */
                    stckid = nneg(stckid - MP.SSTCRR * nneg(fmnet - MP.SSTC1KIDTD));
                    /* add to family credit */
                    famstc2 += stckid;
                }
                /* if more than one child */
                else if (nf->nfnkids > 1) {
                    /* can only claim for 2 kids */
                    stckid = MP.SSTCKID + MP.SSTCKID;
                    /* credit is phased out on family income */
                    stckid = nneg(stckid - MP.SSTCRR * nneg(fmnet - MP.SSTC2KIDTD));
                    /* add to family credit */
                    famstc2 += stckid;
                }
            }
            else {
                /* no spouse */

                /* assign credit for kids */

                /* if there are children, the first gets the equivalent to married amount
                   and the second gets the child amount. */
                if (nf->nfnkids == 1) {
                    stckid = MP.SSTCSP;
                    /* credit is phased out on family income */
                    stckid = nneg(stckid - MP.SSTCRR * nneg(fmnet - MP.SSTCSPTD));
                    /* add to family credit */
                    famstc2 += stckid;
                }
                /* if more than one child */
                else if (nf->nfnkids > 1) {
                    /* first kid gets spousal amount */
                    stckid = MP.SSTCSP;
                    /* credit is phased out on family income */
                    stckid = nneg(stckid - MP.SSTCRR * nneg(fmnet - MP.SSTCSPTD));
                    /* add to family credit */
                    famstc2 += stckid;
                    /* second kid gets child amount */
                    stckid2 = MP.SSTCKID;
                    /* credit is phased out on family income */
                    stckid2 = nneg(stckid2 - MP.SSTCRR * nneg(fmnet - MP.SSTC1KIDTD));
                    /* add to family credit */
                    famstc2 += stckid2;
                }
            }

			if (MP.GSTCYFLG) {

	            stceldp = ZERO;   /* Sales tax credit of ineld */
	            stcspop = ZERO;   /* Sales tax credit of spouse */
	            stckidp = ZERO;   /* Sales tax credit of kids */
	            stckid2p = ZERO;   /* Sales tax credit of second child */
	            famstcp = ZERO;   /* Family Sales tax credit */
	            /* calculate family net income for the previous 2 years */
				/*  test without adjusting income 
				fmnet = (fmnet/MP.PYINC) * MP.PYINCP;
				highnet = (highnet/MP.PYINC) * MP.PYINCP;
				sponet = (sponet/MP.PYINC) * MP.PYINCP;
				eldnet = (eldnet/MP.PYINC) * MP.PYINCP;	  */

	            /* assign credit for adult with greatest income */
	            /* note: since we are calculating the results at the family level, it doesn't
	               matter if this is ineld or inspo  */
	            /* credit is phased in on individual net income */
	            stceldp = minn(MP.SSTCBASP, MP.SSTCBASPIRP * highnet);
	            /* credit is phased out on family income */
	            stceldp = nneg(stceldp - MP.SSTCRRP * nneg(fmnet - MP.SSTCBASTDP));
	            /* add to family credit */
	            famstcp = stceldp;

	            if (nf->nfspoflg) {
	                /* assign credit for spouse */

	                /* credit is not phased in for the second adult */
	                stcspop = (MP.SSTCSPP);
	                /* credit is phased out on family income */
	                stcspop = nneg(stcspop - MP.SSTCRRP * nneg(fmnet - MP.SSTCSPTDP));
	                /* add to family credit */
	                famstcp += stcspop;
	                
	                /* assign credit for kids */

	                /* if one child */
	                if (nf->nfnkids == 1) {
	                    stckidp = MP.SSTCKIDP;
	                    /* credit is phased out on family income */
	                    stckidp = nneg(stckidp - MP.SSTCRRP * nneg(fmnet - MP.SSTC1KIDTDP));
	                    /* add to family credit */
	                    famstcp += stckidp;
	                }
	                /* if more than one child */
	                else if (nf->nfnkids > 1) {
	                    /* can only claim for 2 kids */
	                    stckidp = MP.SSTCKIDP + MP.SSTCKIDP;
	                    /* credit is phased out on family income */
	                    stckidp = nneg(stckidp - MP.SSTCRRP * nneg(fmnet - MP.SSTC2KIDTDP));
	                    /* add to family credit */
	                    famstcp += stckidp;
	                }
	            }
	            else {
	                /* no spouse */

	                /* assign credit for kids */

	                /* if there are children, the first gets the equivalent to married amount
	                   and the second gets the child amount. */
	                if (nf->nfnkids == 1) {
	                    stckidp = MP.SSTCSPP;
	                    /* credit is phased out on family income */
	                    stckidp = nneg(stckidp - MP.SSTCRRP * nneg(fmnet - MP.SSTCSPTDP));
	                    /* add to family credit */
	                    famstcp += stckidp;
	                }
	                /* if more than one child */
	                else if (nf->nfnkids > 1) {
	                    /* first kid gets spousal amount */
	                    stckidp = MP.SSTCSPP;
	                    /* credit is phased out on family income */
	                    stckidp = nneg(stckidp - MP.SSTCRRP * nneg(fmnet - MP.SSTCSPTDP));
	                    /* add to family credit */
	                    famstcp += stckidp;
	                    /* second kid gets child amount */
	                    stckid2p = MP.SSTCKIDP;
	                    /* credit is phased out on family income */
	                    stckid2p = nneg(stckid2p - MP.SSTCRRP * nneg(fmnet - MP.SSTC1KIDTDP));
	                    /* add to family credit */
	                    famstcp += stckid2p;
	                }
	            }

				famstc = ((famstc2/2) * MP.SSTCPI2P) + ((famstcp/2)* MP.SSTCPI1P);

			}
			else {  /* use the July to June amounts for year */
				famstc = famstc2 * MP.SSTCPI;
			}

            /* assign the credit to the spouse with the highest net income */
            if (nf->nfspoflg) {
                if (eldnet>=sponet) {
                    ineld->im.imsstc = famstc ;
                    ineld->im.imptc += famstc ;
                }
                else {
                    inspo->im.imsstc = famstc ;
                    inspo->im.imptc += famstc ;
                }
            }
            else {
                ineld->im.imsstc = famstc ;
                ineld->im.imptc += famstc ;
            }
         }

         /* Child tax benefit */
         if (MP.SCBFLG == 1) {

             if (nf->nfnkids == 0) {
                   DEBUG1("%s cben: skipping nuclear family with no `children'\n");
                   continue;
             }


             /* net income for child benefit matches federal definition */
             nfinet = MP.PYINC * nneg( ineld->im.imicnet - ineld->im.imuccb );


             if (nf->nfspoflg) {
                 nfinet += MP.PYINC * nneg(inspo->im.imicnet - inspo->im.imuccb );
             }

             /* Scale income with user specified rates */
             nfinet = nfinet * MP.CTCINC;

                DEBUG3("%s cben: neti=%.2f empi=%.2f\n", nfinet, nfiemp);

             /***
             * Calculate Saskatchewan Child Benefit
             ***/

             /* initialize base benefits*/
             cben2 = ZERO;
        
             /* Assign base amount which is dependent on number of kids */
             cben2 =  ((MP.SCB1) + (MP.SCB2 * minn(nneg(nf->nfnkids-1),1)) +
                                 (MP.SCB3 * nneg(nf->nfnkids-2))) ;

			 /* Announced in the Saskatchewan 2004 Budget, an extra amount for
			    single parents beginning in July 2004 */

			 if (nf->nfspoflg == 0) {
				cben2 += MP.SCBSPA;
			 }

             /* Tax Back basic benefits 
                Note that benefits are taxed back in two stages.  First, the benefit 
                is taxed back between SCBTD1 and SCBTD2 at a rate of SCBRR1 (which depends
                on the number of children).  The benefit is then taxed back at a different
                rate for any income over SCBTD2. */
             /* no phase in since the program starts in July, and Federal CTB are given from July to June */
             cben2 = nneg(nneg(cben2 - MP.SCBRR1[nf->nfnkids] * nneg(minn(nfinet,MP.SCBTD2)-MP.SCBTD1))-
                          MP.SCBRR2[nf->nfnkids] * nneg(nfinet - MP.SCBTD2));

             DEBUG4("%s cben: cben after reduction cben=%.2f neti=%.2f empi=%.2f\n",
                        cben2, nfinet, nfiemp);

			 /* Calculate the child benefit for the calendar year if FCBENCAL is on */
			 if (MP.FCBENCAL) {
	          
	             /* initialize base benefits*/
	             cben1 = ZERO;

				 /* Adjust net income to represent value for 2 years prior */
				 nfinet = ( nfinet / MP.PYINC ) * MP.PYINCP; 
	        
	             /* Assign base amount which is dependent on number of kids */
	             cben1 =  ((MP.SCB1P) + (MP.SCB2P * minn(nneg(nf->nfnkids-1),1)) +
	                                 (MP.SCB3P * nneg(nf->nfnkids-2))) ;

				 /* Announced in the Saskatchewan 2004 Budget, an extra amount for
				    single parents beginning in July 2004 */

				 if (nf->nfspoflg == 0) {
					cben1 += MP.SCBSPAP;
				 }

	             /* Tax Back basic benefits 
	                Note that benefits are taxed back in two stages.  First, the benefit 
	                is taxed back between SCBTD1P and SCBTD2P at a rate of SCBRR1P (which depends
	                on the number of children).  The benefit is then taxed back at a different
	                rate for any income over SCBTD2P. */
	             /* no phase in since the program starts in July, and Federal CTB are given from July to June */
	             cben1 = nneg(nneg(cben1 - MP.SCBRR1P[nf->nfnkids] * nneg(minn(nfinet,MP.SCBTD2P)-MP.SCBTD1P))-
	                          MP.SCBRR2P[nf->nfnkids] * nneg(nfinet - MP.SCBTD2P));

	             DEBUG4("%s cben: cben after reduction cben=%.2f neti=%.2f empi=%.2f\n",
	                        cben1, nfinet, nfiemp);

				 /* for calendar year amounts use half of each calculation */
				 cben = (cben1/2) + (cben2/2);
			 }
			 else {  /* use the July to June amounts for the entire year */
			 	 cben = cben2;
			 }	

		  
             /* assign child benefits to mother if present */

             if (nf->nfspoflg && (inspo->id.idsex == FEMALE)) {
                DEBUG1("%s cben: spouse is the mother, gets CTC\n");
                inspo->im.imscb  = cben;

              }
              else {
                 DEBUG1("%s cben: elder receives CTC\n");
                 ineld->im.imscb  = cben;

              }
		  }
       }
    }

    DEBUG_OFF("txsask");
}
